import os
import signal
from PySide2.QtCore import Qt, QRect, QMetaObject
from PySide2.QtGui import QIcon, QPixmap
from PySide2.QtWidgets import (
    QAction,
    QFileDialog,
    QDialog,
    QMenuBar,
    QMenu,
    QLineEdit,
    QToolBar,
    QStatusBar,
    QMainWindow,
    QGridLayout,
    QLabel,
    QVBoxLayout,
    QDialogButtonBox,
    QPushButton
)


class Ui_ViewerMainWindow:

    def setup_ui(self, MainWindow):
        MainWindow.resize(800, 600)
        MainWindow.setWindowTitle("General Viewer")

        self.statusbar = QStatusBar(MainWindow)
        MainWindow.setStatusBar(self.statusbar)

        self.menubar = QMenuBar(MainWindow)
        self.menubar.setGeometry(QRect(0, 0, 800, 22))
        MainWindow.setMenuBar(self.menubar)

        self.menuFile = QMenu(self.menubar)
        self.menuFile.setTitle("&File")
        self.menubar.addAction(self.menuFile.menuAction())

        self.actionLoad = QAction(MainWindow)
        self.actionLoad.setText("Load xml file")
        self.actionLoad.setShortcut("Ctrl+L")
        self.actionLoad.triggered.connect(self.open_file)
        self.menuFile.addAction(self.actionLoad)

        self.topToolBar = QToolBar(MainWindow)
        self.topToolBar.setMovable(False)

        MainWindow.addToolBar(self.topToolBar)

        self.bottomToolBar = QToolBar(MainWindow)
        self.bottomToolBar.setMovable(False)
        MainWindow.addToolBar(
            Qt.ToolBarArea.BottomToolBarArea, self.bottomToolBar)

        QMetaObject.connectSlotsByName(MainWindow)


    def setup_info(self, infos):
        self.iconInfo = QIcon.fromTheme('dialog-information')
        self.actionInfo = QAction(self.iconInfo, "info", None)
        self.topToolBar.addAction(self.actionInfo)
        self.actionInfo.triggered.connect(lambda: self.show_info(infos))

    def show_info(self, infos):
        self.dialog = QDialog()
        self.popup = Ui_Info(self.dialog)
        self.popup.setup_ui(self, infos)

        self.dialog.show()

    def open_file(self, extension='xml'):
        file_dialog = QFileDialog
        if extension == 'xml':
            fname = file_dialog.getOpenFileName(
                self.menubar, "Open File", "", "XML Files (*.xml)")
            return fname
        elif extension == 'txml' or extension == 'tfpg':
            fname = file_dialog.getOpenFileName(
                self.menubar, "Open File", "", "TFPG, TXML Files (*.tfpg, *.txml)")
            return fname
        else:
            raise Exception('Invalid extension')





class Ui_Info(object):

    def __init__(self, Dialog):
        self.Dialog = Dialog

    def setup_ui(self, Dialog, infos):
        self.Dialog.resize(350, 300)
        self.Dialog.setWindowTitle("Info")

        self.infos = infos

        self.gridLayout = QGridLayout()
        for i in range(len(self.infos)):
            self.image = QLabel(self.Dialog)
            self.pixmap = QPixmap(list(self.infos)[i])
            self.image.setPixmap(self.pixmap)
            self.description = QLabel(self.Dialog)
            self.description.setText(list(self.infos.values())[i])
            self.gridLayout.addWidget(self.image, i, 0, 1, 1)
            self.gridLayout.addWidget(self.description, i, 1, 1, 1)

        self.buttonBox = QDialogButtonBox(self.Dialog)
        self.buttonBox.setOrientation(Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.StandardButton.Ok)
        self.buttonBox.accepted.connect(self.accept)

        self.verticalLayout = QVBoxLayout(self.Dialog)
        self.verticalLayout.addLayout(self.gridLayout)
        self.verticalLayout.addWidget(self.buttonBox)

        QMetaObject.connectSlotsByName(self.Dialog)

    def accept(self):
        self.Dialog.close()


class ViewerMainWindow(QMainWindow):
    def __init__(self):
        super().__init__()

    def quit_terminal_process(self):
        # Get the process group ID of the current process
        pgid = os.getpid()

        # Send a SIGTERM signal to the process group to terminate it
        os.kill(pgid, signal.SIGTERM)

    def closeEvent(self, event):
        self.quit_terminal_process()
