import sys
import pathlib
sys.path.append(str(pathlib.Path(__file__).parents[2]))
from fault_tree_viewer.FtvNode import TreeNode, Event, Gate
import csv
import re
import xml.etree.ElementTree as ET
class FtvModel(object):

    e_file = ""
    g_file = ""
    xml_file = ""

    gate_fields = [
        "Name",
        "Type",
        "Description",
        "Input  \d+ Type and Name",
        "Page Flag",
        "Notes1", ]
    event_fields = [
        "Name",
        "Description",
        "Failure Model Type",
        "Failure Model Parameter 0", ]

    def __init__(self):
        self.ls_tree = []
        self.liststore = []
        return
    def create_model(self, arg):
        self.e_file, self.g_file, self.xml_file = arg

                # read file and create tree
        self.ls_tree = self._create_tree()
        # add liststore
        for i in self.ls_tree:
            if i.info.name.startswith("Top_Level_Event_"):
                self.liststore.append([i.info.name])
            elif i.info.description is not None and i.info.description != "":
                self.liststore.append([i.info.description])
            else:
                self.liststore.append([i.info.name])
        # calc size
        for i in treewalker(self.ls_tree[0], self._calc_size,
                            order=TW_POSTORDER):
            pass

    def _calc_size(self, node):
        """
        calculate size of sub-tree;
        must be bottom-up
        """
        if node.info.isGate():
            cdepth = 0
            for i in node.children:
                node.size += i.size
                if i.info.isGate() and i.info.page == "On":
                    node.size_p += 1
                else:
                    node.size_p += i.size_p
                cdepth = max(cdepth, i.depth)

            node.depth = cdepth + 1
        return

    def _parse(self, file_name, _node_type):
        """
        import file into a matrix[][]
        and create node dictionary
        or leaf dictionary
        """
        # read file start
        with open(file_name, "r") as csvfile:
            reader = csv.reader(csvfile, delimiter='\t')
            fields = next(reader)
            matrix = [row for row in reader]

        # map{field:idx}
        field_pos = dict([(b, a) for a, b in enumerate(fields)])

        req_fields = (self.gate_fields
                      if _node_type == Gate
                      else self.event_fields)

        # expand possible regex in the list
        exp_fields = []
        for name in req_fields:
            if name in field_pos:
                exp_fields.append(name)
            else:
                found = False
                for name2 in fields:
                    if re.match(name, name2):
                        # expand if any match is found
                        exp_fields.append(name2)
                        found = True

                if not found:
                    # append as it is if not found
                    exp_fields.append(name)

        res = [] # ordered list of (name, gate/event object)
        dict_D = dict() # map(name, gate/event obj)
        names_list = [] # ordered list of names

        for n in range(len(matrix)):
            ls = []
            for name in exp_fields:
                if name in field_pos:
                    pos = field_pos[name]
                    ls.append(matrix[n][pos])
                else:
                    ls.append("") # add empty field
            if ls[0] not in dict_D:
                names_list.append(ls[0])
            dict_D[ls[0]] = _node_type(ls)

        for name in names_list:
            res.append((name, dict_D[name]))

        return res
    def _parse_xml(self, file_name):
        """
        import file into a matrix[][]
        and create node dictionary
        or leaf dictionary
        """
        gates_list = []
        events_list = []

        tree = ET.parse(file_name)
        root = tree.getroot()

        # get the list of all gates
        gates = root.iter('gate')
        # get the list of all events
        events = root.iter('event')

        # dict associating gate names to ids
        # used to know how many gates with the same name are
        gt_names = dict()
        ev_names = set()
        gt_el2name = dict()

        # first loop in gates; it is used to associate
        # each gate object to a different name
        # this is useful in case the same name is assigned
        # to multiple gates
        for gate in gates:
            gt_name = gate.attrib['name']

            if gt_name in gt_names:
                idx = gt_names[gt_name]
                gt_name += "_%03d" % idx
                gt_names[gate.attrib['name']] = idx + 1

            else:
                gt_name += "_000"
                gt_names[gate.attrib['name']] = 1

            gt_el2name[gate] = gt_name

        # second loop in gates
        # for each gate object, we get the name we just created
        gates = root.iter('gate')
        for gate in gates:

            gt_name = gt_el2name[gate]
            gt_vals = [gt_name,
                       gate.attrib['type'],
                       gate.attrib['description'] if gate.attrib['description'] else gate.attrib['name']]

            for ch in gate:
                if ch.tag == 'gate':
                    type_ = "G"
                    gt_vals.append(type_ + ":" + gt_el2name[ch])
                else:
                    type_ = "E"
                    gt_vals.append(type_ + ":" + ch.attrib['name'])

            gt_vals.append(gate.attrib['page'])
            if 'probability' in gate.attrib:
                gt_vals.append(gate.attrib['probability'])
            else:
                gt_vals.append("")

            gt = Gate(gt_vals)
            gates_list.append((gt_name, gt))

        for event in events:

            ev_name = event.attrib['name']
            if ev_name in ev_names:
                continue
            ev_names.add(ev_name)
            ev_vals = [ev_name,
                       event.attrib['description']]
            if 'probability' in event.attrib:
                ev_prob = event.attrib['probability']
                if re.match("^0\.0+$", ev_prob):
                    ev_vals += ['', '0.0']
                else:
                    ev_vals += ['Fixed', ev_prob]
            else:
                ev_vals += ['', '']
            ev = Event(ev_vals)
            events_list.append((ev_name, ev))

        return events_list, gates_list
    def _create_tree(self):
        """
        @return root node of a tree
        """

        if self.e_file and self.g_file:
            # @return: list[(name, obj_event), ]
            e_list = self._parse(self.e_file, Event)
            # @return: list[(name, obj_gate), ]
            g_list = self._parse(self.g_file, Gate)
        else:
            assert self.xml_file
            e_list, g_list = self._parse_xml(self.xml_file)

        # ID = (flag , name)
        # flag = G for Gate OR E for Event
        # name = name of Gate or Event
        dict_D = dict()  # map(ID -> obj_treenode)
        tree = []  # list of tree root node (page)

        # add events in dict_D
        for e_k, e_v in e_list:
            dict_D[("E", e_k)] = TreeNode(e_v, [])

        # add gates in dict_D and add page_node
        for g_k, g_v in g_list:
            dict_D[("G", g_k)] = TreeNode(g_v, list(g_v.children), size=0, size_p=0)
            if g_v.page == "On":
                tree.append(dict_D[("G", g_k)])

        # create tree and find root node
        set_node = set(el[0] for el in g_list)

        while set_node:
            root = set_node.pop()
            set_node -= self._tree_builder(dict_D[("G", root)], dict_D)

        root = dict_D[("G", root)]
        tree.insert(0, root)

        return tree
    def _tree_builder(self, treenode, dict_D):
        """
        convert iteratively Treenode.children[ID] to
        Treenode.children[Treenode]
        and set the parent Treenode in Treenode.parent
        @return set(ID) of converted node
        """

        res = set()
        stack = ([treenode] if treenode.parent is None
                 else [])

        while stack:
            cur_treenode = stack.pop()

            for i in range(len(cur_treenode.children)):
                child_id = cur_treenode.children[i]

                if isinstance(child_id, tuple):
                    child_treenode = dict_D[child_id]

                    if isinstance(child_treenode.info, Event):  # leaf
                        if child_treenode.parent is None:
                            cur_treenode.children[i] = child_treenode
                            child_treenode.parent = cur_treenode

                        else:  # duplicate leaf! create new TreeNode
                            cur_treenode.children[i] = \
                                TreeNode(child_treenode.info, [],
                                         cur_treenode)

                    else:  # gate
                        f, n = child_id
                        res.add(n)

                        if child_treenode.parent is None:
                            stack.append(child_treenode)

                        cur_treenode.children[i] = child_treenode
                        child_treenode.parent = cur_treenode

        return res
#treewalker

(TW_PREORDER,
 TW_POSTORDER, ) = range(2)


def treewalker(tree, foo, arg=None, order=TW_PREORDER):
    """
    visit tree:
    - in preorder when foo() return exception "StopIteration"
      the children of node are not visited
    - in postorder this is not possible because childern
      are visited before; exceptions are ignored
    """

    # Warning: recursive versions are deprecated, substituted by
    # iterative when fixing issue #5223

    def preorder_recur(t, foo):
        try:
            out = foo(t)
        except StopIteration as ex:
            out = ex[0] if ex.args else None
            yield (t, out)
        else:
            yield (t, out)
            for child in t.children:
                for m, o in preorder_recur(child, foo):
                    yield (m, o)
    # --------------------------------------------------

    def preorder_iter(t, foo):
        stack = [t]

        while stack:
            curr = stack.pop()
            try:
                out = foo(curr)
            except StopIteration as ex:
                out = ex[0] if ex.args else None
                yield (curr, out)
            else:
                yield (curr, out)
                stack.extend(reversed(curr.children))
    # --------------------------------------------------

    def postorder_recur(t, foo):
        for child in t.children:
            for m, o in postorder_recur(child, foo):
                yield (m, o)

        try:
            out = foo(t)
        except StopIteration as ex:
            out = ex[0] if ex.args else None

        yield (t, out)
    # --------------------------------------------------

    def postorder_iter(t, foo):
        stack = [t]
        prev = None
        while len(stack):
            curr = stack[-1]
            if not curr.children or prev == curr.children[-1]:
                # leave or done with children
                try:
                    out = foo(curr)
                except StopIteration as ex:
                    out = ex[0] if ex.args else None
                stack.pop()
                yield (curr, out)

            else:
                stack.extend(reversed(curr.children))

            prev = curr
    # --------------------------------------------------

    _foo = (foo if arg is None
            else lambda arg0: foo(arg0, arg))

    _treewalker = (preorder_iter if TW_PREORDER == order
                   else postorder_iter)

    return _treewalker(tree, _foo)
