from PySide2.QtCore import Qt, QRectF, QLineF
from PySide2.QtGui import QColor, QPainterPath, QFont, QFontMetrics
from PySide2.QtWidgets import QGraphicsItem


def calculate_font_size(text, space):
    font = QFont("Helvetica", 10)
    font_metrics = QFontMetrics(font)
    text_width = font_metrics.horizontalAdvance(text)
    text_height = font_metrics.height()
    space_width, space_height = space.width(), space.height()

    font_size = 10
    while text_width < space_width and text_height < space_height:
        font_size += 1
        font.setPointSize(font_size)
        font_metrics = QFontMetrics(font)
        text_width = font_metrics.horizontalAdvance(text)
        text_height = font_metrics.height()

    font_size -= 1
    font.setPointSize(font_size-1)
    return font




class GateItem(QGraphicsItem):

    def __init__(self, _type, text1, text2, page, root=False, x=0, y=0):
        super().__init__()
        self._type = _type
        self.text1 = text1
        self.text2 = text2
        self.page = page
        self.root = root
        self.size = 120
        self.x = x
        self.y = y

    def get_x(self):
        return self.x

    def get_y(self):
        return self.y

    def boundingRect(self):
        return self.shape().boundingRect()

    def shape(self):
        path = QPainterPath()

        if self.page == 'On':
            path = QPainterPath()
            path.moveTo(self.size/2, -(self.size/2))
            path.lineTo(self.size, self.size)
            path.moveTo(self.size/2, -(self.size/2))
            path.lineTo(0, self.size)
            path.lineTo(self.size, self.size)

        elif self._type in ['AND', 'PAND']:
            path.lineTo(0, self.size)
            path.lineTo(self.size, self.size)
            path.lineTo(self.size, 0)
            path.arcTo(0, -(self.size/2), self.size, self.size, 0, 180)

            if self._type == 'PAND':
                path.moveTo(self.size/2, -(self.size/2))
                path.lineTo(self.size, self.size)
                path.moveTo(self.size/2, -(self.size/2))
                path.lineTo(0, self.size)

        elif self._type == 'OR':
            path.lineTo(0, self.size)
            path.moveTo(self.size, self.size)
            path.lineTo(self.size, 0)
            path.arcTo(0, -(self.size/2), self.size, self.size, 0, 180)
            path.moveTo(self.size, self.size)
            path.arcTo(0, self.size/2, self.size, self.size, 0, 180)

        path.moveTo(0, 0)
        return path

    def paint(self, painter, option, widget=None):
        if not self.root:
            self.line = QLineF(60, -190, 60, -140)
            painter.drawLine(self.line)

        self.bigBox = QRectF(-40, -140, 200, 60)
        painter.fillRect(self.bigBox, QColor('white'))
        painter.drawRect(self.bigBox)
        font = calculate_font_size(self.text1, self.bigBox)
        painter.setFont(font)

        painter.drawText(self.bigBox, Qt.AlignmentFlag.AlignCenter | Qt.TextFlag.TextWrapAnywhere, self.text1)
        self.line = QLineF(60, -80, 60, 130)
        painter.drawLine(self.line)
        painter.fillPath(self.shape(), QColor('white'))
        painter.drawPath(self.shape())
        if self._type == 'OR':
            self.line = QLineF(60, 60, 60, 130)
            painter.drawLine(self.line)

        self.rect = QRectF(-40, 40, 200, 30)
        painter.fillRect(self.rect, QColor('white'))
        painter.drawRect(self.rect)

        font = calculate_font_size(self.text2, self.rect)
        painter.setFont(font)
        painter.drawText(self.rect, Qt.AlignmentFlag.AlignCenter | Qt.TextFlag.TextWrapAnywhere, self.text2)


class HorizontalLine(QGraphicsItem):
    def __init__(self, x0, y0, x1, y1, parent=None):
        super().__init__(parent)
        self.x0 = x0
        self.x1 = x1
        self.y0 = y0
        self.y1 = y1

    def boundingRect(self):
        return self.shape().boundingRect()

    def shape(self):
        path = QPainterPath()
        path.moveTo(self.x0, self.y0)
        path.lineTo(self.x1, self.y1)

        return path

    def paint(self, painter, option, widget=None):
        painter.setBrush(QColor(255, 255, 255))
        painter.drawPath(self.shape())


class EventItem(QGraphicsItem):
    def __init__(self, text1, failure_model_parameter, description, parent=None, x=0, y=0):
        super().__init__(parent)
        self._name = text1
        self._failure_model_parameter = failure_model_parameter
        self._description = description
        self.x = x
        self.y = y

    @property
    def text1(self):
        return self._name

    @property
    def failure_model_parameter(self):
        return self._failure_model_parameter

    @property
    def description(self):
        return self._description

    def get_x(self):
        return self.x

    def get_y(self):
        return self.y

    def boundingRect(self):
        return self.shape().boundingRect()

    def shape(self):
        path = QPainterPath()
        path.moveTo(120, 60)
        path.arcTo(0, 0, 120, 120, 0, 360)

        return path

    def paint(self, painter, option, widget=None):

        # Draw a vertical line
        line1 = QLineF(60, -190, 60, -80)
        painter.drawLine(line1)

        # Draw a big box with centered text
        big_box = QRectF(-40, -80, 200, 60)
        font = calculate_font_size(self._description, big_box)
        painter.setFont(font)
        painter.fillRect(big_box, QColor('white'))
        painter.drawRect(big_box)
        painter.drawText(
            big_box, Qt.AlignmentFlag.AlignCenter | Qt.TextFlag.TextWrapAnywhere, self._description)

        # Draw a horizontal line and a custom shape
        line2 = QLineF(60, -20, 60, 0)
        painter.drawLine(line2)
        painter.fillPath(self.shape(), QColor('white'))
        painter.drawPath(self.shape())

        # Draw a small box with centered text
        small_box = QRectF(-40, 40, 200, 30)
        font = calculate_font_size(self._name, small_box)
        painter.setFont(font)
        painter.fillRect(small_box, QColor('white'))
        painter.drawRect(small_box)
        painter.drawText(small_box, Qt.AlignmentFlag.AlignCenter | Qt.TextFlag.TextWrapAnywhere, self._name)

        # Draw text at a specific location
        painter.drawText(40, 140, self._failure_model_parameter)
