import sys

from PySide2.QtSvg import QSvgGenerator
from PySide2.QtCore import QRectF, QSize, QRect, Qt, QMetaObject
from PySide2.QtGui import QIcon, QPainter, QImage, QColor
from PySide2.QtWidgets import (
    QAction,
    QSizePolicy,
    QDialog,
    QFileDialog,
    QPushButton,
    QDialogButtonBox,
    QWidget,
    QApplication,
    QGraphicsItemGroup,
    QGraphicsView,
    QGraphicsScene,
    QHBoxLayout,
    QVBoxLayout,
    QGridLayout,
    QHeaderView,
    QTableWidget,
    QTableWidgetItem,
    QLineEdit,
    QLabel
)
from PIL import Image
from fault_tree_viewer.ftvtreewalker import treewalker
from fault_tree_viewer.FtvModel import FtvModel
from fault_tree_viewer.custom_items import GateItem, HorizontalLine, EventItem
from Viewer.viewer import Ui_ViewerMainWindow, ViewerMainWindow
(TW_PREORDER, TW_POSTORDER) = range(2)


class TreeWidget(QGraphicsView):

    def __init__(self, parent=None):
        super().__init__(parent)

        self.setScene(QGraphicsScene(self))
        self.setRenderHint(QPainter.RenderHint.SmoothPixmapTransform)
        self.setDragMode(QGraphicsView.DragMode.ScrollHandDrag)
        self.setInteractive(True)
    def mouseReleaseEvent(self, event):
        self.scene().update()




class Ui_FaultTreeViewerMainWindow(Ui_ViewerMainWindow):

    def setup_ui(self, MainWindow, model):
        super().setup_ui(MainWindow)

        self.MainWindow = MainWindow
        self.menubar.setVisible(False)
        self.model = model
        self.liststore = [[]]

        MainWindow.setWindowTitle("Fault Tree Viewer")

        central_widget = QWidget()
        MainWindow.setCentralWidget(central_widget)

        self.m_w11 = QWidget()
        self.m_w12 = QWidget()
        self.lay = QGridLayout(central_widget)

        for w, (r, c) in zip(
            (self.m_w11, self.m_w12),
            ((0, 0), (0, 1)),
        ):
            self.lay.addWidget(w, r, c)
        for c in range(1):
            self.lay.setColumnStretch(c, 1)
        for r in range(1):
            self.lay.setRowStretch(r, 1)

        self.treeWidget = TreeWidget()

        self.treeWidget.setSceneRect(-100, -200, 8000, 8000)

        self.originalTransform = self.treeWidget.transform()
        self.horizontalLayout = QHBoxLayout(self.treeWidget)
        self.verticalLayout = QVBoxLayout()
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.treeWidget.scale(0.5, 0.5)
        lay = QVBoxLayout(self.m_w11)
        lay.addWidget(self.treeWidget)
        self.table = QTableWidget()
        lay = QVBoxLayout(self.m_w12)
        lay.addWidget(self.table)
        self.table.setVisible(False)
        self.setup_toolbar()

    def update_ui(self, model):
        self.lineEdits[0].setText(model.xml_file)
        self.lineEdits[1].setText(model.e_file)
        self.lineEdits[2].setText(model.g_file)

    def update_nodes_table(self):
        self.table.setVisible(True)
        self.table.setRowCount(len(self.liststore)),
        self.table.setColumnCount(1)
        self.table.verticalHeader().hide()
        self.table.setHorizontalHeaderLabels(["node"])
        self.table.horizontalHeader().setSectionResizeMode(QHeaderView.ResizeMode.Stretch)
        for i in range(len(self.liststore)):
            cell = QTableWidgetItem(self.liststore[i][0].text1)
            self.table.setItem(i, 0, cell)
        self.table.setEditTriggers(QTableWidget.EditTrigger.NoEditTriggers)
        self.table.cellClicked.connect(self.select_node_at)

    def select_node_at(self, row):

        for i in range(len(self.liststore)):
            for item in self.liststore[i]:
                if i != row:
                    item.setVisible(False)
                else:
                    item.setVisible(True)
        self.treeWidget.scene().update()

    def setup_toolbar(self):

        line_edit_labels = ["&XML file:", "&Gates file:", "&Events file:"]
        self.lineEdits = []
        for text in line_edit_labels:
            line_edit = QLineEdit(self.treeWidget)
            line_edit.setPlaceholderText("None")
            line_edit.setReadOnly(True)
            label = QLabel(text, self.treeWidget)
            label.setBuddy(line_edit)
            self.bottomToolBar.addWidget(label)
            self.bottomToolBar.addWidget(line_edit)
            self.lineEdits.append(line_edit)
        # Define the toolbar actions
        actions = [
            {'icon': QIcon.fromTheme('edit-clear'),
             'text': 'cancel', 'triggered': self.cancel},
            {'icon': QIcon.fromTheme('zoom-in'),
             'text': 'zoom-in', 'triggered': self.zoomIn},
            {'icon': QIcon.fromTheme('zoom-out'),
             'text': 'zoom-out', 'triggered': self.zoomOut},
            {'icon': QIcon.fromTheme('zoom-original'),
             'text': 'zoom-reset', 'triggered': self.zoomReset},
            {'icon': QIcon.fromTheme('document-save'), 'text': 'export selected node',
             'triggered': self.handleExportPressed},
             {'icon': QIcon.fromTheme('document-save-as'), 'text': 'export all nodes',
              'triggered': lambda: self.handleExportPressed(True)}
        ]
        self.extensions = {'Load XML': 'xml',
                           'Load Gates': 'txt', 'Load Events': 'txt'}
        self.setup_load(self.extensions)
        # Create the toolbar actions and add them to the toolbar
        for action_info in actions:
            action = QAction(
                action_info['icon'], action_info['text'], self.treeWidget)
            if action_info['triggered']:
                action.triggered.connect(action_info['triggered'])
            self.topToolBar.addAction(action)

        spacer = QWidget()
        spacer.setSizePolicy(QSizePolicy.Policy.Expanding,
                             QSizePolicy.Policy.Expanding)
        self.topToolBar.addWidget(spacer)
        self.setup_info()

    def setup_load(self, extensions):
        self.iconLoad = QIcon.fromTheme('folder')
        self.actionLoad = QAction(self.iconLoad, "load", None)
        self.topToolBar.addAction(self.actionLoad)
        self.actionLoad.triggered.connect(lambda: self.show_load(extensions))

    def show_load(self, extensions):
        self.dialog = QDialog()
        self.popup = Ui_Upload(self.dialog)
        self.popup.setup_ui(self, extensions, self)

        self.dialog.show()

    def cancel(self):
        self.liststore = [[]]
        self.treeWidget.scene().clear()
        self.treeWidget.scene().update()
        self.table.setVisible(False)
        self.model = FtvModel()
        self.update_ui(self.model)

    def handleExportPressed(self, all = False):

        options = QFileDialog.options
        fileName, format = QFileDialog.getSaveFileName(self.MainWindow,
            "Save File", "", "PNG, JPG, PDF (*.png *.jpg *.pdf)")
        if fileName:
            if all:
                self.exportAll(fileName.split('.')[0], fileName.split('.')[1])
            else:
                self.export(fileName.split('.')[0], fileName.split('.')[1])

    def exportAll(self, fname, output_format):
        for i in range(len(self.liststore)):
            self.export(fname+"_"+self.liststore[i][0].text1, output_format)
            self.select_node_at(i+1)

        self.select_node_at(0)

    def export(self, fname, output_format):
        area = self.treeWidget.scene().sceneRect()
        area.setX(area.x()-100)
        area.setY(area.y()-100)
        area.setWidth(area.width()+100)
        area.setHeight(area.height()+100)

        image = QImage(int(area.width()), int(area.height())+20,
                       QImage.Format.Format_ARGB32_Premultiplied)
        image.fill(QColor(255, 255, 255))  # fill the image with white

        if output_format == 'png':
            self._save_png(image, fname, area)
        elif output_format == 'svg':
            self._save_svg(image, fname, area)
        elif output_format == 'pdf':
            self._save_pdf(image, fname, area)
        else:
            raise ValueError(f'Invalid output format: {output_format}')

    def _save_png(self, image, fname, area):
        painter = QPainter(image)
        self.treeWidget.scene().render(painter, QRectF(image.rect()), area)
        painter.end()
        image.save(f'{fname}.png')

    def _save_svg(self, image, fname, area):
        image.save(f'{fname}.svg')
        generator = QSvgGenerator()
        generator.setFileName(f'{fname}.svg')
        generator.setSize(image.size())
        generator.setViewBox(image.rect())

        painter = QPainter(generator)
        painter.fillRect(image.rect(), QColor('white'))
        self.treeWidget.scene().render(painter, QRectF(image.rect()), area)
        painter.end()

    def _save_pdf(self, image, fname, area):
        painter = QPainter(image)
        self.treeWidget.scene().render(painter, QRectF(image.rect()), area)
        painter.end()
        pil_image = Image.fromqimage(image)
        pil_image.convert('RGB').save(f'{fname}.pdf', 'PDF')

    def _zoom(self, zoomFactor):
        # Scale the QGraphicsView

        self.treeWidget.scale(zoomFactor, zoomFactor)

    def zoomIn(self):
        # Increase the zoom level by 10%
        zoomFactor = 1.1
        self._zoom(zoomFactor)

    def zoomOut(self):
        # Decrease the zoom level by 10%
        zoomFactor = 0.9
        self._zoom(zoomFactor)

    def zoomReset(self):
        # Reset the zoom level to the original transform
        self.treeWidget.setTransform(self.originalTransform)
        self.treeWidget.scale(0.5, 0.5)

    def view_tree(self):
        if not self.model.ls_tree:
            return
        for tree in self.model.ls_tree:
            pair = ({}, QGraphicsItemGroup())
            for i in treewalker(tree, self.draw_tree, pair,
                                order=TW_PREORDER):
                None
        self.update_nodes_table()
        self.select_node_at(0)
        self.visualize()

        return

    def visualize(self):
        for i in range(len(self.liststore)):
            for item in self.liststore[i]:
                self.treeWidget.scene().addItem(item)

    def show_info(self):
        self.dialog = QDialog()
        self.popup = Ui_Info(self.dialog)
        self.popup.setup_ui(self)

        self.dialog.show()

    def create_box(self, node, root_item, page=None, root=None, x=0, y=0):
        """
        create items
        """

        if node.info.isGate():

            page = node.info.page if not page else page

            item = GateItem(node.info.type, node.info.name,
                            node.info.notes, page, root, x, y)
        else:
            item = EventItem(
                node.info.name, node.info.failure_model_parameter, node.info.description, parent=None, x=x, y=y)
        item.setPos(x, y)
        item.setToolTip(node.info.description)
        self.liststore[-1].append(item)
        return item

    def setup_info(self):
        self.iconInfo = QIcon.fromTheme('dialog-information')
        self.actionInfo = QAction(self.iconInfo, "info", None)
        self.topToolBar.addAction(self.actionInfo)
        self.actionInfo.triggered.connect(lambda: self.show_info())

    def draw_tree(self, node, arg):
        """
        Draw the tree structure and add lines.
        Must be top down.
        """
        SIZE_W = 240
        SIZE_H = 240
        DIST_W = 20
        DIST_H = 80

        dict_C, group = arg

        if node in dict_C:
            c1 = dict_C[node]
            if node.info.isGate() and node.info.page == "On":

                raise StopIteration()
        else:  # root node
            center = (node.size_p - 1) * (SIZE_W + DIST_W) / 2
            if node.info.isGate() and node.info.page == "On":
                self.liststore.append([])
            c1 = self.create_box(node, group, "Off", "On", x=center + 10)
            dict_C[node] = c1

        # -------------------------------------------------------
        flag = False
        if node.children:
            y_node = dict_C[node].get_y() + SIZE_H + DIST_H
            x_node = dict_C[node].get_x()
            size_p = node.size_p
            size1 = size_p * (SIZE_W + DIST_W) - DIST_W
            acc = x_node - size1 / 2
            child_0 = node.children[0]

            if child_0.info.isGate() and child_0.info.page=="On":
                if child_0.info.isGate() and child_0.info.page == "On":
                    flag = True
            if not child_0.info.isGate() or child_0.info.page == "On":
                x0 = acc + SIZE_W

            else:
                x0 = acc + (child_0.size_p * (SIZE_W + DIST_W) -
                            DIST_W + SIZE_W) / 2

            for child in node.children:
                if not child.info.isGate() or child.info.page == "On":
                    size2 = SIZE_W
                    if child.info.isGate() and child.info.page == "On":
                        flag = True
                else:
                    size2 = child.size_p * (SIZE_W + DIST_W) - DIST_W

                item_x = acc + size2 / 2
                acc += size2 + DIST_W
                c2 = self.create_box(child, group, x=item_x, y=y_node)
                dict_C[child] = c2

            line_x0 = x0 - SIZE_W / 4
            line_y0 = y_node - SIZE_H + 50
            line_x1 = item_x + SIZE_W / 4
            line_y1 = line_y0
            line = HorizontalLine(line_x0, line_y0, line_x1, line_y1)
            self.liststore[-1].append(line)


class Ui_Info(object):

    def __init__(self, Dialog):
        self.Dialog = Dialog
        self.captions = {GateItem('AND', None, None, None) : 'This is an AND gate',
                            GateItem('OR', None, None, None): 'This is an OR gate',
                            GateItem('PAND', None, None, None): 'This is a PAND gate',
                            EventItem(None, None, None): 'This is an event' }

    def setup_ui(self, Dialog):
        self.Dialog.resize(350, 300)
        self.Dialog.setWindowTitle("Info")



        self.gridLayout = QGridLayout()
        for i in range(4):
            self.gate = TreeWidget()
            self.gate.setSceneRect(-100, -200, 350, 300)
            self.gate.scale(0.3, 0.3)
            self.gate.scene().addItem(list(self.captions.keys())[i])
            self.description = QLabel(self.Dialog)
            self.description.setText(list(self.captions.values())[i])
            self.gridLayout.addWidget(self.gate, i, 0, 1, 1)
            self.gridLayout.addWidget(self.description, i, 1, 1, 1)


        self.buttonBox = QDialogButtonBox(self.Dialog)
        self.buttonBox.setOrientation(Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.StandardButton.Ok)
        self.buttonBox.accepted.connect(self.accept)

        self.verticalLayout = QVBoxLayout(self.Dialog)
        self.verticalLayout.addLayout(self.gridLayout)
        self.verticalLayout.addWidget(self.buttonBox)

        QMetaObject.connectSlotsByName(self.Dialog)

    def accept(self):
        self.Dialog.close()



def run(efile, gfile, xmlfile, outfile, output_format):
    app = QApplication(sys.argv)
    app.setQuitOnLastWindowClosed(False)

    model = FtvModel()

    if ((efile and gfile) or xmlfile):
        model.create_model((efile, gfile, xmlfile))

    ui = Ui_FaultTreeViewerMainWindow()
    main_window = ViewerMainWindow()

    ui.setup_ui(main_window, model)
    ui.update_ui(model)
    ui.view_tree()
    if outfile is not None:
        ui.export(outfile, output_format)
        print("Fault Tree dumped to file: %s.%s" % (outfile, output_format))
        main_window.quit_terminal_process()
    else:
        main_window.show()
    sys.exit(app.exec_())

class Ui_Upload(object):

    def __init__(self, Dialog):
        self.Dialog = Dialog

    def setup_ui(self, Dialog, infos, hehe):
        self.Dialog.resize(350, 300)
        self.Dialog.setWindowTitle("Load model")
        self.hehe = hehe
        self.infos = infos

        self.gridLayout = QGridLayout()

        actions = [
            {'text': 'Choose', 'triggered': lambda: self.open_file(list(self.infos.values())[0], 0)},
            {'text': 'Choose', 'triggered': lambda: self.open_file(list(self.infos.values())[1], 1)},
            {'text': 'Choose', 'triggered': lambda: self.open_file(list(self.infos.values())[2], 2)}
        ]

        self.lineEdits = []

        for i in range(len(actions)):
            self.description = QLabel(self.Dialog)
            self.description.setText(list(self.infos)[i])
            lineEdit = QLineEdit(self.Dialog)
            lineEdit.setReadOnly(True)
            self.gridLayout.addWidget(self.description, i, 0, 1, 1)
            self.gridLayout.addWidget(lineEdit, i, 1, 1, 1)
            self.lineEdits.append(lineEdit)
            action = QPushButton(actions[i]['text'], self.Dialog)
            if actions[i]['triggered']:
                action.clicked.connect(actions[i]['triggered'])
            self.gridLayout.addWidget(action, i, 2, 1, 1)



        self.buttonBox = QDialogButtonBox(self.Dialog)
        self.buttonBox.setOrientation(Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.StandardButton.Ok)
        self.buttonBox.accepted.connect(self.accept)

        self.verticalLayout = QVBoxLayout(self.Dialog)
        self.verticalLayout.addLayout(self.gridLayout)
        self.verticalLayout.addWidget(self.buttonBox)

        QMetaObject.connectSlotsByName(self.Dialog)

    def accept(self):
        self.hehe.cancel()
        self.hehe.model.create_model((self.lineEdits[2].text(), self.lineEdits[1].text(), self.lineEdits[0].text()))
        self.hehe.update_ui(self.hehe.model)
        self.hehe.view_tree()
        self.Dialog.close()

    def open_file(self, extension='xml', index=0):

        file_dialog = QFileDialog
        if extension in ['xml', 'txt']:
            fname,_ = file_dialog.getOpenFileName(
                self.Dialog, "Open File", "", "{} Files (*.{})".format(extension, extension))
            if index == 0:
                self.lineEdits[1].clear()
                self.lineEdits[2].clear()
            else:
                self.lineEdits[0].clear()
            self.lineEdits[index].setText(fname)
            return fname
        else:
            raise Exception('Invalid extension')
