#treewalker

(TW_PREORDER,
 TW_POSTORDER, ) = range(2)


def treewalker(tree, foo, arg=None, order=TW_PREORDER):
    """
    visit tree:
    - in preorder when foo() return exception "StopIteration"
      the children of node are not visited
    - in postorder this is not possible because childern
      are visited before; exceptions are ignored
    """

    # Warning: recursive versions are deprecated, substituted by
    # iterative when fixing issue #5223

    def preorder_recur(t, foo):
        try:
            out = foo(t)
        except StopIteration as ex:
            out = ex[0] if ex.args else None
            yield (t, out)
        else:
            yield (t, out)
            for child in t.children:
                for m, o in preorder_recur(child, foo):
                    yield (m, o)
    # --------------------------------------------------

    def preorder_iter(t, foo):
        stack = [t]

        while stack:
            curr = stack.pop()
            try:
                out = foo(curr)
            except StopIteration as ex:
                out = ex[0] if ex.args else None
                yield (curr, out)
            else:
                yield (curr, out)
                stack.extend(reversed(curr.children))
    # --------------------------------------------------

    def postorder_recur(t, foo):
        for child in t.children:
            for m, o in postorder_recur(child, foo):
                yield (m, o)

        try:
            out = foo(t)
        except StopIteration as ex:
            out = ex[0] if ex.args else None

        yield (t, out)
    # --------------------------------------------------

    def postorder_iter(t, foo):
        stack = [t]
        prev = None
        while len(stack):
            curr = stack[-1]
            if not curr.children or prev == curr.children[-1]:
                # leave or done with children
                try:
                    out = foo(curr)
                except StopIteration as ex:
                    out = ex[0] if ex.args else None
                stack.pop()
                yield (curr, out)

            else:
                stack.extend(reversed(curr.children))

            prev = curr
    # --------------------------------------------------

    _foo = (foo if arg is None
            else lambda arg0: foo(arg0, arg))

    _treewalker = (preorder_iter if TW_PREORDER == order
                   else postorder_iter)

    return _treewalker(tree, _foo)