import os

class TFPGModel(object):
    """Model for the TFPG graph visualization"""

    def __init__(self):
        """Class constructor"""
        return

    def build_graph(self, tfpg_fname):
        """Builds the corresponding graph for a tfpg file.

        The structural file can be in xml syntax or human readable.
        Depending on its extension ('xml' or 'tfpg'), it calls
        the proper function

        :param tfpg_fname: the input structural file
        :type  tfpg_fname: string

        """
        _, file_extension = os.path.splitext(tfpg_fname)
        from xsap.data_structure.tfpg.xml.tfpg import parse_tfpg_xml_file_to_internal_struct
        from xsap.data_structure.tfpg.txt.tfpg import parse_tfpg_txt_file_to_internal_struct


        if file_extension in [".xml", ".txml"]:
            return parse_tfpg_xml_file_to_internal_struct(tfpg_fname)

        elif file_extension == ".tfpg":
            return parse_tfpg_txt_file_to_internal_struct(tfpg_fname)

        else:
            raise Exception('Invalid extension')

    def build_trace(self, xml_file):
        """Builds the corresponding trace for the xml file

        :param xml_file: the xml file containing the tfpg trace
        :type xml_file: string

        """

        from viewers.tfpg_viewer.src.models.xml_trace_parser import XmlTraceParser

        xml_trace_parser = XmlTraceParser()
        return xml_trace_parser.build_trace(xml_file)


pass  #
