import re


class Filter():
    def __init__(self):
        self.__filtername = ''
        self.__filter = {'names': [], 'names_compiled': [],
                         'ranges': [], 'types': []}

    def get_filter_name(self):
        return self.__filtername

    def get_names(self):
        """ get admitted names """
        ls = []
        ls.extend(self.__filter['names'])
        return ls

    def get_range_iter(self):
        for r in self.__filter['ranges']:
            yield r

    def get_ranges(self):
        ls = []
        ls.extend(self.__filter['ranges'])
        return ls

    def get_types(self):
        ls = []
        ls.extend(self.__filter['types'])
        return ls

    def get_type_iter(self):
        for t in self.__filter['types']:
            yield t

    def set_filter_name(self, new):
        """
        change the name that identify a filter.
        """
        if isinstance(new, str) or isinstance(new, unicode):
            self.__filtername = new
            return

    def set_names(self, new):
        """
            set the 'names' field of this filter.
            The names will be filtered with the python function 'strip'.
            @param: the new names (a string of comma separated names)
            @returns the list of invalid names if there are
        """
        # building list of filters

        if isinstance(new, list):
            # strip the names
            # and avoid invalid filters
            _ls = [f for f in map(str.strip, new) if f]

        elif isinstance(new, str) or isinstance(new, unicode):
            _ls = [f for f in map(str.strip, new.split(',')) if f]

        elif not isinstance(new, str) and not isinstance(new, unicode):
            raise TypeError("This "+str(new)+" is not a string")

        else:
            raise TypeError("Invalid type passed "+str(new))

        self.__filter['names'] = []
        self.__filter['names_compiled'] = []

        errors = []
        # strip and avoid invalid filters
        for f in _ls:
            try:
                pattern = re.compile(str(f))
            except Exception as e:
                errors.append(f)
                continue

            self.__filter['names'].append(f)
            self.__filter['names_compiled'].append(pattern)
            pass

        return errors

    def set_ranges(self, ranges):
        """
        @param: a list of tuple, each tuple represent a range
                the values in the tuple must be integers
                -1 is used to represent "until the end".
        """
        if not isinstance(ranges, list):
            raise TypeError("This "+str(ranges)+" is not a list")

        try:
            self.__filter['ranges'] = [(int(x), int(y)) for (x, y) in ranges]
        except (TypeError, ValueError):
            raise TypeError(
                "the parameter passed have to be a list of tuple, with 2 integers inside")
            pass

        return

    def set_types(self, types):
        """
        @param: a list of strings, each string represent a type
        admitted type: input, combinatorial, state
        """
        if isinstance(types, list):
            st = map(str.strip, types)
            # filter the non admitted types
            self.__filter['types'] = [t for t in st if str(t) == "state"
                                      or str(t) == "input"
                                      or str(t) == "combinatorial"
                                      or str(t) == "error"]
        elif isinstance(types, str) or isinstance(types, unicode):
            st = map(str.strip, types.split(','))
            self.__filter['types'] = [t for t in st if str(t) == "state"
                                      or str(t) == "input"
                                      or str(t) == "combinatorial"
                                      or str(t) == "error"]
        else:
            raise TypeError(
                "The parameter have to be a list of strings, or a string of comma-separated values")

        return

    def is_valid_step(self, step):
        """
        take an integer, positive value, and return
        true if the param is in the ranges of the filter,
        false otherwise.
        both start and end values are included.
        @param: must be integer
        """
        # if no ranges specified return true (each range)
        if not self.__filter['ranges']:
            return True

        result = False
        # check if the step statisfy a range
        for r in self.__filter['ranges']:
            start, end = r
            # i'll sum 1 because steps start from 0, ranges from 1.
            bs = step + 1 >= start
            if end != -1:
                be = step + 1 <= end
            else:
                be = True

            result = result or (bs and be)

            pass

        return result

    def is_valid_type(self, t):
        """
        take a type, and return True if this type is admitted
        false otherwise.
        """
        if not self.__filter['types']:
            return True
        return t in self.__filter['types']

    def is_valid_name(self, name):
        """
        take a name, and return True if this type is admitted
        false otherwise.
        """
        def match(regex, name):  # return yes if the matched string is the whole string
            m = regex.match(name)
            if m:
                return m.group() == name
            else:
                return False

        # if no names specified return true (each name)
        if not self.__filter['names']:
            return True

        result = False
        for regex in self.__filter['names_compiled']:
            result = result or match(regex, name)
            pass

        return result
