from PySide2.QtCore import Qt, QMetaObject
from PySide2.QtWidgets import (
    QDialog,
    QDialogButtonBox,
    QGridLayout,
    QLabel,
    QLineEdit,
    QTextEdit,
    QVBoxLayout,
)

from trace_viewer.filter import Filter


class Ui_Dialog(object):
    def __init__(self, Dialog):
        self.Dialog = Dialog

    def setup_ui(self, parentWindow, trace):

        self.RANGE_CHAR = '-'
        self.RANGES_SEPARATOR = ','
        self.DESCRIPTION = "(Comma seperated fields, each comma is equivalent to OR)\n\nExamples: a.*, [abc]*, [abd]"
        self.filter = filter
        self.trace = trace
        self.parentWindow = parentWindow

        self.Dialog.resize(350, 300)
        self.Dialog.setWindowTitle("Edit Filters")

        self.label_filter_name = QLabel(self.Dialog)
        self.lineEdit_filter_name = QLineEdit(self.Dialog)
        self.lineEdit_filter_name.setText(trace.activeFilter.get_filter_name())

        self.textEdit_description = QTextEdit(self.Dialog)
        self.textEdit_description.setPlainText(self.DESCRIPTION)
        self.textEdit_description.setReadOnly(True)
        self.textEdit_description.setStyleSheet(
            "background: rgba(255,255,255,0%)")

        self.label_names = QLabel(self.Dialog)
        self.lineEdit_names = QLineEdit(self.Dialog)
        self.lineEdit_names.setText(
            self.RANGES_SEPARATOR.join(trace.activeFilter.get_names()))

        self.label_ranges = QLabel(self.Dialog)
        self.lineEdit_ranges = QLineEdit(self.Dialog)
        self.lineEdit_ranges.setText(
            self.RANGES_SEPARATOR.join(self.__steps()))

        self.label_types = QLabel(self.Dialog)
        self.lineEdit_types = QLineEdit(self.Dialog)
        self.lineEdit_types.setText(
            self.RANGES_SEPARATOR.join(trace.activeFilter.get_types()))

        self.buttonBox = QDialogButtonBox(self.Dialog)
        self.buttonBox.setOrientation(Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(
            QDialogButtonBox.StandardButton.Cancel | QDialogButtonBox.StandardButton.Save)
        self.buttonBox.accepted.connect(self.accept)
        self.buttonBox.rejected.connect(self.reject)

        self.label_filter_name.setText("Filter name: ")
        self.label_names.setText("Filtered names: ")
        self.label_ranges.setText("Filtered steps: ")
        self.label_types.setText("Filtered types: ")

        self.gridLayout = QGridLayout()
        self.gridLayout.addWidget(self.label_filter_name, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.lineEdit_filter_name, 0, 1, 1, 1)
        self.gridLayout.addWidget(self.textEdit_description, 1, 0, 1, 2)
        self.gridLayout.addWidget(self.label_names, 2, 0, 1, 1)
        self.gridLayout.addWidget(self.lineEdit_names, 2, 1, 1, 1)
        self.gridLayout.addWidget(self.label_ranges, 3, 0, 1, 1)
        self.gridLayout.addWidget(self.lineEdit_ranges, 3, 1, 1, 1)
        self.gridLayout.addWidget(self.label_types, 4, 0, 1, 1)
        self.gridLayout.addWidget(self.lineEdit_types, 4, 1, 1, 1)

        self.verticalLayout = QVBoxLayout(self.Dialog)
        self.verticalLayout.addLayout(self.gridLayout)
        self.verticalLayout.addWidget(self.buttonBox)

        QMetaObject.connectSlotsByName(self.Dialog)

    def accept(self):

        if self.trace.activeFilter.get_filter_name() == "":

            new_filter = Filter()

            new_name = self.lineEdit_filter_name.text()
            new_names = self.lineEdit_names.text()
            new_ranges = self.__parse_steps(self.lineEdit_ranges.text())
            new_types = self.lineEdit_types.text()

            new_filter.set_filter_name(new_name)
            new_filter.set_names(new_names)
            new_filter.set_ranges(new_ranges)
            new_filter.set_types(new_types)

            self.trace.add_filter(new_filter)
            self.parentWindow.comboBox.addItem(new_filter.get_filter_name())
            new_index = self.parentWindow.comboBox.count()-1
            self.parentWindow.comboBox.setCurrentIndex(new_index)

        else:

            new_name = self.lineEdit_filter_name.text()
            new_names = self.lineEdit_names.text()
            new_ranges = self.__parse_steps(self.lineEdit_ranges.text())
            new_types = self.lineEdit_types.text()

            self.trace.activeFilter.set_filter_name(new_name)
            self.trace.activeFilter.set_names(new_names)
            self.trace.activeFilter.set_ranges(new_ranges)
            self.trace.activeFilter.set_types(new_types)

            index = self.parentWindow.comboBox.currentIndex()
            self.parentWindow.comboBox.setItemText(index, new_name)

            self.parentWindow.combo_box_changed(self.trace)

        self.parentWindow.lineEditNameSearch.clear()
        self.Dialog.close()

    def reject(self):
        self.Dialog.close()

    def __steps(self):
        # replace -1 with ''
        ls = []
        for x, y in self.trace.activeFilter.get_ranges():
            if x == y:
                # this means that there is only a column in the range
                ls.append(str(x))
                continue

            if x == -1:
                v1 = ''
            else:
                v1 = str(x)

            if y == -1:
                v2 = ''
            else:
                v2 = str(y)

            ls.append(v1+self.RANGE_CHAR+v2)

        return ls

    def __parse_steps(self, text):

        def empty_to_minus1(ls):
            # transform the list of ranges and set the 'empty' to -1 in order to match the model requirements
            ls1 = []
            for r in ls:
                try:
                    x, y = r
                except ValueError:
                    # this means that there is only a value in the range
                    ls1.append((int(r[0]), int(r[0])))
                    continue

                if not x:
                    v1 = -1
                else:
                    v1 = int(x)

                if not y:
                    v2 = -1
                else:
                    v2 = int(y)
                ls1.append((v1, v2))

            return ls1

        if not text:
            return []
        return empty_to_minus1([r for r in [se.split(self.RANGE_CHAR) for se in text.split(self.RANGES_SEPARATOR)]])
