import xml.dom.minidom
from trace_viewer.trace import TraceModel


class NuSMVTraceReader(object):

    def parse(self, filename, fault_file = None, timed_var = None):
        """Parses the trace in xml format contained in the given file.
        @filename the name of the xml file to parse
        It must be a traceModel or a derived class.
        @return a new filled TraceModel instance, or None if no trace is
        found"""

        root = xml.dom.minidom.parse(filename)
        node = root.documentElement

        if node.tagName != "counter-example":
            return None

        steps = node.getElementsByTagName("node")
        loops = node.getElementsByTagName("loops")

        # collects steps info
        names = {}
        names_list = {'state': [],
                      'combinatorial': [],
                      'input': [],
                      'fault':[]
                      }
        timed = None

        loop_list = []
        for step in steps:
            for field in names_list:
                values = step.getElementsByTagName(field)

                for val in values:
                    for var in val.getElementsByTagName("value"):
                        name = var.getAttribute("variable")

                        if name not in names:
                            names[name] = []
                            names_list[field].append(name)

                        val = ""
                        for text_child in var.childNodes:
                            if text_child.nodeType == xml.dom.Node.TEXT_NODE:
                                val += text_child.data

                        names[name].append(val)
                        

        loops_strs = ""

        for loop in loops:
            val = ""
            for text_child in loop.childNodes:
                if text_child.nodeType == xml.dom.Node.TEXT_NODE:
                    val += text_child.data

            val = val.strip()
            if val:
                loops_strs += val + " "

        loops_strs = loops_strs.replace(",", " ").strip()
        for loop in loops_strs.strip().split(" "):
            if loop != "":
                loop_list.append(int(loop))

        if fault_file is not None:
            not_found_vars = []
            root = xml.dom.minidom.parse(fault_file)
            node = root.documentElement
        
            ftvlist = node.getElementsByTagName("fm")
            for ftv in ftvlist:
                name = ftv.getAttribute("name")
                if name not in names.keys():
                    not_found_vars.append(name)
                else:
                    names_list['fault'].append(name)
            
            if len(not_found_vars)!=0:
                raise Exception('Following variables are not in the trace: ',not_found_vars)


        if timed_var is not None:
            for name in names:
                if name == timed_var:
                    timed = name

            if timed is None:
                raise Exception('Following timed variable is not in the trace: ', timed_var)

            if timed in names_list['fault']:
                raise Exception('Timed variable can not be a fault variable ', timed_var)
        return TraceModel(names, names_list, loop_list, timed, filename)
    
