from PySide2.QtGui import QStandardItemModel, QColor
from PySide2.QtCore import Qt
from trace_viewer.filter import Filter


class TraceModel(QStandardItemModel):

    name = None

    def __init__(self, names=None, names_list=None, loops=None, timed_var = None, fname=None):
        super().__init__()
        self.names = names
        self.names_list = names_list
        self.loops = loops
        self.fname = fname
        self.timed_var = timed_var

        self.activeFilter = Filter()
        self.filter = Filter()
        self.filter.set_filter_name("no filter")
        self.filters = [self.filter]

    def get_fname(self):
        return self.fname

    def set_active_filter(self, filter):
        self.activeFilter = filter

    def get_active_filter(self):
        return self.activeFilter

    def get_type(self, name):
        for field in self.names_list:
            if name in self.names_list[field]:
                return field

    def get_filter_names(self):
        return [x.get_filter_name() for x in self.filters]

    def get_filter(self, name, index=False):
        """
        Get a particular TraceFilter instance.
        @param name: the name of the filter
        @param index: If true, an index in the list of filter will be returned
                      (it can be used as a path in function get_iter).
        NB: The filter name is different from the 'names' field of the filter.
        """
        for i in range(len(self.filters)):
            f = self.filters[i]

            if f.get_filter_name() == name:
                if index:
                    return i
                else:
                    return f

        raise KeyError("This filter was not found!")

    def add_filter(self, filter):
        self.filters.append(filter)

    def change_filter_at_pos(self, new_filter, pos):
        self.filters[pos] = new_filter

    def get_filter_at_pos(self, pos):
        return self.filters[pos]

    def get_filters_count(self):
        return len(self.filters)

    def get_length(self):
        return len(self.names)

    def get_value(self, pos):
        return sorted(list(self.names))[pos]

    def get_best_row_color(self, pos):
        for field in self.names_list:
            if sorted(list(self.names))[pos] in self.names_list[field]:
                if field == 'input':
                    return QColor(0, 0, 255)
                elif field == 'state':
                    return QColor(0, 0, 0)
                elif field == 'combinatorial':
                    return QColor( 0, 128, 0)
                else:
                    raise Exception("Invalid field name")

    def get_best_cell_color(self, val, pos):
        color = QColor(238, 237, 231)
        changed = False
        if pos == 0:
            return QColor(238, 237, 231)
        else:
            if self.get_value_at_step(val, pos) == self.get_value_at_step(val, pos - 1):
                return self.get_best_cell_color(val, pos - 1)
            else:
                if self.get_best_cell_color(val, pos - 1) == QColor(238, 237, 231):
                    return QColor(185, 183, 189)
                else:
                    return QColor(238, 237, 231)
    def get_best_row_background_color(self, pos):
        if self.get_value(pos) in self.names_list['fault']:
            return QColor(112, 0, 0)
        elif self.get_value(pos) is self.timed_var:
            return QColor(255, 204, 255)
        elif self.get_value(pos) in self.names:
            return QColor(255, 255, 255)
        else:
            raise Exception("Invalid field name")
        
    def get_best_step_color(self, step):
        if self.is_step_a_loop(step):
            return Qt.GlobalColor.red
        else:
            return Qt.GlobalColor.gray

    def get_steps(self):
        max_len = 0
        for key in self.names:
            max_len = max(len(self.names[key]), max_len)
        return max_len

    def get_value_at_step(self, val, pos):
        if len(self.names[val]) > pos:
            return list(self.names[val])[pos]
        else:
            return ""

    def add_loop_at_step(self, step):
        """Adds a loop from the last state to the state at given step"""
        if step not in self.loops:
            self.loops.append(step)
            pass
        return

    def is_step_a_loop(self, step):
        """@return True if the given step is a loop step"""
        return step in self.loops
