'''
Created on Oct 21, 2011

@author: rayan
'''
from lxml import etree
from string import Template

from feic.fe.data_structure.model.xmlBound import XmlBound


class Effect(XmlBound):
    """Manages the creation of the structure effect. Can generate a new
    structure based on its value or read from an existing one. Also
    handles the dumping to FE.
    """

    def __init__(self, **kwargs):
        XmlBound.__init__(self, {}, "effect", **kwargs)
        setattr(self, "entering", kwargs.get('entering', ""))
        setattr(self, "during", kwargs.get('during', ""))

    def toXML(self):
        root = etree.Element(self.tag)
        self.solveTemplates(map)
        entering = etree.SubElement(root, "entering")
        during = etree.SubElement(root, "during")
        entering.text = self.entering
        during.text = self.during
        return root

    def fromXML(self, root):
        self._fromXMLAttrs(root, iter(self.m_map.values()))
        self.entering = (
            self.get_mandatory_singleton_elem_text(
                root, "./%s" % "entering"))
        self.during = (
            self.get_mandatory_singleton_elem_text(
                root, "./%s" % "during"))

    def solveTemplates(self, _map):
        """
        For each attribute in the current element finds and replaces templates.
        @param _map: The template _map containing the replace strings
        """
        for member in self.m_map.values():
            setattr(self, member,
                    Template(getattr(self, member)).substitute(_map))

        self.entering = Template(self.entering).substitute(_map)
        self.during = Template(self.during).substitute(_map)

    pass  # end of class
