'''
Created on Oct 10, 2011

@author: rayan
'''


from lxml import etree

from feic.fe.data_structure.model.event import Event
from feic.fe.data_structure.model.transition import Transition
from feic.fe.data_structure.model.xmlBound import XmlBoundLineno


class GlobalDynamics(XmlBoundLineno):
    """Manages the creation of the structure global dynamics. Can
    generate a new structure based on its value or read from an
    existing one. Also handles the dumping to FE.
    """
    GDM = "global_dynamics_model"
    EVNT = "event"
    EVNTS = "events"
    TRNS = "transitions"
    TRN = "transition"

    def __init__(self, **kwargs):
        XmlBoundLineno.__init__(self, {}, self.GDM, **kwargs)
        setattr(self, "events", kwargs.get('events', []))
        setattr(self, "transitions", kwargs.get('transitions', []))
        setattr(self, "enabled", kwargs.get('enabled', "yes"))

    def toXML(self):
        """Creates a lxml element based on the values contained in the
        current instance of the element. Assigns parameters and
        processes subelements.
        """
        root = None

        if self.enabled == "yes":
            root = etree.Element(self.tag)
            events = etree.SubElement(root, "events")
            transitions = etree.SubElement(root, "transitions")
            self._toXMLAttr(root, iter(self.m_map.values()))
            self._toXMLSubElems(events, self.events)
            self._toXMLSubElems(transitions, self.transitions)
            pass

        return root

    def fromXML(self, root):
        """
        Creates the root element and lists the needed parameters.
        If exists calls the creation of all the sub elements.

        @param root: lxml Element from which we start picking elements.
        """
        # reading all attributes
        self._fromXMLAttrs(root, iter(self.m_map.values()))

        self.events = self._fromXMLSubElems(self.get_all_sub_elems(
            root, "./%s" % self.EVNTS))
        self.transitions = self._fromXMLSubElems(self.get_all_sub_elems(
            root, "./%s" % self.TRNS))

    def generateSubelement(self, elem):
        """
        Instantiates an element depending on the tag name.

        @param elem: The element to be instantiated.
        """
        map = {self.EVNT: Event, self.TRN: Transition}

        # instantiates the element in the correct class
        obj = (map[elem.tag]() if elem.tag in map
               else None)
        assert obj
        obj.fromXML(elem)
        return obj

    pass  # end of class
