'''
Created on Oct 7, 2011

@author: rayan
'''

from lxml import etree
from feic.fe.data_structure.model.fault_model import FaultModel
from feic.fe.data_structure.model.xmlBound import XmlBoundLineno


class NCExtension(XmlBoundLineno):
    """
    Manages the creation of the structure nominal component extension. Can
    generate a new structure based on its value or read from an existing one.
    Also handles the dumping to FE.
    """
    MODULE = "module"
    DESC = "desc"
    FMDLS = "fault_models"
    NCX = "nominal_component_extensions"
    FMDL = "fault_model"
    FMDLS = "fault_models"

    def __init__(self, **kwargs):
        XmlBoundLineno.__init__(self,
                                {self.MODULE: "module",
                                 self.DESC: "desc"},
                                self.NCX, **kwargs)

        setattr(self, "fault_models", kwargs.get('fault_models', []))
        return

    def toXML(self):
        """
        Creates a lxml element based on the values contained in the current
        instance of the element. Assigns parameters and processes subelements.
        """
        root = etree.Element(self.tag)
        fmodels = etree.SubElement(root, self.FMDLS)
        self._toXMLAttr(root, iter(self.m_map.values()))
        self._toXMLSubElems(fmodels, self.fault_models)
        if XmlBoundLineno.dumping_fe:
            self.deleteAttrs(root, ["desc"])
        return root

    def fromXML(self, root):
        """
        Creates the root element and lists the needed parameters.
        If exists calls the creation of all the sub elements.

        @param root: lxml Element from which we start picking elements.
        """
        self._fromXMLAttrs(root, iter(self.m_map.values()))
        self.fault_models = (
            self._fromXMLSubElems(
                self.get_all_sub_elems(root,
                                       "./%s" % self.FMDLS)))
        return

    def generateSubelement(self, elem):
        """
        Instantiates an element depending on the tag name.

        @param elem: The element to be instantiated.
        """
        _map = {self.FMDL: FaultModel}
        # instantiates the element in the correct class
        obj = _map[elem.tag]() if elem.tag in _map else None
        assert obj
        obj.fromXML(elem)
        return(obj)

    pass  # end of class
