'''
Created on Oct 21, 2011

@author: rayan
'''

from lxml import etree
from string import Template

from feic.fe.data_structure.model.xmlBound import XmlBound


class RawSmv(XmlBound):
    """
    Manages the creation of the structure raw smv. Can generate a new structure
    based on its value or read from an existing one. Also handles the dumping
    to FE.
    """
    def __init__(self, **kwargs):
        XmlBound.__init__(self, {}, "raw", **kwargs)
        self.code = kwargs.get('code', "")
        return

    def toXML(self):
        """
        Creates a lxml element based on the values contained in the current
        instance of the element. Assigns parameters and processes subelements.
        """
        self.solveTemplates(map)
        root = etree.Element(self.tag)
        root.text = self.code
        return root

    def fromXML(self, root):
        """
        Creates the root element and lists the needed parameters.
        If exists calls the creation of all the sub elements.

        @param root: lxml Element from which we start picking elements.
        """
        self._fromXMLAttrs(root, iter(self.m_map.values()))
        self.code = root.text
        return

    def solveTemplates(self, _map):
        """
        For each attribute in the current element finds and replaces templates.
        @param _map: The template _map containing the replace strings
        """
        for member in self.m_map.values():
            setattr(self, member,
                    Template(getattr(self, member)).substitute(_map))
            pass

        self.code = Template(self.code).substitute(_map)
        return

    pass  # end of class
