'''
Created on Oct 10, 2011

@author: rayan
'''

from lxml import etree
from string import Template
from feic.fe.data_structure.model.xmlBound import XmlBoundLineno


class Transition (XmlBoundLineno):
    """
    Manages the creation of the structure transitions. Can generate a new
    structure based on its value or read from an existing one. Also handles the
    dumping to FE.
    """
    FRM = "from"
    TO = "to"
    TRNS = "transition"
    GUARD = "guard"
    TRIGGER = "trigger"
    ENABLED = "enabled"

    def __init__(self, **kwargs):
        XmlBoundLineno.__init__(self,
                                {self.FRM: "from",
                                 self.TO: "to",
                                 self.ENABLED: "enabled"},
                                self.TRNS, **kwargs)

        self.guard = kwargs.get('guard', "")
        self.trigger = kwargs.get('trigger', "")
        self.enabled = kwargs.get('enabled', "yes")
        return

    def toXML(self):
        root = None

        if self.enabled == "yes" or not XmlBoundLineno.dumping_fe:
            root = etree.Element(self.tag)

            self._toXMLAttr(root, iter(self.m_map.values()))
            if XmlBoundLineno.dumping_fe:
                self.deleteAttrs(root, [self.ENABLED])
                pass

            etree.SubElement(root, self.GUARD).text = self.guard

            etree.SubElement(root, self.TRIGGER).text = self.trigger
            pass

        return root

    def fromXML(self, root):
        self._fromXMLAttrs(root, iter(self.m_map.values()))

        self._fromXMLSubElems(self.get_all_sub_elems(root, "."))
        return

    def _fromXMLSubElems(self, subElems):
        _map = {Transition.GUARD: "guard", Transition.TRIGGER: "trigger"}
        for elem in subElems:
            assert elem.tag in _map and hasattr(self, _map[elem.tag])
            if elem.text:
                setattr(self, _map[elem.tag], str(elem.text))
            else:
                setattr(self, _map[elem.tag], None)
            pass
        return

    def solveTemplates(self, _map):
        """
        For each attribute in the current element finds and replaces templates.
        @param _map: The template _map containing the replace strings
        """
        for member in self.m_map.values():
            if member:
                setattr(self, member,
                        Template(getattr(self, member)).substitute(_map))

        if self.guard:
            self.guard = Template(self.guard).substitute(_map)
        if self.trigger:
            self.trigger = Template(self.trigger).substitute(_map)

    pass  # end of class
