'''
Created on Oct 5, 2011

@author: rayan
'''

from lxml import etree
from feic.fe.data_structure.model.xmlBound import XmlBound


class UserInfo(XmlBound):
    """Manages the creation of the structure user info. Can generate a
    new structure based on its value or read from an existing
    one. Also handles the dumping to FE.
    """
    UINF = "user_info"
    NAME = "name"
    DESC = "desc"

    def __init__(self, **kwargs):
        XmlBound.__init__(self, {}, self.UINF, **kwargs)
        setattr(self, "name", kwargs.get('name', ""))
        setattr(self, "desc", kwargs.get('desc', ""))
        return

    def toXML(self):
        """Creates a lxml element based on the values contained in the
        current instance of the element. Assigns parameters and
        processes subelements.
        """
        root = etree.Element(self.tag)
        name = etree.SubElement(root, self.NAME)
        desc = etree.SubElement(root, self.DESC)
        name.text = self.name
        desc.text = self.desc
        return root

    def fromXML(self, root):
        """
        Creates the root element and lists the needed parameters.
        If exists calls the creation of all the sub elements.

        @param root: lxml Element from which we start picking elements.
        """
        self._fromXMLAttrs(root, iter(self.m_map.values()))
        self.name = (
            self.get_mandatory_singleton_elem_text(
                root, "./%s" % self.NAME))
        self.desc = (
            self.get_optional_singleton_elem_text(
                root, "./%s" % self.DESC))

    pass  # end of class
