'''
Created on Sep 23, 2011

@author: Cristian Mattarei
'''
from lxml import etree

from feic.fe.lib_mgr.elements.effectModel import Raw

### Tokens definition ###

LOCAL_DYNAMICS_MODEL = "local_dynamics_model"
LOCAL_DYNAMIC_LIBRARY = "local_dynamic_library"
DESCRIPTION = "description"
DEFINITION = "definition"
LOC = "loc"
NAME = "name"
TYPE = "type"
EVENT = "event"
EVENTS = "events"
TEMPLATES = "templates"
TEMPLATE = "template"
TRANSITION = "transition"
TRANSITIONS = "transitions"
FROM = "from"
TO = "to"
GUARD = "guard"
TRIGGER = "trigger"

###


class Template(object):

    _name = None
    _type = None
    _description = None

    def __init__(self, name, type_, description):
        if not isinstance(name, str):
            raise Exception("String instance expected (got %s instead)"
                            % type(name))

        if not isinstance(type_, str):
            raise Exception("String instance expected (got %s instead)"
                            % type(type_))

        if not isinstance(description, str):
            raise Exception("String instance expected (got %s instead)"
                            % type(description))

        self._name = name
        self._type = type_
        self._description = description

    def getObjectify(self):
        template = etree.Element(TEMPLATE)
        template.attrib[NAME] = self._name
        template.attrib[TYPE] = self._type
        template.text = self._description

        return template

    def __str__(self):
        return etree.tostring(self.getObjectify(), pretty_print=True)


class Event(object):

    _name = None
    _type = None

    def __init__(self, name, type):
        if not isinstance(name, str):
            raise Exception("String instance expected (got %s instead)"
                            % type(name))

        self._name = name
        self._type = type

    def getObjectify(self):
        event = etree.Element(EVENT)
        event.attrib[NAME] = self._name
        event.attrib[TYPE] = self._type

        return event

    def __str__(self):
        return etree.tostring(self.getObjectify(), pretty_print=True)


class Guard(object):

    _value = None

    def __init__(self, value):
        if not isinstance(value, str):
            raise Exception("String instance expected (got %s instead)"
                            % type(value))

        self._value = value

    def getObjectify(self):
        guard = etree.Element(GUARD)
        guard.text = self._value

        return guard

    def __str__(self):
        return etree.tostring(self.getObjectify(), pretty_print=True)


class Trigger(object):

    _value = None

    def __init__(self, value):
        if not isinstance(value, str):
            raise Exception("String instance expected (got %s instead)"
                            % type(value))

        self._value = value

    def getObjectify(self):
        trigger = etree.Element(TRIGGER)
        trigger.text = self._value

        return trigger

    def __str__(self):
        return etree.tostring(self.getObjectify(), pretty_print=True)


class Transition(object):

    _from = None
    _to = None
    _guard = None
    _trigger = None

    def __init__(self, from_, to, guard, trigger):
        if not isinstance(from_, str):
            raise Exception("String instance expected (got %s instead)"
                            % type(from_))

        if not isinstance(to, str):
            raise Exception("String instance expected (got %s instead)"
                            % type(to))

        if guard is not None and not isinstance(guard, Guard):
            raise Exception("Guard instance expected (got %s instead)"
                            % type(guard))

        if trigger is not None and not isinstance(trigger, Trigger):
            raise Exception("Trigger instance expected (got %s instead)"
                            % type(trigger))

        self._from = from_
        self._to = to
        self._guard = guard
        self._trigger = trigger
        return

    def getObjectify(self):
        transition = etree.Element(TRANSITION)
        transition.attrib[FROM] = self._from
        transition.attrib[TO] = self._to
        if self._guard is not None:
            transition.append(self._guard.getObjectify())
        if self._trigger is not None:
            transition.append(self._trigger.getObjectify())

        return transition

    def __str__(self):
        return etree.tostring(self.getObjectify(), pretty_print=True)


class LDefinition(object):
    '''
    classdocs
    '''

    _templates = None
    _events = None
    _transitions = None
    _raw = None

    def __init__(self, templates, events, transitions, raw):
        '''
        Constructor
        '''

        self._templates = None
        self._events = None
        self._transitions = None
        self._raw = None

        if not isinstance(templates, list):
            raise Exception("List of Value instance expected (got %s instead)"
                            % type(templates))

        if not isinstance(events, list):
            raise Exception("List of Effect instance expected (got %s instead)"
                            % type(events))

        if not isinstance(transitions, list):
            raise Exception("List of Effect instance expected (got %s instead)"
                            % type(transitions))

        if not isinstance(raw, Raw):
            raise Exception("Raw instance expected (got %s instead)"
                            % type(raw))

        self._templates = templates
        self._events = events
        self._transitions = transitions
        self._raw = raw

    def getTemplates(self):
        return self._templates

    def getEvents(self):
        return self._events

    def getTransitions(self):
        return self._transitions

    def getRaw(self):
        return self._raw

    def getObjectify(self):
        lDefinition = etree.Element(LOCAL_DYNAMICS_MODEL)
        lDefinition.attrib[NAME] = self._name
        templates = etree.SubElement(lDefinition, TEMPLATES)
        for el in self._templates:
            templates.append(el.getObjectify())

        events = etree.SubElement(lDefinition, EVENTS)
        for el in self._events:
            events.append(el.getObjectify())

        transitions = etree.SubElement(lDefinition, TRANSITIONS)
        for el in self._transitions:
            transitions.append(el.getObjectify())

        lDefinition.append(self._raw.getObjectify())

        return lDefinition

    def __str__(self):
        return etree.tostring(self.getObjectify(), pretty_print=True)


class LocalDynamic(object):
    '''
    classdocs
    '''

    _name = None
    _description = None
    _templates = None
    _events = None
    _transitions = None
    _raw = None

    def __init__(self, name, description, templates, events, transitions, raw):
        '''
        Constructor
        '''

        self._name = None
        self._description = None
        self._templates = None
        self._events = None
        self._transitions = None
        self._raw = None

        if not isinstance(name, str):
            raise Exception("String instance expected (got %s instead)"
                            % type(name))

        if not isinstance(description, str):
            raise Exception("String instance expected (got %s instead)"
                            % type(description))

        if not isinstance(templates, list):
            raise Exception("List of Value instance expected (got %s instead)"
                            % type(templates))

        if not isinstance(events, list):
            raise Exception("List of Effect instance expected (got %s instead)"
                            % type(events))

        if not isinstance(transitions, list):
            raise Exception("List of Effect instance expected (got %s instead)"
                            % type(transitions))

        if not isinstance(raw, Raw):
            raise Exception("Raw instance expected (got %s instead)"
                            % type(raw))

        self._name = name
        self._description = description
        self._templates = templates
        self._events = events
        self._transitions = transitions
        self._raw = raw

    def getName(self):
        return self._name

    def getObjectify(self, definition=None):
        lDefinition = etree.Element(LOCAL_DYNAMICS_MODEL)
        lDefinition.attrib[NAME] = self._name

        if definition is None:
            templates = etree.SubElement(lDefinition, TEMPLATES)
            for el in self._templates:
                templates.append(el.getObjectify())

            events = etree.SubElement(lDefinition, EVENTS)
            for el in self._events:
                events.append(el.getObjectify())

            transitions = etree.SubElement(lDefinition, TRANSITIONS)
            for el in self._transitions:
                transitions.append(el.getObjectify())

            lDefinition.append(self._raw.getObjectify())
        else:
            des = etree.SubElement(lDefinition, DESCRIPTION)
            des.text = self._description
            def_ = etree.SubElement(lDefinition, DEFINITION)
            def_.attrib[LOC] = definition

        return lDefinition

    def __str__(self):
        return etree.tostring(self.getObjectify(), pretty_print=True)
