'''
Created on Sep 21, 2011

@author: Cristian Mattarei
'''
import os.path
from lxml import etree

from feic.fe.lib_mgr.elements.effectModel import WRITES
from feic.fe.lib_mgr.elements.effectModel import (
    EffectModel, Value, INPUT, NAME,
    TYPE, READS, DESC, OUTPUT, ENTERING, LOCAL, Effect,
    FALSE, TRUE, LOC, Raw,
    EDefinition, RAW, EFFECTS_MODEL_LIBRARY, DURING)

from feic.fe.lib_mgr.elements.localDynamics import (
    Event, LDefinition, TEMPLATES,
    Template, Transition, FROM, TO, GUARD, TRIGGER,
    EVENTS, TRANSITIONS, Guard,
    Trigger, LocalDynamic, LOCAL_DYNAMIC_LIBRARY)

from feic.fe.lib_mgr.utils.utils import (
    GetPrefix, StrFromFile, LoadXML)

FSEP = "/"
MAIN = "main"
EPREFIX = "eml_behavior"
EPOSTFIX = "definition.xml"

LPREFIX = "ldl_behavior"
LPOSTFIX = "definition.xml"


class LibraryManager(object):
    '''
    classdocs
    '''

    _eml = None
    _ldl = None
    _location = None

    def __init__(self, location, eml, emlS, ldl, ldlS, eDefinS, lDefinS):
        '''
        Constructor
        '''

        self._eml = []
        self._ldl = []
        self._location = ""

        if not isinstance(location, str):
            raise Exception("String instance expected (got %s instead)",
                            type(location))

        if not isinstance(eml, str):
            raise Exception("String instance expected (got %s instead)",
                            type(eml))

        if not isinstance(emlS, str):
            raise Exception("String instance expected (got %s instead)",
                            type(emlS))

        if not isinstance(ldl, str):
            raise Exception("String instance expected (got %s instead)",
                            type(ldl))

        if not isinstance(ldlS, str):
            raise Exception("String instance expected (got %s instead)",
                            type(ldlS))

        if not isinstance(eDefinS, str):
            raise Exception("String instance expected (got %s instead)",
                            type(eDefinS))

        if not isinstance(lDefinS, str):
            raise Exception("String instance expected (got %s instead)",
                            type(lDefinS))

        self._eml = self.__loadEml(location, eml, emlS, eDefinS)

        self._ldl = self.__loadLdl(location, ldl, ldlS, lDefinS)

    def getEml(self):
        return self._eml

    def getLdl(self):
        return self._ldl

    def getEmlObjectify(self):
        return self.__generateEMLObjectify()

    def getLdlObjectify(self):
        return self.__generateLDLObjectify()

### Internal functions ###

    def __generateEMLObjectify(self):
        ret = {}
        ret[EFFECTS_MODEL_LIBRARY] = etree.Element(EFFECTS_MODEL_LIBRARY)
        for em in self._eml:
            ret[EFFECTS_MODEL_LIBRARY].append(
                em.getObjectify(EPREFIX + FSEP +
                                em.getName().lower()+FSEP+EPOSTFIX))
            ret[em.getName()] = em.getObjectify()

        return ret

    def __generateLDLObjectify(self):
        ret = {}
        ret[LOCAL_DYNAMIC_LIBRARY] = etree.Element(LOCAL_DYNAMIC_LIBRARY)
        for ld in self._ldl:
            ret[LOCAL_DYNAMIC_LIBRARY].append(ld.getObjectify(
                LPREFIX+FSEP+ld.getName().lower()+FSEP+LPOSTFIX))
            ret[ld.getName()] = ld.getObjectify()

        return ret

    def __loadEml(self, location, eml, emlS, definS):
        effects = LoadXML(eml, emlS).getchildren()
        eml = []
        for effect in effects:
            emdef = self.__loadEMDefinition(
                os.path.join(location,
                             effect.definition.attrib[LOC]), definS)
            eml.append(self.__generateEffect(effect, emdef))

        return eml

    def __loadLdl(self, location, ldl, ldlS, definS):
        dynamics = LoadXML(ldl, ldlS).getchildren()
        ldl = []
        for dynamic in dynamics:
            lddef = self.__loadLDDefinition(
                os.path.join(location, dynamic.definition.attrib[LOC]), definS)
            ldl.append(self.__generateDynamic(dynamic, lddef))

        return ldl

    def __loadEMDefinition(self, defin, definS):
        definitionTL = LoadXML(defin, definS)
        definition = definitionTL.getchildren()

        name_EM = definitionTL.attrib[NAME]
        templates = []
        for template in (definition[0].getchildren()
                         if definition[0].tag == TEMPLATES else []):
            name = template.attrib[NAME]
            type_ = template.attrib[TYPE]
            des = str(template)
            templates.append(Template(name, type_, des))

        values = []
        for value in (definition[0]
                      if definition[0].tag == "values" else definition[1]):
            inputs = value.xpath("./input")
            for x in inputs:
                type_ = x.attrib[TYPE]
                name = x.attrib[NAME] if (NAME in x.attrib) else None
                reads = x.attrib[READS] if (READS in x.attrib) else None
                writes = None
                desc = x.attrib[DESC]
                values.append(Value(INPUT, name, type_, reads, writes, desc))

            outputs = value.xpath("./output")
            for x in outputs:
                type_ = None
                reads = x.attrib[READS] if (READS in x.attrib) else None
                name = x.attrib[NAME] if (NAME in x.attrib) else None
                writes = x.attrib[WRITES] if (WRITES in x.attrib) else None
                desc = x.attrib[DESC]
                values.append(Value(OUTPUT, name, type_, reads, writes, desc))

        effects = []
        for effect in (definition[1]
                       if definition[1].tag == "effect"
                       else definition[2]):
            if effect.find(ENTERING) is not None:
                type_ = effect.entering.attrib[TYPE]
                local = {FALSE: False, TRUE: True}[
                    effect.entering.attrib[LOCAL]]
                desc = str(effect.entering)
                if not local:
                    desc = StrFromFile(GetPrefix(defin, FSEP)+desc)
                effects.append(Effect(ENTERING, type_, desc))

            if effect.find(DURING) is not None:
                type_ = effect.during.attrib[TYPE]
                local = {FALSE: False, TRUE: True}[effect.during.attrib[LOCAL]]
                desc = str(effect.during)
                if not local:
                    desc = StrFromFile(GetPrefix(defin, FSEP)+desc)
                effects.append(Effect(DURING, type_, desc))

        raw = (definition[2] if definition[2].tag == "raw" else definition[3])
        raw_type = None

        if str(raw) != "":
            type_ = raw.attrib[TYPE]
            local = {FALSE: False, TRUE: True}[raw.attrib[LOCAL]]
            if not local:
                raw = StrFromFile(GetPrefix(defin, FSEP)+str(raw))
            raw_type = type_

        return EDefinition(name_EM, templates, values,
                           effects, Raw(raw_type, str(raw)))

    def __loadLDDefinition(self, defin, definS):
        definition = LoadXML(defin, definS).getchildren()

        templates = []
        events = []
        transitions = []
        raw_type = None

        for template in (definition[0].getchildren()
                         if definition[0].tag == TEMPLATES else []):
            name = template.attrib[NAME]
            type_ = template.attrib[TYPE]
            des = str(template)
            templates.append(Template(name, type_, des))

        for event in (definition[0].getchildren()
                      if definition[0].tag == EVENTS
                      else definition[1].getchildren()):
            name = event.attrib[NAME]
            type = event.attrib[TYPE]
            events.append(Event(name, type))

        for transition in (definition[1].getchildren()
                           if definition[1].tag == TRANSITIONS
                           else definition[2].getchildren()):
            from_ = transition.attrib[FROM]
            to = transition.attrib[TO]
            guard = None
            if hasattr(transition, GUARD):
                guard = Guard(str(transition.guard))
            trigger = None
            if hasattr(transition, TRIGGER):
                trigger = Trigger(str(transition.trigger))
            transitions.append(Transition(from_, to, guard, trigger))

        raw = definition[2] if definition[2].tag == RAW else definition[3]
        if str(raw) != "":
            type_ = raw.attrib[TYPE]
            local = {FALSE: False, TRUE: True}[raw.attrib[LOCAL]]
            if not local:
                raw = StrFromFile(GetPrefix(defin, FSEP)+str(raw))
            raw_type = type_

        return LDefinition(templates, events, transitions,
                           Raw(raw_type, str(raw)))

    def __generateEffect(self, effect, definition):
        return EffectModel(effect.attrib[NAME], definition.getTemplates(),
                           str(effect.description), definition.getValues(),
                           definition.getEffects(), definition.getRaw())

    def __generateDynamic(self, dynamic, definition):
        return LocalDynamic(dynamic.attrib[NAME], str(dynamic.description),
                            definition.getTemplates(), definition.getEvents(),
                            definition.getTransitions(), definition.getRaw())
