def antlr4_parse_assoc_txt_file_to_internal_struct(filename):
    import antlr4

    from xsap.data_structure.tfpg.txt.grammars.generated4.Tfpg_associationsVisitor import Tfpg_associationsVisitor
    from xsap.data_structure.tfpg.tfpg_associations import TfpgAssociations

    class Visitor(Tfpg_associationsVisitor):
        def __init__(self):
            super(Visitor, self).__init__()
            self.tfpg_assoc = TfpgAssociations()


        def visitAssociations(self, ctx):
            fm = ctx.FAILURE_MODES()
            #print(fm.getText())

            failure_modes = ctx.failure_mode()
            for failure_mode in failure_modes:
                self.visitFailure_mode(failure_mode)

            md = ctx.MONITORED_DISCREPANCIES()
            #print(md.getText())

            monitored_discrepancies = ctx.monitored_discr()
            for monitored_discr in monitored_discrepancies:
                self.visitMonitored_discr(monitored_discr)

            ud = ctx.UNMONITORED_DISCREPANCIES()
            #print(ud.getText())

            unmonitored_discrepancies = ctx.unmonitored_discr()
            for unmonitored_discr in unmonitored_discrepancies:
                self.visitUnmonitored_discr(unmonitored_discr)

            tm = ctx.MODES()
            #print(tm.getText())

            tfpg_modes = ctx.tfpg_mode()
            for tfpg_mode in tfpg_modes:
                self.visitTfpg_mode(tfpg_mode)


        def visitFailure_mode(self, ctx):
            _id = ctx.IDENT()
            expr = ctx.SMV_EXPR()
            #print ("FM " + " IDENT " +_id.getText() + " " + expr.getText().strip())
            self.tfpg_assoc[_id.getText()] = ('OK', expr.getText().strip(), 'FailureMode')


        def visitMonitored_discr(self, ctx):
            _id = ctx.IDENT()
            expr = ctx.SMV_EXPR()
            #print ("MD " + _id.getText() + " " + expr.getText().strip())
            self.tfpg_assoc[_id.getText()] = ('OK', expr.getText().strip(), 'MonitoredDiscr')


        def visitUnmonitored_discr(self, ctx):
            _id = ctx.IDENT()
            expr = ctx.SMV_EXPR()
            #print ("UD " + _id.getText() + " " + expr.getText().strip())
            self.tfpg_assoc[_id.getText()] = ('OK', expr.getText().strip(), 'UnmonitoredDiscr')


        def visitTfpg_mode(self, ctx):
            _id = ctx.IDENT()
            expr = ctx.SMV_EXPR()
            #print ("TM " + " IDENT "+ _id.getText() + " " + expr.getText().strip())
            self.tfpg_assoc[_id.getText()] = ('OK', expr.getText().strip(), 'TfpgMode')

        def get_tfpg_assoc(self):
            return self.tfpg_assoc

    from xsap.data_structure.tfpg.txt.grammars.generated4 import Tfpg_associationsLexer, Tfpg_associationsParser

    char_stream = antlr4.FileStream(filename)
    lexer = Tfpg_associationsLexer.Tfpg_associationsLexer(char_stream)
    tokens = antlr4.CommonTokenStream(lexer)
    parser = Tfpg_associationsParser.Tfpg_associationsParser(tokens)

    visitor = Visitor()
    visitor.visit(parser.associations())

    tfpg_associations = visitor.get_tfpg_assoc()

    return tfpg_associations

def parse_assoc_txt_file_to_internal_struct(filename):
    tfpg_associations = None
    tfpg_associations = antlr4_parse_assoc_txt_file_to_internal_struct(filename)
    return tfpg_associations

def dump_associations_to_txt_file(tfpg_associations_struct, out_filename):
    """
    """

    out_file = open(out_filename, "w")

    failure_modes = [
        (_id,
         v[1]) for (
            _id,
            v) in tfpg_associations_struct.items() if v[2] == "FailureMode"]
    mon_discr = [
        (_id,
         v[1]) for (
            _id,
            v) in tfpg_associations_struct.items() if v[2] == "MonitoredDiscr"]
    unmon_discr = [
        (_id,
         v[1]) for (
            _id,
            v) in tfpg_associations_struct.items()
        if v[2] == "UnmonitoredDiscr"]
    tfpg_modes = [
        (_id,
         v[1]) for (
            _id,
            v) in tfpg_associations_struct.items() if v[2] == "TfpgMode"]

    out_file.write("FAILURE_MODES\n")
    for fm in failure_modes:
        out_file.write("%s %s\n" % (fm[0], fm[1]))
    out_file.write("MONITORED_DISCREPANCIES\n")
    for md in mon_discr:
        out_file.write("%s %s\n" % (md[0], md[1]))
    out_file.write("UNMONITORED_DISCREPANCIES\n")
    for ud in unmon_discr:
        out_file.write("%s %s\n" % (ud[0], ud[1]))
    out_file.write("MODES\n")
    for tm in tfpg_modes:
        out_file.write("%s %s\n" % (tm[0], tm[1]))

    out_file.close()
    return
