
# %HEADER


import re


def split_behval_completeness_tracefile(trace_filename):
    """This method takes a trace as generated by Xsap model checking for
    TFPG completeness analysis, and splits them into two traces: One
    being the system trace, and one being the corresponding trace over the
    TFPG alphabet violating the TFPG constraints.

    The filenames of the two new trace files are returned.

    @param trace_filename: the filename of the original trace
    @type trace_filename: string

    @return: the name of the system trace and the one of the tfpg trace
    @type: dict
    """

    # new trace files
    sys_trace_fn = trace_filename.replace('.xml', '-SYS.xml')
    tfpg_trace_fn = trace_filename.replace('.xml', '-TFPG.xml')
    sys_trace = open(sys_trace_fn, "w")
    tfpg_trace = open(tfpg_trace_fn, "w")

    # parse trace file line by line

    orig_trace = open(trace_filename, "r")

    while True:
        line = orig_trace.readline()
        if not line:
            break

        tfpg_line = False
        common_line = False

        tfpg_tokens = [
            "failuremode_",
            "or_node_",
            "and_node_",
            "ETime_",
            "EState_",
            "Mode_"]
        for t in tfpg_tokens:
            # TODO: solve problem of 'b_' prefix to avoid name clashes when
            # running behavioral validation on TFPG-SMV dumps as system
            # files. see also TODO note below and in
            # dump_extended_sys_as_smv_for_bv()
            match = re.search(r'^\s*<value variable="(b_)?%s' % t, line)
            if match:
                tfpg_line = True
                break

        common_line = not tfpg_line and "value" not in line

        if common_line:
            sys_trace.write(line)
            tfpg_trace.write(line)

        elif tfpg_line:

            # convert smv var to tfpg var
            if "failuremode_" in line:
                # TODO: see TODO node above.
                line = line.replace('b_failuremode_', '')
            elif "or_node_" in line:
                # TODO: see TODO node above.
                line = line.replace('b_or_node_', '')
            elif "and_node_" in line:
                # TODO: see TODO node above.
                line = line.replace('b_and_node_', '')
            elif "ETime_" in line:
                line = line.replace('ETime_', 'EDGE_CLOCK_')
            elif "EState_" in line:
                line = line.replace('EState_', 'EDGE_STATE_')
            elif "Mode_" in line:
                line = line.replace('Mode_', '')
            else:
                # there should be no other nodes in the tfpg trace variables
                assert False

            tfpg_trace.write(line)

        else:
            sys_trace.write(line)

    # done

    sys_trace.close()
    tfpg_trace.close()
    orig_trace.close()

    return {'sys_trace': sys_trace_fn, 'tfpg_trace': tfpg_trace_fn}
