from __future__ import print_function

from builtins import range
import sys

# INTERFACES

# TODO: function name??


def generate_diag_conditions_and_get_smv_prop(
        target_fm_list,
        tfpg_struct,
        diag_cond_fname,
        system_mode_var_name,
        target_system_mode):
    """Given a set of target FMs, a file containing the relative diagnosability
    conditions is created.
    Additionally, the smv property to be checked is returned.

    @param target_fm_list: the list of FMs that need to be diagnosed
                           (colon-separated)
    @type  target_fm_list: string

    @param tfpg_struct: the tfpg struct for which we want to generate the diag
                        conditions
    @type  tfpg_struct: data_structure.tfpg.tfpg.Tfpg

    @param diag_cond_fname: the name of the file to be created
    @type  diag_cond_fname: string

    @param system_mode_var_name: the name of the variable represeting the
                                 system mode
    @type  system_mode_var_name: string

    @param target_system_modee: the system mode for which effectiveness should
                                be analyzed
    @type  target_system_mode: string
    """

    # Extract fm list
    fm_list = target_fm_list.split(":")
    fm_names = [nd.get_name() for nd in tfpg_struct.get_fm_nodes()]
    for fm in fm_list:
        if fm not in fm_names:
            print("%s is not the name of a valid failure mode." % fm)
            sys.exit(1)
            return

    # used for the diagnosability conditions file
    target_fm_expression = "("
    # the property to be checked
    smv_prop = "("

    for i in range(0, len(fm_list)):

        target_fm_expression += ("failuremode_%s.is_active" % fm_list[i])
        smv_prop += ("twin_L.failuremode_%s.is_active" % fm_list[i])

        if i < (len(fm_list) - 1):
            target_fm_expression += " | "
            smv_prop += " | "
        else:
            target_fm_expression += ")"
            smv_prop += " ) "

    smv_prop = ("((G F !stutter_L) & (G F !stutter_R)) "
                "-> G ! (%s & G TRUE)" % smv_prop)

    system_mode_expression = None if target_system_mode is None else (
        "(%s = %s)" % (system_mode_var_name, target_system_mode))

    with open(diag_cond_fname, "w") as cond_out:

        cond_out.write("CONDITION_L %s\n" % target_fm_expression)
        cond_out.write("CONDITION_R TRUE\n")

        if system_mode_expression is not None:
            cond_out.write("CONTEXT_L %s\n" % system_mode_expression)
            cond_out.write("CONTEXT_R %s & !%s\n" %
                           (system_mode_expression, target_fm_expression))
        else:
            cond_out.write("CONTEXT_R !%s\n" % target_fm_expression)

    print("written %s" % diag_cond_fname)
    return smv_prop


# HELPERS
