from __future__ import print_function


# INTERFACES

def extract_and_dump_observables(
        tfpg_struct,
        obs_fname,
        time_tick_var_name,
        system_mode_var_name):
    """Extracts and dumps the observables of a given TFPG.

    @param tfpg_struct: the tfpg for which we want to get the observables
    @type  tfpg_struct: data_structure.tfpg.tfpg.Tfpg

    @param obs_fname: the name of the file where to dump observables
    @type  obs_fname: string

    @param time_tick_var_name: the name of the time tick variable
    @type  time_tick_var_name: string

    @param system_mode_var_name: the name of the system mode variable
    @type  system_mode_var_name: string
    """

    observables = tfpg_struct.get_monitored_discrepancies()
    write_obs_file(
        observables,
        obs_fname,
        time_tick_var_name,
        system_mode_var_name)
    return


# HELPERS

def write_obs_file(observables, obs_fname, time_tick, system_mode):
    """Dumps the observables to file

    @param observables: the list of observables to be dumped
    @type  observables: list of data_structure.tfpg.tfpg.Node

    @param obs_fname: the name of the file where to dump the observables
    @type  obs_fname: string

    @param time_tick: the name of the time tick variable
    @type  time_tick: string

    @param system_mode: the name of the system mode variable
    @type  system_mode: string
    """
    from xsap.features.common.tfpg_symb_converter import compute_smv_node_name

    f = open(obs_fname, 'w')

    for n in observables:
        f.write("%s.is_active\n" % compute_smv_node_name(n))

    f.write("%s\n" % time_tick)
    f.write("%s\n" % system_mode)

    f.close()

    print("written %s" % obs_fname)
    return
