

def split_effectiveness_tracefile(trace_filename):
    """This method takes a trace as generated by diagnosability analysis of
    xSAP, and splits them into two traces, hiding internal implementation
    details of xSAP. The filenames of the two new trace files are
    returned.

    @param trace_filename: the filename of the original trace
    @type  trace_filename: string
    """

    import xml.dom
    import xml.dom.minidom

    doc = xml.dom.minidom.parse(trace_filename)
    system_mode = doc.getElementsByTagName('value')[0]

    # new trace files

    trace_fn_1 = trace_filename.replace('.xml', '-A.xml')
    trace_fn_2 = trace_filename.replace('.xml', '-B.xml')
    trace_1 = open(trace_fn_1, "w")
    trace_2 = open(trace_fn_2, "w")

    # parse trace file line by line

    orig_trace = open(trace_filename, "r")

    while True:
        line = orig_trace.readline()
        if not line:
            break

        # skip lines with path monitors of restrictions
        if 'have_C' in line:
            continue

        # skip lines with edge variables
        if "edge_" in line:
            continue

        # can/must fire flags
        if "must_fire" in line or "can_fire" in line:
            continue

        # replace tfpg2smv translation artifacts from variable names
        # TODO: maybe use the features in tfpg_symb_converter?
        if "failuremode_" in line:
            line = line.replace('failuremode_', '')
        if "or_node_" in line:
            line = line.replace('or_node_', '')
        if "and_node_" in line:
            line = line.replace('and_node_', '')
        if ".is_active" in line:
            line = line.replace('.is_active', '')

        # LB# The ENUM values of system_mode and instant_trans_type are
        # converted in SMV Format as CONST_
        if "system_mode" in line:
            line = line.replace(
                ''.join([node.data for node in system_mode.childNodes]),
                "const_" + ''.join(
                    [node.data for node in system_mode.childNodes]))
        if "instant_trans_type" in line and "NODE_ACTIVATION" in line:
            line = line.replace('NODE_ACTIVATION', 'const_NODE_ACTIVATION')
        if "instant_trans_type" in line and "NONE" in line:
            line = line.replace('NONE', 'const_NONE')
        if "instant_trans_type" in line and "MODE_CHANGE" in line:
            line = line.replace('MODE_CHANGE', 'const_MODE_CHANGE')

        # redirect line to correct file (and drop twin tags)
        if line.count('twin_L.') > 0:
            trace_1.write(line.replace('twin_L.', 'root.sc_'))
        elif line.count('twin_R.') > 0:
            trace_2.write(line.replace('twin_R.', 'root.sc_'))
        else:
            trace_1.write(line)
            trace_2.write(line)

    # done

    trace_1.close()
    trace_2.close()
    orig_trace.close()

    return (trace_fn_1, trace_fn_2)
