
import os

from xsap.features.common.ProofObligationINVAR import ProofObligationINVAR
from xsap.features.common.tfpg_sys_extender import append_smv_extension


def generate_model_and_prop(fn_smv, t_delta_var, mtl_proof_obligation, outdir):
    """ Converts the MTL proof obligation to an invariant proof obligation, returns
    the respective property string, as well as the SMV model extended with symbols
    needed to evaluate the invariant.

    @param fn_smv: flat SMV model (manual time)
    @type  fn_smv: str

    @param t_delta_var: the t_delta variable in the time encoding of the SMV file
    @type  t_delta_var: str

    @param mtl_proof_obligation: the MTL proof obligation
    @type  mtl_proof_obligation: ProofObligationMTL

    @param outdir: the output directory for the SMV extension
    @type  outdir: str

    @return: extended SMV file, INVAR property corresponding to the given MTL property, and an error
             message which is None if no errors were encountered.
    @rtype:  tuple
    """

    # convert MTL proof obligation

    invar_proof_obligation = ProofObligationINVAR(mtl_proof_obligation)

    # extend SMV model

    extension_data = invar_proof_obligation.get_extension_data()

    fn_smv_base, _ = os.path.splitext(os.path.basename(fn_smv))
    fn_smv_ext = os.path.join(outdir, "%s_tightening_extended.smv" % fn_smv_base)

    res_msg = append_smv_extension(fn_smv, fn_smv_ext, extension_data, t_delta_var)

    if res_msg != "ok":
        return None, None, res_msg

    return fn_smv_ext, str(invar_proof_obligation), None
