
from builtins import object
from xsap.features.tfpg_tightening.tightening_engine_bisection.Parameter import Parameter


class ParameterManager(object):

    ############################################################################

    def __init__(self, dic_params, tmax_bound, step_size):
        self.parameters = {}
        self.initialize_parameters(dic_params, tmax_bound, step_size)

    ############################################################################

    def initialize_parameters(self, dic_params, tmax_bound, step_size):

        for p_name in dic_params:

            domain = dic_params[p_name]["type"]
            monotonicity = dic_params[p_name]["monotonicity"]
            tag = dic_params[p_name]["tag"]

            val_min = dic_params[p_name]["lower_bound"]
            assert val_min != "none"

            val_max = dic_params[p_name]["upper_bound"]

            if val_max == "none":
                assert tag in ("tmin", "tmax") and "bool" not in p_name
                val_max = tmax_bound

            if tag == "modes" or "tmax_bool" in p_name:
                step_delta = "none"
            else:
                step_delta = step_size

            param_string = "%s / %s / %s / %s / %s / %s / %s" % \
                           (p_name, domain, monotonicity, val_min, val_max, step_delta, tag)

            self.parameters[p_name] = Parameter(param_string)

    ############################################################################

    def get_param(self, param_name):
        if param_name in self.parameters:
            return self.parameters[param_name]
        else:
            return None

    ############################################################################

    def get_lattice_top(self):
        res = {}
        for p_name in self.parameters:
            if self.parameters[p_name].has_top_val():
                res[p_name] = self.parameters[p_name].get_top_val()
        return res

    ############################################################################

    def get_filtered(self, tag):
        if tag is None:
            return list(self.parameters.keys())
        else:
            return [p_name for p_name in self.parameters
                    if self.parameters[p_name].tag == tag]
