from __future__ import division

from builtins import object
from past.utils import old_div
from math import floor


class SearchBoundManager(object):

    def __init__(self, params_mgr, ls_params):
        assert len(ls_params) > 0

        self.search_bounds = {}
        self.params_mgr = params_mgr

        for p_name in ls_params:
            p_obj = self.params_mgr.get_param(p_name)
            self.search_bounds[p_name] = p_obj.get_range()
            self.range_is_consistent(p_name)

    def get_range_string(self, p_name):
        min_str = self.search_bounds[p_name]['min']
        min_str = "None" if min_str is None else str(min_str)
        max_str = self.search_bounds[p_name]['max']
        max_str = "None" if max_str is None else str(max_str)
        return "[%s,%s]" % (min_str, max_str)

    def min_val_is_none(self, p_name):
        return self.search_bounds[p_name]['min'] is None

    def max_val_is_none(self, p_name):
        return self.search_bounds[p_name]['max'] is None

    def one_limit_is_none(self, p_name):
        return self.min_val_is_none(p_name) or self.max_val_is_none(p_name)

    def val_in_range(self, p_name, val):
        if self.one_limit_is_none(p_name): return True
        return self.search_bounds[p_name]['min'] <= val <= self.search_bounds[p_name]['max']

    def range_is_consistent(self, p_name):
        if self.one_limit_is_none(p_name): return True
        return 0 <= self.search_bounds[p_name]['min'] <= self.search_bounds[p_name]['max']

    def range_is_singular(self, p_name):
        if self.one_limit_is_none(p_name): return False
        return str(self.search_bounds[p_name]['min']) == str(self.search_bounds[p_name]['max'])

    def get_middle_val(self, p_name):

        rmin = self.search_bounds[p_name]['min']
        rmax = self.search_bounds[p_name]['max']

        step_delta = self.params_mgr.get_param(p_name).get_step_delta()
        if step_delta < 1:
            rmin = old_div(rmin, step_delta)
            rmax = old_div(rmax, step_delta)

        half_delta = int(floor((rmax - rmin) / 2.0))
        if step_delta < 1:
            new_val = (rmin + half_delta) * step_delta
            rmin = rmin * step_delta
            rmax = rmax * step_delta
        else:
            new_val = rmin + half_delta

        assert rmin <= new_val <= rmax
        #print("NEW_VAL " + str(new_val))
        return round(new_val, 1)

    def update_w_good_val(self, p_name, good_val):
        assert p_name in self.search_bounds
        assert self.val_in_range(p_name, good_val)

        p_obj = self.params_mgr.get_param(p_name)
        if p_obj.is_upward_monotonic():
            # lower is better -> good value is new upper bound
            self.search_bounds[p_name]['max'] = good_val
        else:
            # higher is better -> good value is new lower bound
            self.search_bounds[p_name]['min'] = good_val

        assert self.val_in_range(p_name, good_val)
        assert self.range_is_consistent(p_name)

    def update_w_bad_val(self, p_name, bad_val):
        assert p_name in self.search_bounds
        assert self.val_in_range(p_name, bad_val)

        if not self.range_is_singular(p_name):

            p_obj = self.params_mgr.get_param(p_name)
            if p_obj.is_upward_monotonic():
                # lower is better -> bad value +1 is new lower bound
                self.search_bounds[p_name]['min'] = bad_val + p_obj.get_step_delta()
            else:
                # higher is better -> bad value -1 is new upper bound
                self.search_bounds[p_name]['max'] = bad_val - p_obj.get_step_delta()

        assert self.range_is_consistent(p_name)
        assert not self.val_in_range(p_name, bad_val)

    def get_search_val(self, p_name):
        assert not self.min_val_is_none(p_name)

        if self.max_val_is_none(p_name):
            if self.search_bounds[p_name]['min'] == 0:
                return 1
            else:
                return self.search_bounds[p_name]['min'] * 2
        else:
            return self.get_middle_val(p_name)

