from __future__ import print_function
# %HEADER

from xsap.features.tfpg_tightening.tightening_engine_bisection.ParamSynthesizer import ParamSynthesizer


def do_tighening_bisection(fn_vmt, options, dic_params, tmax_bound, step_size, silent=False):

    # set ic3 mode

    if contains_term_ites(fn_vmt):
        ic3_mode = "ia"
        if "engine" in options and options["engine"] == "conc":
            print("  > switching IC3-mode to IA instead of CONC due to model features")
    elif "engine" not in options:
        ic3_mode = "conc"
    else:
        assert options["engine"] in ("conc", "ia")
        ic3_mode = options["engine"]

    # update tags if necessary

    assert len(options["tags"]) > 0
    for tag in options["tags"]:
        assert tag in ('tmin', 'tmax', 'modes')

    untimed = options["t_delta_var"] is None

    if untimed and "modes" not in options["tags"]:
        return None, "nothing to be tightened (t_delta not given and modes not in selected tags)"

    if untimed:
        # doesn't make sense to tighten tmin/tmax when t_delta is not given
        options["tags"] = ["modes"]

    # compute other parameters

    bmc_len = options["bmc_len"] if "bmc_len" in options else 10
    ic3_len = options["ic3_len"] if "ic3_len" in options else None
    skip_bmc = options["skip_bmc"] if "skip_bmc" in options else False

    # call synthesis procedure

    msat_dic_params = param_names_to_msat(dic_params)

    psynth = ParamSynthesizer(fn_vmt, msat_dic_params, 'reuse-invariant', bmc_len,
                              ic3_len, ic3_mode, skip_bmc, tmax_bound, step_size)

    res = psynth.tfpg_find_one_optimal(options["tags"], untimed, silent=silent)

    if res['result'] != 'ok':
        return None, res['message']

    return param_names_to_smv(res['candidate']), None


def contains_term_ites(fn_vmt):
    have_ite = False
    f = open(fn_vmt, 'r')
    for line in f:
        if '(ite ' in line:
            have_ite = True
            break
    f.close()
    return have_ite


def param_names_to_msat(dic_params):
    res = {}
    for k in dic_params:
        assert not k.endswith("__AT0")
        res[k + "__AT0"] = dic_params[k]
    return res


def param_names_to_smv(dic_params):
    res = {}
    for k in dic_params:
        assert k.endswith("__AT0")
        res[k[:k.find("__AT0")]] = dic_params[k]
    return res
