
from xsap.features.tfpg_tightening.generate_problem_invar import generate_model_and_prop
from xsap.features.tfpg_tightening.tightening_engine_bisection.perform_tightening \
    import perform_tightening_invar


def tighten_tfpg_manualtime(fn_smv, po_mtl, options, dic_params, tfpg_struct):
    """ Reduces the tightening problem from MTL to INVAR and calls the manual-time TFPG tightening.

    @param fn_smv: SMV file representing the system the TFPG refers to (manual time encoding)
    @type  fn_smv: str

    @param po_mtl: MTL proof obligation
    @type  po_mtl: ProofObligationMTL

    @param options: various tightening options
    @type  options: dict

    @param dic_params: parameter dictionary generated by generate_parameters_dictionary()
    @type  dic_params: dict

    @param tfpg_struct: the TFPG to be tightened
    @type  tfpg_struct: Tfpg

    @return: returns a tuple where the first element is a dictionary with the resulting
             parameter values if everything went well and None otherwise, and the second
             element is an error message in case errors occurred.
    @rtype:  tuple
    """

    t_delta_var = options["t_delta_var"]
    outdir = options["outdir"]

    # generate problem
    fn_smv_ext, po_invar, err_msg = generate_model_and_prop(fn_smv, t_delta_var, po_mtl, outdir)

    if err_msg is not None:
        return None, err_msg

    # call tightening engine
    return perform_tightening_invar(fn_smv_ext, po_invar, options, dic_params, tfpg_struct)
