
from xsap.features.tfpg_tightening.tightening_manual_time import tighten_tfpg_manualtime


def tighten_tfpg_nativetime(fn_smv, po_mtl, options, dic_params, tfpg_struct):
    """ Untimes the model (conversion from nuXmv native time to manual time encoding),
    then calls the manual-time TFPG tightening.

    @param fn_smv: SMV file representing the system the TFPG refers to
    @type  fn_smv: str

    @param po_mtl: MTL proof obligation
    @type  po_mtl: ProofObligationMTL

    @param options: various tightening options
    @type  options: dict

    @param dic_params: parameter dictionary generated by generate_parameters_dictionary()
    @type  dic_params: dict

    @param tfpg_struct: the TFPG to be tightened
    @type  tfpg_struct: Tfpg

    @return: returns a tuple where the first element is a dictionary with the resulting
             parameter values if everything went well and None otherwise, and the second
             element is an error message in case errors occurred.
    @rtype:  tuple
    """

    # untime model
    fn_untimed_smv, t_delta_var = untime_model(fn_smv)

    if fn_untimed_smv is None:
        return None, "model untiming failed"

    # set t_delta in options
    options["t_delta_var"] = t_delta_var

    # call manual time tightener
    return tighten_tfpg_manualtime(fn_untimed_smv, po_mtl, options, dic_params, tfpg_struct)


def untime_model(fn_smv):
    """ Converts a nuXmv native-time model to manual time.

    @param fn_smv: the native-time SMV file
    @type  fn_smv: str

    @return: a tuple where the first element is the untimed SMV file and the second element is the
             t_delta symbol that was introduced.
    @rtype:  tuple
    """

    raise NotImplementedError

    fn_untimed_smv = fn_smv
    t_delta_var = None

    return fn_untimed_smv, t_delta_var
