/* ---------------------------------------------------------------------------
  This file is part of the ``shell'' package.
  %COPYRIGHT%
-----------------------------------------------------------------------------*/

/*!
  \author Samuele Gallerani
  \brief It contains an array with the couple command_name and command_description

  This source file contains an empty array, with cmake it is populated with the actual shell commands and relative description.

*/


#include <stdio.h>
#include <stdlib.h>

#include "xsap/shell/shell.h"
#include "nusmv/shell/cmd/cmdCmd.h"
#include "nusmv/core/cinit/NuSMVEnv.h"


/*---------------------------------------------------------------------------*/
/* Constant declarations                                                     */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* Type declarations                                                         */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* Structure declarations                                                    */
/*---------------------------------------------------------------------------*/

/* this is computed by scripts/create_help.py -c */
command_item xsap_command_help[] = {
"fe_load_doc", "COMMAND= fe_load_doc   Loads the fault extension specification XML\n"
"file. \n"
"usage: fe_load_doc [-h] [-o errors-fname] [-F txt|xml] [-c] [-p\n"
"dtd-path] -i extension-file.xml\n"
" Reads the given\n"
"extension-file.xml containing the specification of the fault\n"
"extension, and fills internal structures to prepare the extension.\n"
"It performs sintactic and semantic checks, and report checking\n"
"errors and warnings.\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -i extension-file.xml\n"
"\n"
"    Reads the specification from given xml file.\n"
"\n"
"    -o errors-fname\n"
"\n"
"    Writes errors and warnings into given file name.\n"
"\n"
"    -F txt|xml\n"
"\n"
"    output format of error reporting.\n"
"\n"
"    -c\n"
"\n"
"    Disables semantic checking (use only if you know what you are\n"
"    doing)\n"
"\n"
"    -p dtd-path\n"
"\n"
"    Path to the DTD schema of the input xml specification. Default\n"
"    value is expected to work fine.\n"
"\n"
"",
"_fe_print_doc", "COMMAND= _fe_print_doc   Internal command for printing internal\n"
"representation of the fault extension structures. \n"
"usage: _fe_print_doc \n"
"\n"
"Used for debugging only.",
"fe_extend_module", "COMMAND= fe_extend_module   Performs the automatic fault\n"
"extension of the nominal model, according to the previously loaded\n"
"specification. \n"
"usage: fe_extend_module [-h] [-A] [-c] [-m fm-file.xml] -o\n"
"extended-model-file.smv\n"
" By using the information previously\n"
"loaded with command fe_load_doc, the command performs the automatic\n"
"fault extension of the currently loaded model, and dumps the extended\n"
"model to a new smv file. Optionally, it creates also an XML file\n"
"containing information about Fault Modes and Common Causes, with the\n"
"possible associated information about fault probability. Notice\n"
"that at the moment the currently loaded FSM remains untouched,\n"
"meaning that if you need to work on the extended model, you will have to\n"
"reset and load it explicitly with read_model.\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -o extended-model-file.smv\n"
"\n"
"    Dumps the extended model to the given filename.\n"
"\n"
"    -m fm-file.xml\n"
"\n"
"    Dumps fault modes and Common Causes information into the given\n"
"    XML file. This file is requested by most of SA analysis.\n"
"\n"
"    -A\n"
"\n"
"    Anonymizes the output (use e.g. with NDA).\n"
"\n"
"    -c\n"
"\n"
"    Suppresses the generation of the encoding of Common Causes (use\n"
"    for debugging, or when Common Causes are not interesting for the\n"
"    analysis)\n"
"\n"
"",
"synth_set_observation_semantics", "COMMAND= synth_set_observation_semantics   Defines the\n"
"observation semantics in the specification\n"
"\n"
"usage: synth_set_observation_semantics [-c|-n|-b]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -c current state\n"
"\n"
"    -x next state\n"
"\n"
"    -b both (current and next state\n"
"\n"
"",
"synth_edit_controllables", "COMMAND= synth_edit_controllables   Manages the list of\n"
"controllables \n"
"usage: synth_edit_controllables [-h] [-l | [[ -a | -r ] \"var\"]]\n"
"\n"
"Manages the list of controllables/actions, performing addition and\n"
"removal of controllables/actions Command options:\n"
"\n"
"    \n"
"\n"
"    -a\n"
"\n"
"    Adds a controllable (action)\n"
"\n"
"    -r\n"
"\n"
"    Removes a controllalbe (action)\n"
"\n"
"    -l\n"
"\n"
"    Lists actions\n"
"\n"
"",
"synth_edit_plans_spec", "COMMAND= synth_edit_plans_spec   Edits the set of plans\n"
"specifications \n"
"usage: synth_edit_plans_spec [-a | -r | -l] [-n ] [-i ] [-g ] [-t ] [-c ]\n"
"[-m ]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -r\n"
"\n"
"    Remove\n"
"\n"
"    -a\n"
"\n"
"    Add\n"
"\n"
"    -l\n"
"\n"
"    list\n"
"\n"
"    -n plan_name\n"
"\n"
"    Plan identifier\n"
"\n"
"    -i init_expr\n"
"\n"
"    Init condition\n"
"\n"
"    -g goal_expr\n"
"\n"
"    Goal condition\n"
"\n"
"    -t trigger_expr\n"
"\n"
"    Trigger condition,\n"
"\n"
"    -c constraint_expr\n"
"\n"
"    FSM restriction constraint (INVAR)\n"
"\n"
"    -m alarm_name\n"
"\n"
"    Alarm label\n"
"\n"
"",
"synth_build_plan", "COMMAND= synth_build_plan   Generates a strong contingent plan for\n"
"the alarm \n"
"usage: synth_build_plan [-h ] -n [-o ]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -n alarm_name\n"
"\n"
"    The plan to be built\n"
"\n"
"    -o file\n"
"\n"
"    The file on which to print, default is stdout.\n"
"\n"
"",
"synth_dump_plan", "COMMAND= synth_dump_plan   Outputs the plan \n"
"usage: synth_dump_plan [-n ]\n"
"[-o ] [-f {tree,smv\n"
"]} Generates a plan for the given alarm, that\n"
"starting from the initial condition reaches the goal. If such plan\n"
"does not exists an error is provided. Command options:\n"
"\n"
"    \n"
"\n"
"    -n\n"
"\n"
"    -o\n"
"\n"
"    The file on which to print, default is stdout\n"
"\n"
"    -f {tree,smv}\n"
"\n"
"",
"synth_build_recovery", "COMMAND= synth_build_recovery   Generates an automaton for the\n"
"given plan \n"
"usage: synth_build_recovery [-h] -n \n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -n alarm_name\n"
"\n"
"    The plan to be converted to an automaton\n"
"\n"
"",
"synth_FD", "COMMAND= synth_FD   Launches the FD synthesis process\n"
"\n"
"usage: synth_FD [-h] [-o] [-f]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -o file\n"
"\n"
"    File containing list of observables variables\n"
"\n"
"    <-f file\n"
"\n"
"    File containing the ASL specification\n"
"\n"
"",
"synth_FR", "COMMAND= synth_FR   Launches the FR synthesis process\n"
"\n"
"usage: synth_FR [-h]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"",
"synth_dump_fdir", "COMMAND= synth_dump_fdir   Dumps the Synthesised FDIR\n"
"\n"
"usage: synth_dump_fdir [-o ] [-c] [-s] [-v]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -o\n"
"\n"
"    -c\n"
"\n"
"    Dumps the FDIR combined within the original FSM\n"
"\n"
"    -s\n"
"\n"
"    Prints some additional statistics\n"
"\n"
"    -v\n"
"\n"
"    Disable BDD and DAG simplifications (verbose printing)\n"
"\n"
"",
"show_fault_variables", "COMMAND= show_fault_variables   \n"
"usage: show_fault_variables [-h] [-m| -o file]\n"
"[-v]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the command to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX command \"more\".\n"
"\n"
"    -o file\n"
"\n"
"    Writes the output generated by the command to the file file.\n"
"\n"
"    -v\n"
"\n"
"    Prints verbosely\n"
"\n"
"",
"compute_fault_tree", "COMMAND= compute_fault_tree   Computes minimal cut sets for a top\n"
"level event given as an LTL expression \n"
"usage: compute_fault_tree [-h] [-m | -o\n"
"output-file] [-t] [-e] [-p] [-x prefix_string] [-P | -n | ltl-expr]\n"
"\n"
"The output is the description of failure states corresponding to\n"
"minimal cut sets. Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the program to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX \"more\" command.\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -t\n"
"\n"
"    Computes a dynamic fault tree\n"
"\n"
"    -t\n"
"\n"
"    Computes a dynamic fault tree\n"
"\n"
"    -t\n"
"\n"
"    Generates fault tree corresponding to the minimal cut sets\n"
"\n"
"    -N\n"
"\n"
"    Bounds the number of faults (integer number)\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability.\n"
"\n"
"    -x prefix_string\n"
"\n"
"    Prefixes all generated file names by the string prefix_string.\n"
"\n"
"    -n index\n"
"\n"
"    Takes INVARSPEC or LTLSPEC property (given its index) to be used\n"
"    as !TLE .\n"
"\n"
"    -P\n"
"\n"
"    Takes INVARSPEC or LTLSPEC property (given its name) to be used as\n"
"    !TLE .\n"
"\n"
"",
"compute_fault_tree_param_klive", "COMMAND= compute_fault_tree_param_klive   \n"
"usage: compute_fault_tree_param_klive [-h]\n"
"[-m| -o file] [-t [-Q] -I] [-e] [-p [-S]] [-N NR_FAIL] [-x\n"
"\"prefix_string\"] [-i] [-L] [-A] [-n index | -P name |\n"
"\"ltl-expr\"]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes output through the program specified by the \"PAGER\"\n"
"    shell variable if defined, else through UNIX \"more\"\n"
"\n"
"    -o file\n"
"\n"
"    Writes the generated output to \"file\"\n"
"\n"
"    -t\n"
"\n"
"    Generates fault tree\n"
"\n"
"    -Q\n"
"\n"
"    Disables construction of ordered FT and computes probability\n"
"    only for the top level event (to speedup computation)\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability\n"
"\n"
"    -S\n"
"\n"
"    Computes probability also in symbolic form (python and\n"
"    octave/matlab)\n"
"\n"
"    -x prefix\n"
"\n"
"    Prefixes generated file names with \"prefix\"\n"
"\n"
"    -i\n"
"\n"
"    Forces the use of SMT for finite models\n"
"\n"
"    -L\n"
"\n"
"    Disable layering\n"
"\n"
"    -A\n"
"\n"
"    Force the computing of upper and lower probability bounds\n"
"\n"
"    -n index\n"
"\n"
"    Use given INVARSPEC or LTLSPEC property as !TLE\n"
"\n"
"    -P name\n"
"\n"
"    Use given INVARSPEC or LTLSPEC property (from name) as !TLE\n"
"\n"
"    ltl-expr\n"
"\n"
"    Use given expression as TLE\n"
"\n"
"",
"compute_prime_implicants_param", "COMMAND= compute_prime_implicants_param   \n"
"usage: compute_prime_implicants_param [-m|\n"
"-o file] [-t [-Q]] [-e] [-p [-S]] [-x prefix_string] [-i] [-L] [-n\n"
"index | -P name | next-expr\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes output through the program specified by the PAGER shell\n"
"    variable if defined, through UNIX more\n"
"\n"
"    -o\n"
"\n"
"    Writes the generated output to file\n"
"\n"
"    -t\n"
"\n"
"    Generates fault tree\n"
"\n"
"    -Q\n"
"\n"
"    Disables construction of ordered FT and computes probability\n"
"    only for the top level event (to speedup computation)\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability\n"
"\n"
"    -S\n"
"\n"
"    Computes probability also in symbolic form (python and\n"
"    octave/matlab)\n"
"\n"
"    -x\n"
"\n"
"    Prefixes generated file names with prefix\n"
"\n"
"    -i\n"
"\n"
"    Forces the use of SMT for finite models\n"
"\n"
"    -L\n"
"\n"
"    Disable layering\n"
"\n"
"    -n\n"
"\n"
"    Use given LTL property as !TLE\n"
"\n"
"    -P\n"
"\n"
"    Use given LTL property (from name) as !TLE\n"
"\n"
"    next-expr\n"
"\n"
"    Use given expression as TLE",
"compute_mode_transition_cut_sets", "COMMAND= compute_mode_transition_cut_sets  \n"
"\n"
"usage: compute_mode_transition_cut_sets [-h] [-v] [-e] [-M] [-s] [-i] [-L] [-t] [-o format]\n"
"[-g] [-x prefix] expr_1 ... expr_n\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -h\n"
"\n"
"    Prints the command usage\n"
"\n"
"    -v\n"
"\n"
"    Use provided expressions as variables. The states are computed\n"
"    as all evaluations of the variables\n"
"\n"
"    -e\n"
"\n"
"    Compute only transitions from the first mode to the others\n"
"\n"
"    -M\n"
"\n"
"    Turn off monotonicity\n"
"\n"
"    -s\n"
"\n"
"    Use strict MTCS computation\n"
"\n"
"    -i\n"
"\n"
"    Forces the use of SMT for finite models\n"
"\n"
"    -L\n"
"\n"
"    Disable layering\n"
"\n"
"    -t\n"
"\n"
"    Print XML output in file\n"
"\n"
"    -o format\n"
"\n"
"    Print visual output in file (dot, tex)\n"
"\n"
"    -g\n"
"\n"
"    Paging, print each mode transition in separate file\n"
"\n"
"    -x prefix\n"
"\n"
"    Prefixes generated file names with \"prefix\"",
"compute_prime_implicants", "COMMAND= compute_prime_implicants   \n"
"usage: compute_prime_implicants [-h] [-m| -o\n"
"file] [-t] [-e] [-p [-S]] [-x \"prefix_string\"] [-n index | -P name |\n"
"\"next-expr\"]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the command to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX command \"more\".\n"
"\n"
"    -o file\n"
"\n"
"    Writes the output generated by the command to the file file.\n"
"\n"
"    -t\n"
"\n"
"    Generates fault tree\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexamples traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability\n"
"\n"
"    -S\n"
"\n"
"    Computes probability also in symbolic form (python and\n"
"    octave/matlab)\n"
"\n"
"    -x prefix\n"
"\n"
"    Prefixes generated file name with \"prefix\"\n"
"\n"
"    -n index\n"
"\n"
"    Use diven LTL property as !TLE\n"
"\n"
"    -P name\n"
"\n"
"    Use given LTL property (from name) as !TLE\n"
"\n"
"    \"next-expr\"\n"
"\n"
"    Use given expression as TLE\n"
"\n"
"Options -n, -P and \"next-expr\" are mutually exclusive.\n\"); If\n"
"none is specified, all INVAR properties will be used to generate\n\");\n"
"multiple fault trees. In this case, the index of each property will\n"
"\n\"); appear in the prefix of generated files.\n\n\"); Also notice\n"
"that -n and -P take a property which is supposed to\n\"); hold in the\n"
"nominal model, i.e. that describes a good behaviour\n\"); (\"the\n"
"airplane can always fly\"). Instead the expression in\n"
"\"next-expr\"\n\"); represents a bad state, a failure TLE.\n\");",
"compute_fault_tree_bmc", "COMMAND= compute_fault_tree_bmc   \n"
"usage: compute_fault_tree_bmc [-m| -o file]\n"
"[-k length] [-l loopback] [-T mcs] [-N NR_FAIL] [-d] [-t [-Q]] [-e] [-p\n"
"[-S]] [-x \"prefix_string\"] [-n index | -P name | \"next-expr]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes output through the program specified by the PAGER shell\n"
"    variable if defined, else through UNIX more\n"
"\n"
"    -o\n"
"\n"
"    Writes the generated output to file\n"
"\n"
"    -k\n"
"\n"
"    Set problem length to length\n"
"\n"
"    -l\n"
"\n"
"    Set loopback value to loop\n"
"\n"
"    -T\n"
"\n"
"    Limit number of cut sets to be computed to mcs\n"
"\n"
"    -N NR_FAIL\n"
"\n"
"    limit number of possible failures to NR_FAIL\n"
"\n"
"    -d\n"
"\n"
"    Generates a dynamic fault tree\n"
"\n"
"    -t\n"
"\n"
"    tGenerates fault tree\n"
"\n"
"    -Q\n"
"\n"
"    Disables construction of ordered FT and computes probability\n"
"    only for the top level event (to speedup computation\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability\n"
"\n"
"    -S\n"
"\n"
"    Computes probability also in symbolic form (python and\n"
"    octave/matlab)\n"
"\n"
"    -x\n"
"\n"
"    Prefixes generated file names with prefix\n"
"\n"
"    -n\n"
"\n"
"    Use given INVARSPEC or LTLSPEC property as !TLE\n"
"\n"
"    -P\n"
"\n"
"    Use given INVARSPEC or LTLSPEC property (from name) as !TLE\n"
"\n"
"    next-expr\n"
"\n"
"    Use given expression as TLE\n"
"\n"
"",
"compute_fault_tree_bmc_inc", "COMMAND= compute_fault_tree_bmc_inc   \n"
"usage: compute_fault_tree_bmc_inc [-m| -o\n"
"file] [-k length] [-l loopback] [-T mcs] [-N NR_FAIL] [-d] [-t [-Q]]\n"
"[-e] [-p [-S]] [-x prefix_string] [-n index | -P name | next-expr]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes output through the program specified by the PAGER shell\n"
"    variable if defined, else through UNIX more\n"
"\n"
"    -o\n"
"\n"
"    Writes the generated output to file\n"
"\n"
"    -k\n"
"\n"
"    Set problem length to length\n"
"\n"
"    -l\n"
"\n"
"    Set loopback value to loop\n"
"\n"
"    -T\n"
"\n"
"    Limit number of cut sets to be computed to mcs\n"
"\n"
"    -N NR_FAIL\n"
"\n"
"    limit number of possible failures to NR_FAIL\n"
"\n"
"    -d\n"
"\n"
"    Generates a dynamic fault tree\n"
"\n"
"    -t\n"
"\n"
"    tGenerates fault tree\n"
"\n"
"    -Q\n"
"\n"
"    Disables construction of ordered FT and computes probability\n"
"    only for the top level event (to speedup computation\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability\n"
"\n"
"    -S\n"
"\n"
"    Computes probability also in symbolic form (python and\n"
"    octave/matlab)\n"
"\n"
"    -x\n"
"\n"
"    Prefixes generated file names with prefix\n"
"\n"
"    -n\n"
"\n"
"    Use given INVARSPEC or LTLSPEC property as !TLE\n"
"\n"
"    -P\n"
"\n"
"    Use given INVARSPEC or LTLSPEC property (from name) as !TLE\n"
"\n"
"    next-expr\n"
"\n"
"    Use given expression as TLE\n"
"\n"
"",
"compute_fault_tree_sbmc_inc", "COMMAND= compute_fault_tree_sbmc_inc   \n"
"usage: compute_fault_tree_sbmc_inc [-m| -o\n"
"file] [-k length] [-T mcs] [-N NR_FAIL] [-d] [-t [-Q]] [-e] [-p [-S]]\n"
"[-V] [-B] [-c] [-x prefix_string] [-n index | -P name | next-expr]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes output through the program specified by the PAGER shell\n"
"    variable if defined, else through UNIX more\n"
"\n"
"    -o\n"
"\n"
"    Writes the generated output to file\n"
"\n"
"    -k\n"
"\n"
"    Set problem length to length\n"
"\n"
"    -T\n"
"\n"
"    Limit number of cut sets to be computed to mcs\n"
"\n"
"    -N NR_FAIL\n"
"\n"
"    limit number of possible failures to NR_FAIL\n"
"\n"
"    -d\n"
"\n"
"    Generates a dynamic fault tree\n"
"\n"
"    -t\n"
"\n"
"    tGenerates fault tree\n"
"\n"
"    -Q\n"
"\n"
"    Disables construction of ordered FT and computes probability\n"
"    only for the top level event (to speedup computation\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability\n"
"\n"
"    -S\n"
"\n"
"    Computes probability also in symbolic form (python and\n"
"    octave/matlab)\n"
"\n"
"    -V\n"
"\n"
"    Does not perform virtual unrolling\n"
"\n"
"    -B\n"
"\n"
"    Adds blocking clauses at all time steps\n"
"\n"
"    -c\n"
"\n"
"    Performs completeness check\n"
"\n"
"    -x\n"
"\n"
"    Prefixes generated file names with prefix\n"
"\n"
"    -n\n"
"\n"
"    Use given INVARSPEC or LTLSPEC property as !TLE\n"
"\n"
"    -P\n"
"\n"
"    Use given INVARSPEC or LTLSPEC property (from name) as !TLE\n"
"\n"
"    next-expr\n"
"\n"
"    Use given expression as TLE\n"
"\n"
"",
"compute_fault_tree_bmc_bdd", "COMMAND= compute_fault_tree_bmc_bdd   \n"
"usage: compute_fault_tree_bmc_bdd [-h] [-m|\n"
"-o file] [-k length] [-l loopback] [-T mcs] [-N NR_FAIL] [-d] [-t [-Q]]\n"
"[-e] [-p [-S]] [-x prefix_string] [-n index | -P name | next-expr]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes output through the program specified by the \"PAGER\"\n"
"    shell variable if defined, else through UNIX \"more\"\n"
"\n"
"    -o file\n"
"\n"
"    Writes the generated output to \"file\"\n"
"\n"
"    -k length\n"
"\n"
"    set problem length to length\n"
"\n"
"    -l loop\n"
"\n"
"    set loopback value to loop\n"
"\n"
"    -T mcs\n"
"\n"
"    limit number of cut sets to be computed to mcs\n"
"\n"
"    -N NR_FAIL\n"
"\n"
"    limit number of possible failures to NR_FAIL\n"
"\n"
"    -d\n"
"\n"
"    Generates a dynamic fault tree\n"
"\n"
"    -t\n"
"\n"
"    Generates fault tree\n"
"\n"
"    -Q\n"
"\n"
"    Disables construction of ordered FT and computes probability\n"
"    only for the top level event (to speedup computation)\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability\n"
"\n"
"    -S\n"
"\n"
"    Computes probability also in symbolic form (python and\n"
"    octave/matlab)\n"
"\n"
"    -x prefix\n"
"\n"
"    Prefixes generated file names with \"prefix\"\n"
"\n"
"    -n index\n"
"\n"
"    Use given INVARSPEC or LTLSPEC property as !TLE\n"
"\n"
"    -P name\n"
"\n"
"    Use given INVARSPEC or LTLSPEC property (from name) as !TLE\n"
"\n"
"    next-expr\n"
"\n"
"    Use given expression as TLE\n"
"\n"
"",
"compute_fault_tree_bmc_inc_bdd", "COMMAND= compute_fault_tree_bmc_inc_bdd   \n"
"usage: compute_fault_tree_bmc_inc_bdd [-h]\n"
"[-m| -o file] [-k length] [-l loopback] [-T mcs] [-N NR_FAIL] [-d] [-t\n"
"[-Q]] [-e] [-p [-S]] [-x prefix_string] [-n index | -P name |\n"
"next-expr]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes output through the program specified by the \"PAGER\"\n"
"    shell variable if defined, else through UNIX \"more\"\n"
"\n"
"    -o file\n"
"\n"
"    Writes the generated output to \"file\"\n"
"\n"
"    -k length\n"
"\n"
"    set problem length to length\n"
"\n"
"    -l loop\n"
"\n"
"    set loopback value to loop\n"
"\n"
"    -T mcs\n"
"\n"
"    limit number of cut sets to be computed to mcs\n"
"\n"
"    -N NR_FAIL\n"
"\n"
"    limit number of possible failures to NR_FAIL\n"
"\n"
"    -d\n"
"\n"
"    Generates a dynamic fault tree\n"
"\n"
"    -t\n"
"\n"
"    Generates fault tree\n"
"\n"
"    -Q\n"
"\n"
"    Disables construction of ordered FT and computes probability\n"
"    only for the top level event (to speedup computation)\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability\n"
"\n"
"    -S\n"
"\n"
"    Computes probability also in symbolic form (python and\n"
"    octave/matlab)\n"
"\n"
"    -x prefix\n"
"\n"
"    Prefixes generated file names with \"prefix\"\n"
"\n"
"    -n index\n"
"\n"
"    Use given INVARSPEC or LTLSPEC property as !TLE\n"
"\n"
"    -P name\n"
"\n"
"    Use given INVARSPEC or LTLSPEC property (from name) as !TLE\n"
"\n"
"    next-expr\n"
"\n"
"    Use given expression as TLE\n"
"\n"
"",
"compute_fault_tree_sbmc_inc_bdd", "COMMAND= compute_fault_tree_sbmc_inc_bdd   \n"
"usage: compute_fault_tree_sbmc_inc_bdd [-h]\n"
"[-m | -o file] [-k length] [-T mcs] [-N NR_FAIL] [-d] [-t [-Q]] [-e] [-p\n"
"[-S]] [-V] [-B] [-c] [-x prefix-string] [-n index | -P name |\n"
"next-expr]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes output through the program specified by the \"PAGER\"\n"
"    shell variable if defined, else through UNIX \"more\"\n"
"\n"
"    -o file\n"
"\n"
"    Writes the generated output to \"file\"\n"
"\n"
"    -k length\n"
"\n"
"    set problem length to length\n"
"\n"
"    -T mcs\n"
"\n"
"    limit number of cut sets to be computed to mcs\n"
"\n"
"    -N NR_FAIL\n"
"\n"
"    limit number of possible failures to NR_FAIL\n"
"\n"
"    -d\n"
"\n"
"    Generates a dynamic fault tree\n"
"\n"
"    -t\n"
"\n"
"    Generates fault tree\n"
"\n"
"    -Q\n"
"\n"
"    Disables construction of ordered FT and computes probability\n"
"    only for the top level event (to speedup computation)\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability\n"
"\n"
"    -S\n"
"\n"
"    Computes probability also in symbolic form (python and\n"
"    octave/matlab)\n"
"\n"
"    -V\n"
"\n"
"    Does not perform virtual unrolling\n"
"\n"
"    -B\n"
"\n"
"    Adds blocking clauses at all time steps\n"
"\n"
"    -c\n"
"\n"
"    Performs completeness check\n"
"\n"
"    -x prefix\n"
"\n"
"    Prefixes generated file names with \"prefix\"\n"
"\n"
"    -n index\n"
"\n"
"    Use given INVARSPEC or LTLSPEC property as !TLE\n"
"\n"
"    -P name\n"
"\n"
"    Use given INVARSPEC or LTLSPEC property (from name) as !TLE\n"
"\n"
"    next-expr\n"
"\n"
"    Use given expression as TLE\n"
"\n"
"",
"compute_fault_tree_msat_bmc", "COMMAND= compute_fault_tree_msat_bmc   \n"
"usage: compute_fault_tree_msat_bmc [-h] [-m|\n"
"-o file] [-k length] [-l loopback] [-T mcs] [-N NR_FAIL] [-t [-Q]] [-e]\n"
"[-p [-S]] [-M] [-x prefix_string] [-n index | -P name | next-expr]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes output through the program specified by the \"PAGER\"\n"
"    shell variable if defined, else through UNIX \"more\"\n"
"\n"
"    -o file\n"
"\n"
"    Writes the generated output to \"file\"\n"
"\n"
"    -k length\n"
"\n"
"    set problem length to length\n"
"\n"
"    -l loop\n"
"\n"
"    set loopback value to loop\n"
"\n"
"    -T mcs\n"
"\n"
"    limit number of cut sets to be computed to mcs\n"
"\n"
"    -N NR_FAIL\n"
"\n"
"    limit number of possible failures to NR_FAIL\n"
"\n"
"    -t\n"
"\n"
"    Generates fault tree\n"
"\n"
"    -Q\n"
"\n"
"    Disables construction of ordered FT and computes probability\n"
"    only for the top level event (to speedup computation)\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability\n"
"\n"
"    -S\n"
"\n"
"    Computes probability also in symbolic form (python and\n"
"    octave/matlab)\n"
"\n"
"    -M\n"
"\n"
"    Set the generation of monotonic cutsets to FALSE\n"
"\n"
"    -x prefix\n"
"\n"
"    Prefixes generated file names with \"prefix\"\n"
"\n"
"    -n index\n"
"\n"
"    Use given INVARSPEC or LTLSPEC property as !TLE\n"
"\n"
"    -P name\n"
"\n"
"    Use given INVARSPEC or LTLSPEC property (from name) as !TLE\n"
"\n"
"    next-expr\n"
"\n"
"    Use given expression as TLE\n"
"\n"
"",
"compute_fmea_table", "COMMAND= compute_fmea_table   \n"
"usage: compute_fmea_table [-h] [-m| -o file]\n"
"[-c] [-d] [-N NR_FAIL] [-t] [-e] [-x prefix_string] [-n props | -P\n"
"props | next-expr ... next-expr]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes output through the program specified by the \"PAGER\"\n"
"    shell variable if defined, else through UNIX \"more\"\n"
"\n"
"    -o file\n"
"\n"
"    Writes the generated output to \"file\"\n"
"\n"
"    -c\n"
"\n"
"    Generates a compact FMEA table\n"
"\n"
"    -d\n"
"\n"
"    Generates a dynamic FMEA table\n"
"\n"
"    -N NR_FAIL\n"
"\n"
"    set number of failures to NR_FAIL\n"
"\n"
"    -t\n"
"\n"
"    Generates fmea table\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -x prefix\n"
"\n"
"    Prefixes generated file names with \"prefix\"\n"
"\n"
"    -n props\n"
"\n"
"    A subset of INVARSPEC or LTLSPEC properties whose indices are\n"
"    given as argument. Indices are separated by comma ',' or colon\n"
"    ':'. Ranges are allowed where lower and upper bounds are\n"
"    separated by dash '-'. For example \"1:3-6:8\" for indices\n"
"    1,3,4,5,6,8\n"
"\n"
"    -P props\n"
"\n"
"    A subset of INVARSPEC or LTLSPEC properties whose names are given\n"
"    as argument. Names are separated by comma ',' or colon ':'.\n"
"\n"
"",
"compute_fmea_table_msat_bmc", "COMMAND= compute_fmea_table_msat_bmc   \n"
"usage: compute_fmea_table_msat_bmc [-h] [-m|\n"
"-o file] [-k length] [-l loopback] [-c] [-N NR_FAIL] [-t] [-e] [-x\n"
"prefix_string] [-n props | -P props | next-expr ... next-expr]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes output through the program specified by the \"PAGER\"\n"
"    shell variable if defined, else through UNIX \"more\"\n"
"\n"
"    -o file\n"
"\n"
"    Writes the generated output to \"file\"\n"
"\n"
"    -k\n"
"\n"
"    Set problem length\n"
"\n"
"    -l\n"
"\n"
"    Set loopback value\n"
"\n"
"    -c\n"
"\n"
"    Generates a compact FMEA table\n"
"\n"
"    -N NR_FAIL\n"
"\n"
"    set number of failures to NR_FAIL\n"
"\n"
"    -t\n"
"\n"
"    Generates fmea table\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -x prefix\n"
"\n"
"    Prefixes generated file names with \"prefix\"\n"
"\n"
"    -n props\n"
"\n"
"    A subset of INVARSPEC or LTLSPEC properties whose indices are\n"
"    given as argument. Indices are separated by comma ',' or colon\n"
"    ':'. Ranges are allowed where lower and upper bounds are\n"
"    separated by dash '-'. For example \"1:3-6:8\" for indices\n"
"    1,3,4,5,6,8\n"
"\n"
"    -P props\n"
"\n"
"    A subset of INVARSPEC or LTLSPEC properties whose names are given\n"
"    as argument. Names are separated by comma ',' or colon ':'.\n"
"\n"
"",
"compute_fmea_table_bmc_inc", "COMMAND= compute_fmea_table_bmc_inc   \n"
"usage: compute_fmea_table_bmc_inc [-h] [-m|\n"
"-o file] [-k length] [-l loopback] [-c] [-N NR_FAIL] [-t] [-e] [-x\n"
"prefix_string] [-n props | -P props | next-expr ... next-expr]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes output through the program specified by the \"PAGER\"\n"
"    shell variable if defined, else through UNIX \"more\"\n"
"\n"
"    -o file\n"
"\n"
"    Writes the generated output to \"file\"\n"
"\n"
"    -k\n"
"\n"
"    Set problem length\n"
"\n"
"    -l\n"
"\n"
"    Set loopback value\n"
"\n"
"    -c\n"
"\n"
"    Generates a compact FMEA table\n"
"\n"
"    -N NR_FAIL\n"
"\n"
"    set number of failures to NR_FAIL\n"
"\n"
"    -t\n"
"\n"
"    Generates fmea table\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -x prefix\n"
"\n"
"    Prefixes generated file names with \"prefix\"\n"
"\n"
"    -n props\n"
"\n"
"    A subset of INVARSPEC or LTLSPEC properties whose indices are\n"
"    given as argument. Indices are separated by comma ',' or colon\n"
"    ':'. Ranges are allowed where lower and upper bounds are\n"
"    separated by dash '-'. For example \"1:3-6:8\" for indices\n"
"    1,3,4,5,6,8\n"
"\n"
"    -P props\n"
"\n"
"    A subset of INVARSPEC or LTLSPEC properties whose names are given\n"
"    as argument. Names are separated by comma ',' or colon ':'.\n"
"\n"
"",
"compute_fault_detection", "COMMAND= compute_fault_detection   \n"
"usage: compute_fault_detection [-m| -o file]\n"
"[-t] [-x prefix_string]] next-expr\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes output through the program specified by the PAGER shell\n"
"    variable if defined, else through UNIX more\n"
"\n"
"    -o\n"
"\n"
"    Writes the generated output to file\n"
"\n"
"    -t\n"
"\n"
"    Generates fault detection means\n"
"\n"
"    -x\n"
"\n"
"    Prefixes generated file names with prefix\n"
"\n"
"",
"compute_fault_detection_msat_bmc", "COMMAND= compute_fault_detection_msat_bmc   \n"
"usage: compute_fault_detection_msat_bmc [-d\n"
"float | -c string] [-m| -o file] [-k length] [-l loopback] [-t] [-x\n"
"prefix_string]] next-expr\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -d\n"
"\n"
"    Upper bound for detection delay\n"
"\n"
"    -c\n"
"\n"
"    Delay counter name (to be used instead of next-expr; requires -d\n"
"\n"
"    -m\n"
"\n"
"    Pipes output through the program specified by the PAGER shell\n"
"    variable if defined, else through UNIX more\n"
"\n"
"    -o\n"
"\n"
"    Writes the generated output to file\n"
"\n"
"    -k\n"
"\n"
"    Set problem length\n"
"\n"
"    -l\n"
"\n"
"    Set loopback value\n"
"\n"
"    -t\n"
"\n"
"    Generates fault detection means\n"
"\n"
"    -x\n"
"\n"
"    Prefixes generated file names with prefix\n"
"\n"
"",
"compute_fault_isolation", "COMMAND= compute_fault_isolation   \n"
"usage: compute_fault_isolation [-m| -o file]\n"
"[-t [-Q]] [-e] [-p [-S]] [-x prefix_string]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes output through the program specified by the PAGER shell\n"
"    variable if defined, else through UNIX more\n"
"\n"
"    -o\n"
"\n"
"    Writes the generated output to file\n"
"\n"
"    -t\n"
"\n"
"    Generates fault trees\n"
"\n"
"    -Q\n"
"\n"
"    Disables construction of ordered FT and computes probability\n"
"    only for the top level event (to speedup computation)\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability\n"
"\n"
"    -S\n"
"\n"
"    Computes probability also in symbolic form (python and\n"
"    octave/matlab)\n"
"\n"
"    -x\n"
"\n"
"    Prefixes generated file names with prefix\n"
"\n"
"",
"compute_fault_isolation_msat_bmc", "COMMAND= compute_fault_isolation_msat_bmc  \n"
"\n"
"usage: compute_fault_isolation_msat_bmc [-m| -o file] [-k length] [-l loopback] [-T mcs] [-t\n"
"[-Q]] [-e] [-p [-S]] [-x prefix_string]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes output through the program specified by the PAGER shell\n"
"    variable if defined, else through UNIX more\n"
"\n"
"    -o\n"
"\n"
"    Writes the generated output to file\n"
"\n"
"    -k\n"
"\n"
"    Set problem length\n"
"\n"
"    -l\n"
"\n"
"    Set loopback value\n"
"\n"
"    -T\n"
"\n"
"    Limit number of cut sets to be computed\n"
"\n"
"    -t\n"
"\n"
"    Generates fault trees\n"
"\n"
"    -Q\n"
"\n"
"    Disables construction of ordered FT and computes probability\n"
"    only for the top level event (to speedup computation)\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability\n"
"\n"
"    -S\n"
"\n"
"    Computes probability also in symbolic form (python and\n"
"    octave/matlab)\n"
"\n"
"    -x\n"
"\n"
"    Prefixes generated file names with prefix\n"
"\n"
"",
"generate_allocations", "COMMAND= generate_allocations   \n"
"usage: generate_allocations [-p opt_type] [-x\n"
"\"prefix_string\"] in_file_1 ... in_file_n\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -p\n"
"\n"
"    Computes optimal allocation (possible values are unrel_sr,\n"
"    unrel_br, cost, user_defined_expression\n"
"\n"
"    -x\n"
"\n"
"    Prefixes generated file names\n"
"\n"
"",
"compute_fault_tree", "COMMAND= compute_fault_tree   Computes minimal cut sets for a top\n"
"level event given as a simple expression or invar specification\n"
"\n"
"usage: compute_fault_tree [-f] [-h] [-m | -o output-file] [-t] [-e] [-p] [-x\n"
"prefix_string] [-P | -n | next-expr]\n"
" The output is the description of\n"
"failure states corresponding to minimal cut sets. Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the program to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX \"more\" command.\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -t\n"
"\n"
"    Computes a dynamic fault tree\n"
"\n"
"    -t\n"
"\n"
"    Computes a dynamic fault tree\n"
"\n"
"    -t\n"
"\n"
"    Generates fault tree corresponding to the minimal cut sets\n"
"\n"
"    -f\n"
"\n"
"    Bounds the number of faults (integer number)\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability.\n"
"\n"
"    -x prefix_string\n"
"\n"
"    Prefixes all generated file names by the string prefix_string.\n"
"\n"
"    -n index\n"
"\n"
"    Takes invar property (given its index) to be used as !TLE .\n"
"\n"
"    -P\n"
"\n"
"    Takes invar property (given its name) to be used as !TLE .\n"
"\n"
"",
"compute_fault_tree_bmc", "COMMAND= compute_fault_tree_bmc   Interactively computes\n"
"minimal cut sets for a top level event given as a simple expression or\n"
"invar specification \n"
"usage: compute_fault_tree_bmc [-h] [-m | -o output-file] [-t]\n"
"[-e] [-p] [-x prefix_string] [-P | -n | next-expr]\n"
" The output is the\n"
"description of failure states corresponding to minimal cut sets.\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the program to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX \"more\" command.\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -t\n"
"\n"
"    Generates fault tree corresponding to the minimal cut sets\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability.\n"
"\n"
"    -x prefix_string\n"
"\n"
"    Prefixes all generated file names by the string prefix_string.\n"
"\n"
"    -n index\n"
"\n"
"    Takes invar property (given its index) to be used as !TLE .\n"
"\n"
"    -P\n"
"\n"
"    Takes invar property (given its name) to be used as !TLE .\n"
"\n"
"",
"compute_fault_tree_bmc_inc", "COMMAND= compute_fault_tree_bmc_inc   Interactively computes\n"
"minimal cut sets for a top level event given as a simple expression or\n"
"invar specification \n"
"usage: compute_fault_tree_bmc_inc [-h] [-m | -o output-file] [-t]\n"
"[-e] [-p] [-x prefix_string] [-P | -n | next-expr]\n"
" The output is the\n"
"description of failure states corresponding to minimal cut sets.\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the program to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX \"more\" command.\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -t\n"
"\n"
"    Generates fault tree corresponding to the minimal cut sets\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability.\n"
"\n"
"    -x prefix_string\n"
"\n"
"    Prefixes all generated file names by the string prefix_string.\n"
"\n"
"    -n index\n"
"\n"
"    Takes invar property (given its index) to be used as !TLE .\n"
"\n"
"    -P\n"
"\n"
"    Takes invar property (given its name) to be used as !TLE .\n"
"\n"
"",
"compute_fault_tree_sbmc_inc", "COMMAND= compute_fault_tree_sbmc_inc   Interactively computes\n"
"minimal cut sets for a top level event given as a simple expression or\n"
"invar specification \n"
"usage: compute_fault_tree_sbmc_inc [-h] [-m | -o output-file] [-t]\n"
"[-e] [-p] [-x prefix_string] [-P | -n | next-expr]\n"
" The output is the\n"
"description of failure states corresponding to minimal cut sets.\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the program to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX \"more\" command.\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -t\n"
"\n"
"    Generates fault tree corresponding to the minimal cut sets\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability.\n"
"\n"
"    -x prefix_string\n"
"\n"
"    Prefixes all generated file names by the string prefix_string.\n"
"\n"
"    -V\n"
"\n"
"    Does not perform virtual unrolling.\n"
"\n"
"    -B\n"
"\n"
"    Adds blocking clauses at all time steps.\n"
"\n"
"    -c\n"
"\n"
"    Performs completeness check.\n"
"\n"
"    -n index\n"
"\n"
"    Takes invar property (given its index) to be used as !TLE .\n"
"\n"
"    -P\n"
"\n"
"        Takes invar property (given its name) to be used as !TLE . The\n"
"        following informations are displayed:\n"
"\n"
"        - bdd representing the min terms;\n"
"\n"
"        - failure states\n"
"\n"
"        - counterexample traces (optionally)\n"
"\n"
"    SideEffects [] SeeAlso [CommandComputePrimeImplicants]\n"
"    *****************************************************************************[EXTRACT_DOC_NOTE:\n"
"    * /] ",
"compute_fault_tree_bmc_bdd", "COMMAND= compute_fault_tree_bmc_bdd   Interactively computes\n"
"minimal cut sets for a top level event given as a simple expression or\n"
"invar specification \n"
"usage: compute_fault_tree_bmc_bdd [-h] [-m | -o output-file] [-t]\n"
"[-e] [-p] [-x prefix_string] [-P | -n | next-expr]\n"
" The output is the\n"
"description of failure states corresponding to minimal cut sets.\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the program to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX \"more\" command.\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -t\n"
"\n"
"    Generates fault tree corresponding to the minimal cut sets\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability.\n"
"\n"
"    -x prefix_string\n"
"\n"
"    Prefixes all generated file names by the string prefix_string.\n"
"\n"
"    -n index\n"
"\n"
"    Takes invar property (given its index) to be used as !TLE .\n"
"\n"
"    -P\n"
"\n"
"    Takes invar property (given its name) to be used as !TLE .\n"
"\n"
"",
"compute_fault_tree_bmc_inc", "COMMAND= compute_fault_tree_bmc_inc   Interactively computes\n"
"minimal cut sets for a top level event given as a simple expression or\n"
"invar specification \n"
"usage: compute_fault_tree_bmc_inc [-h] [-m | -o output-file] [-t]\n"
"[-e] [-p] [-x prefix_string] [-P | -n | next-expr]\n"
" The output is the\n"
"description of failure states corresponding to minimal cut sets.\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the program to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX \"more\" command.\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -t\n"
"\n"
"    Generates fault tree corresponding to the minimal cut sets\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability.\n"
"\n"
"    -x prefix_string\n"
"\n"
"    Prefixes all generated file names by the string prefix_string.\n"
"\n"
"    -n index\n"
"\n"
"    Takes invar property (given its index) to be used as !TLE .\n"
"\n"
"    -P\n"
"\n"
"    Takes invar property (given its name) to be used as !TLE .\n"
"\n"
"",
"compute_fault_tree_sbmc_inc", "COMMAND= compute_fault_tree_sbmc_inc   Interactively computes\n"
"minimal cut sets for a top level event given as a simple expression or\n"
"invar specification \n"
"usage: compute_fault_tree_sbmc_inc [-h] [-m | -o output-file] [-t]\n"
"[-e] [-p] [-x prefix_string] [-P | -n | next-expr]\n"
" The output is the\n"
"description of failure states corresponding to minimal cut sets.\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the program to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX \"more\" command.\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -t\n"
"\n"
"    Generates fault tree corresponding to the minimal cut sets\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability.\n"
"\n"
"    -x prefix_string\n"
"\n"
"    Prefixes all generated file names by the string prefix_string.\n"
"\n"
"    -V\n"
"\n"
"    Does not perform virtual unrolling.\n"
"\n"
"    -B\n"
"\n"
"    Adds blocking clauses at all time steps.\n"
"\n"
"    -c\n"
"\n"
"    Performs completeness check.\n"
"\n"
"    -n index\n"
"\n"
"    Takes invar property (given its index) to be used as !TLE .\n"
"\n"
"    -P\n"
"\n"
"        Takes invar property (given its name) to be used as !TLE . The\n"
"        following informations are displayed:\n"
"\n"
"        - bdd representing the min terms;\n"
"\n"
"        - failure states\n"
"\n"
"        - counterexample traces (optionally)\n"
"\n"
"    SideEffects [] SeeAlso [CommandComputePrimeImplicants]\n"
"    *****************************************************************************[EXTRACT_DOC_NOTE:\n"
"    * /] ",
"compute_fault_tree_msat_bmc", "COMMAND= compute_fault_tree_msat_bmc   Interactively computes\n"
"minimal cut sets for a top level event given as a simple expression or\n"
"invar specification \n"
"usage: compute_fault_tree_msat_bmc [-h] [-m | -o output-file] [-t]\n"
"[-e] [-p] [-x prefix_string] [-P | -n | next-expr]\n"
" The output is the\n"
"description of failure states corresponding to minimal cut sets.\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the program to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX \"more\" command.\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -t\n"
"\n"
"    Generates fault tree corresponding to the minimal cut sets\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability.\n"
"\n"
"    -x prefix_string\n"
"\n"
"    Prefixes all generated file names by the string prefix_string.\n"
"\n"
"    -n index\n"
"\n"
"    Takes invar property (given its index) to be used as !TLE .\n"
"\n"
"    -P\n"
"\n"
"    Takes invar property (given its name) to be used as !TLE .\n"
"\n"
"",
"compute_fmea_table", "COMMAND= compute_fmea_table   Computes an FMEA table\n"
"\n"
"usage: compute_fmea_table [-h] [-m | -o output-file] [-t] [-N NR_FAIL] [-x\n"
"prefix_string] \n"
" The output is the description of failure states\n"
"corresponding to minimal cut sets. Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the program to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX \"more\" command.\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -t\n"
"\n"
"    Generates the output fmea table\n"
"\n"
"    -c\n"
"\n"
"    Computes a compact fmea table\n"
"\n"
"    -d\n"
"\n"
"    Computes a dynamic fmea table\n"
"\n"
"    -N NR_FAIL\n"
"\n"
"    Sets the number of failures\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -x prefix_string\n"
"\n"
"    Prefixes all generated file name by the string prefix_string.\n"
"\n"
"",
"compute_fmea_table_msat_bmc", "COMMAND= compute_fmea_table_msat_bmc   Computes an FMEA table\n"
"\n"
"usage: compute_fmea_table_msat_bmc [-h] [-m | -o output-file] [-t] [-k length] [-l\n"
"loopback] [-N NR_FAIL] [-x prefix_string] \n"
" The output is the\n"
"description of failure states corresponding to minimal cut sets.\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the program to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX \"more\" command.\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -t\n"
"\n"
"    Generates the output fmea table\n"
"\n"
"    -c\n"
"\n"
"    Computes a compact fmea table\n"
"\n"
"    -d\n"
"\n"
"    Sets the number of failures\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -x prefix_string\n"
"\n"
"    Prefixes all generated file name by the string prefix_string.\n"
"\n"
"",
"compute_fmea_table_bmc_inc", "COMMAND= compute_fmea_table_bmc_inc   Computes an FMEA table\n"
"\n"
"usage: compute_fmea_table_bmc_inc [-h] [-m | -o output-file] [-t] [-k length] [-l\n"
"loopback] [-N NR_FAIL] [-x prefix_string] \n"
" The output is the\n"
"description of failure states corresponding to minimal cut sets.\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the program to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX \"more\" command.\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -t\n"
"\n"
"    Generates the output fmea table\n"
"\n"
"    -c\n"
"\n"
"    Computes a compact fmea table\n"
"\n"
"    -d\n"
"\n"
"    Sets the number of failures\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -x prefix_string\n"
"\n"
"    Prefixes all generated file name by the string prefix_string.\n"
"\n"
"",
"compute_fault_detection", "COMMAND= compute_fault_detection   Computes list of detection\n"
"means for a fault (fault detection \n"
"usage: compute_fault_detection [-h] [-m | -o\n"
"output-file] [-t] [-x prefix_string] \n"
" The output is the list of\n"
"detection means. Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the program to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX \"more\" command.\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -t\n"
"\n"
"    Generates the output list of fault detection means\n"
"\n"
"    -x prefix_string\n"
"\n"
"    Prefixes all generated file name by the string prefix_string.\n"
"\n"
"",
"generate_allocations", "COMMAND= generate_allocations   Generates allocations and\n"
"computes optimal one \n"
"usage: generate_allocations [-h] [-x prefix_string]\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -x prefix_string\n"
"\n"
"    Prefixes all generated file names by the string prefix_string.\n"
"\n"
"",
"compute_fault_tree_param", "COMMAND= compute_fault_tree_param   Computes minimal cut sets for\n"
"a top level event given as a simple expression or invar specification,\n"
"using parameter synthesis \n"
"usage: compute_fault_tree_param [-f] [-h] [-m | -o\n"
"output-file] [-t] [-e] [-p] [-x prefix_string] [-P | -n |\n"
"simple-expr]\n"
" The output is the description of failure states\n"
"corresponding to minimal cut sets. Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the program to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX \"more\" command.\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -t\n"
"\n"
"    Generates fault tree corresponding to the minimal cut sets\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability.\n"
"\n"
"    -x prefix_string\n"
"\n"
"    Prefixes all generated file names by the string prefix_string.\n"
"\n"
"    -n index\n"
"\n"
"    Takes invar property (given its index) to be used as !TLE .\n"
"\n"
"    -P\n"
"\n"
"    Takes invar property (given its name) to be used as !TLE .\n"
"\n"
"",
"compute_fault_tree_param", "COMMAND= compute_fault_tree_param   Computes minimal cut sets for\n"
"a top level event given as a simple expression or invar specification,\n"
"using parameter synthesis \n"
"usage: compute_fault_tree_param [-f] [-h] [-m | -o\n"
"output-file] [-t] [-e] [-p] [-x prefix_string] [-P | -n |\n"
"simple-expr]\n"
" The output is the description of failure states\n"
"corresponding to minimal cut sets. Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the program to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX \"more\" command.\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -t\n"
"\n"
"    Generates fault tree corresponding to the minimal cut sets\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability.\n"
"\n"
"    -x prefix_string\n"
"\n"
"    Prefixes all generated file names by the string prefix_string.\n"
"\n"
"    -n index\n"
"\n"
"    Takes invar property (given its index) to be used as !TLE .\n"
"\n"
"    -P\n"
"\n"
"    Takes ltl property (given its name) to be used as !TLE .\n"
"\n"
"",
"compute_prime_implicants_param", "COMMAND= compute_prime_implicants_param   Computes prime\n"
"implicants and generates a fault tree for a top level event given as a\n"
"simple expression, using parameter synthesis. \n"
"usage: compute_prime_implicants_param [-f]\n"
"[-h] [-m | -o output-file] [-t] [-e] [-p] [-x prefix_string] [-P | -n |\n"
"simple-expr]\n"
" The output is the description of failure states\n"
"corresponding to prime implicants. Command options:\n"
"\n"
"    \n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the program to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX \"more\" command.\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -t\n"
"\n"
"    Generates fault tree corresponding to the minimal cut sets\n"
"\n"
"    -e\n"
"\n"
"    Prints counterexample traces\n"
"\n"
"    -p\n"
"\n"
"    Computes probability.\n"
"\n"
"    -x prefix_string\n"
"\n"
"    Prefixes all generated file names by the string prefix_string.\n"
"\n"
"    -n index\n"
"\n"
"    Takes invar property (given its index) to be used as !TLE .\n"
"\n"
"    -P\n"
"\n"
"    Takes invar property (given its name) to be used as !TLE .\n"
"\n"
"",
"diag_optimize_observables_asl", "COMMAND= diag_optimize_observables_asl   Optimizes observables\n"
"wrt ASL spec. \n"
"usage: diag_optimize_observables_asl [-f |-a -c -d [-x ]] [-e ] [-h]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -f\n"
"\n"
"    file containing the list of ASL specifications\n"
"\n"
"    -a\n"
"\n"
"    ASL alarm type (exact/bounded/finite)\n"
"\n"
"    -c\n"
"\n"
"    diagnosis condition (LTL expression)\n"
"\n"
"    -d\n"
"\n"
"    delay bound (for exact and bounded delay)\n"
"\n"
"    -x\n"
"\n"
"    context (LTL expression)\n"
"\n"
"    -e\n"
"\n"
"    engine (bmc/bmc_ic3; default: bmc_ic3)\n"
"\n"
"",
"diag_load_asl_spec", "COMMAND= diag_load_asl_spec   Loads an ASL specification from\n"
"command line or from a file. \n"
"usage: diag_load_asl_spec [-f |-a -c -d [-x ]] [-p]\n"
"[-h]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -f\n"
"\n"
"    file containing the list of ASL specifications\n"
"\n"
"    -a\n"
"\n"
"    ASL alarm type (exact/bounded/finite)\n"
"\n"
"    -c\n"
"\n"
"    diagnosis condition (LTL expression)\n"
"\n"
"    -d int\n"
"\n"
"    delay bound (for exact and bounded delay)\n"
"\n"
"    -x\n"
"\n"
"    context (LTL expression)\n"
"\n"
"    -p\n"
"\n"
"    remove existing properties from property manager\n"
"\n"
"",
"simplify_twin_plant", "COMMAND= simplify_twin_plant   Calls the simplifier on the twin\n"
"plant. \n"
"usage: simplify_twin_plant [-h]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"",
"check_diagnosability", "COMMAND= check_diagnosability   Calls custom functions for\n"
"checking diagnosability. \n"
"usage: check_diagnosability [-E] [-d]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -E\n"
"\n"
"    reachability direction [forward/backward] (default is\n"
"    forward)\n"
"\n"
"    -d\n"
"\n"
"    diagnosability delay (default is 0; here only integers allowed)\n"
"\n"
"",
"compute_pareto_optimal_frontier", "COMMAND= compute_pareto_optimal_frontier   \n"
"usage: compute_pareto_optimal_frontier [-s ]\n"
"[-o ] [-e ] [-E ] [-k ] [-t] [-r] [-p ]\n"
" Command is deprecated.\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -s\n"
"\n"
"    search strategy [bottom_up/top_down/mixed] (default is\n"
"    bottom_up)\n"
"\n"
"    -o\n"
"\n"
"    optimization strategy [cost-delay/delay-cost] (default is\n"
"    cost-delay)\n"
"\n"
"    -e\n"
"\n"
"    verification engine\n"
"    [sat/bdd/ic3/sat_bdd/sat_ic3/reach_bdd] (default is sat)\n"
"\n"
"    -E\n"
"\n"
"    search direction of reach_bdd engine [forward/backward]\n"
"    (default is backward)\n"
"\n"
"    -k\n"
"\n"
"    default BMC depth\n"
"\n"
"    -t\n"
"\n"
"    dump diagnosability counterexamples\n"
"\n"
"    -r\n"
"\n"
"    disable use of preferred variables during SAT search\n"
"\n"
"    -p\n"
"\n"
"    SAT polarity mode [true/false] (default is true)\n"
"\n"
"",
"compute_optimal_observables", "COMMAND= compute_optimal_observables   Finds the optimal set of\n"
"observables for a given system. \n"
"usage: compute_optimal_observables -a -m -d [-s ] [-f] [-e ]\n"
"[-E ] [-k ] [-t] [-p ] [-r] [-h]\n"
" Command is deprecated.\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -a\n"
"\n"
"    number of activations [one/all]\n"
"\n"
"    -m\n"
"\n"
"    minimization requirement [minimal/minimum/none]\n"
"\n"
"    -d\n"
"\n"
"    diagnosis delay [non-negative (fixed delay) or -1 (any finite\n"
"    delay)]\n"
"\n"
"    -s\n"
"\n"
"    search strategy [bottom_up/top_down/mixed] (default is\n"
"    bottom_up)\n"
"\n"
"    -f\n"
"\n"
"    minimize on-the-fly\n"
"\n"
"    -e\n"
"\n"
"    verification engine\n"
"    [sat/bdd/ic3/sat_bdd/sat_ic3/reach_bdd] (default is sat)\n"
"\n"
"    -E\n"
"\n"
"    search direction of reach_bdd engine [forward/backward]\n"
"    (default is backward)\n"
"\n"
"    -k\n"
"\n"
"    default BMC depth (default is system setting)\n"
"\n"
"    -t\n"
"\n"
"    dump diagnosability counterexamples\n"
"\n"
"    -p\n"
"\n"
"    SAT polarity mode [true/false] (default is true)\n"
"\n"
"    -r\n"
"\n"
"    disable use of preferred variables during SAT search\n"
"\n"
"",
"load_diag_conditions", "COMMAND= load_diag_conditions   Loads the diagnosability\n"
"conditions from a file. \n"
"usage: load_diag_conditions -c [-d ] [-r] [-i] [-x] [-p]\n"
"[-s]\n"
" Command options:\n"
"\n"
"    \n"
"\n"
"    -c diag_conditions_file\n"
"\n"
"    Contains the diagnosability conditions and optional path\n"
"    restrictions.\n"
"\n"
"    -d\n"
"\n"
"    maximum diagnosis delay for all properties (default: 0;\n"
"    bounded-delay: -1)\n"
"\n"
"    -r\n"
"\n"
"    express fixed-delay conditions as invariant properties\n"
"    (default: false; uses counter based on steps or delta_t to\n"
"    measure the delay\n"
"\n"
"    -i\n"
"\n"
"    Path restrictions are ignored (default: false).\n"
"\n"
"    -x\n"
"\n"
"    Context on twins are ignored (default: false)\n"
"\n"
"    -p\n"
"\n"
"    Existing properties are removed from the property manager\n"
"    (default: false).\n"
"\n"
"    -s\n"
"\n"
"    Path restrictions are required to happen synchronously\n"
"    (default: false).\n"
"\n"
"OUTDATED DESCRIPTION Diagnosability conditions are loaded from the\n"
"specified file and expanded to the full CTL form they need to be\n"
"expressed in. A diagnosability condition is a simple expression that\n"
"describes a state. There always needs to be a couple of conditions,\n"
"following the principles of diagnosability verification. The file\n"
"format is as follows: CONDITION_L simple_expression CONDITION_R\n"
"simple_expression Optionally, also path restrictions can be\n"
"specified for a pair of conditions. Each restriction is also\n"
"formulated as a simple expression, describing a state the path must\n"
"cross before the conditions are considered to be relevant. The order\n"
"of specification in the file represents the chronological order of\n"
"the restriction appearance in the path. Restrictions immediately\n"
"follow the pair of conditions they apply to in the file, and have the\n"
"following format: RESTRICTION simple_expression RESTRICTION\n"
"simple_expression [...] Attention! Maximum line length is 1000\n"
"characters; longer lines result in aborting the process. Side Effects:\n"
"allocates memory, adds specs to flattened hierarchy",
"build_twin_plant", "COMMAND= build_twin_plant   Reads the list of observable variables\n"
"from a file and compiles the twin plant of the model. \n"
"usage: build_twin_plant -o\n"
"[-a] [-f] [-F] [-p] [-t ]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -o\n"
"\n"
"    file containing list of observable variables\n"
"\n"
"    -a\n"
"\n"
"    asynchronous twinplant composition [option is deprecated]\n"
"    (default: false)\n"
"\n"
"    -f\n"
"\n"
"    configure twin plant for synthesis [option is deprecated]\n"
"    (c1/c2 specification)\n"
"\n"
"    -A\n"
"\n"
"    configure twin plant for synthesis (ASL specification)\n"
"\n"
"    -p\n"
"\n"
"    use history variables to encode twin plant\n"
"\n"
"    -t\n"
"\n"
"    name of delta_t variable [option is deprecated] OUTDATED\n"
"    DESCRIPTION Reads the model (if necessary), reads the list of\n"
"    observable variables list (input and output variables),\n"
"    inserts a twin plant harness, flattens the model, and sets all\n"
"    observable (twin) variables to be invariant. If a flattened\n"
"    model is already present, the command is aborted. Side Effects: This\n"
"    function will alter the parsed_tree.",
"_test_diag_benchmarks", "COMMAND= _test_diag_benchmarks   \n"
"usage: _test_diag_benchmarks [-a min_obs] [-b\n"
"max_obs] [-v]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -a\n"
"\n"
"    minimum number of observables (default 20)\n"
"\n"
"    -b\n"
"\n"
"    maximum number of observables (default 40)\n"
"\n"
"    -v\n"
"\n"
"    observables have different cost\n"
"\n"
"",
"_test_diag_unittests", "COMMAND= _test_diag_unittests   \n"
"usage: _test_diag_unittests [-h]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"",
"synth_print_slim_stats", "COMMAND= synth_print_slim_stats   Prints statistical\n"
"information about the generated SLIM component \n"
"usage: synth_print_slim_stats -o\n"
"outputfile\n"
" Command options:\n"
"\n"
"    \n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"",
"synth_slim_fdir", "COMMAND= synth_slim_fdir   Prints the FDIR component in SLIM format\n"
"\n"
"usage: synth_slim_fdir [-h] [-p] [-o outputfile]\n"
" This command is temporary\n"
"and should be merged with synth_dump_fdir\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -o outputfile\n"
"\n"
"    -p {1,2}\n"
"\n"
"    1: Dumps the FDIR with FR interface but no implementation. 2:\n"
"    Dumps the FR implementation w/o FDIR nor interface\n"
"\n"
"",
"synth_dump_change", "COMMAND= synth_dump_change   tPrints the change XML file\n"
"\n"
"usage: synth_dump_change -o outputfile\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"",
"synth_parse_xml", "COMMAND= synth_parse_xml   Parse the input XML file and creates a\n"
"SynthSpecification \n"
"usage: synth_parse_xml \n"
" Command options:\n"
"\n"
"",
"synth_dump_result_xml", "COMMAND= synth_dump_result_xml   Dumps the XML file containing the\n"
"result of the synthesis process \n"
"usage: synth_dump_result_xml [-o ] [-u]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -u\n"
"\n"
"    Update file insteading of generating it\n"
"\n"
"",
"read_rat_file", "COMMAND= read_rat_file   Read an XML file generated by the RATSY\n"
"tool. \n"
"usage: read_rat_file  [-h] [-i file] \n"
" Reads an XML file generated\n"
"by the RATSY (formerly RAT) tool. If the -i option is not specified, it\n"
"reads from the file specified in the environment variable\n"
"input_file.\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -h\n"
"\n"
"    Prints the command usage.\n"
"\n"
"    -i file\n"
"\n"
"    Sets the environment variable input_file to file, and reads the\n"
"    model from the specified file.\n"
"\n"
"Side Effects: ",
"flatten_hierarchy", "COMMAND= flatten_hierarchy   Flattens the hierarchy of modules.\n"
"\n"
"usage: flatten_hierarchy  [-h] \n"
" This command is responsible of the\n"
"instantiation of modules. The instantiation is performed by\n"
"substituting the actual parameters for the formal parameters, and\n"
"then by prefixing the result via the instance name. Command options:\n"
"\n"
"    \n"
"\n"
"    -h\n"
"\n"
"    Prints the command usage.\n"
"\n"
"\n"
"\n"
"Side Effects: ",
"encode_variables", "COMMAND= encode_variables   Builds the BDD variables necessary to\n"
"compile the model into BDD. \n"
"usage: encode_variables  [-h] [-i order-file]\n"
"\n"
" Generates the boolean BDD variables and the ADD needed to encode\n"
"propositionally the (symbolic) variables declared in the model.\n"
"\n"
"The variables are created as default in the order in which they appear\n"
"in a depth first traversal of the hierarchy.\n"
"\n"
"The input order file can be partial and can contain variables not\n"
"declared in the model. Variables not declared in the model are simply\n"
"discarded. Variables declared in the model which are not listed in the\n"
"ordering input file will be created and appended at the end of the given\n"
"ordering list, according to the default ordering.\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -h\n"
"\n"
"    Prints the command usage.\n"
"\n"
"    -i order-file\n"
"\n"
"    Sets the environment variable input_order_file to order-file,\n"
"    and reads the variable ordering to be used from file order-file.\n"
"    This can be combined with the write_order command. The variable\n"
"    ordering is written to a file, which can be inspected and\n"
"    reordered by the user, and then read back in.\n"
"\n"
"Side Effects: ",
"build_model", "COMMAND= build_model   Compiles the flattened hierarchy into BDD.\n"
"\n"
"usage: build_model  [-h] [-f] [-m Method] \n"
" Compiles the flattened\n"
"hierarchy into BDD (initial states, invariants, and transition\n"
"relation) using the method specified in the environment variable\n"
"partition_method for building the transition relation.\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -h\n"
"\n"
"    Prints the command usage.\n"
"\n"
"    -m Method\n"
"\n"
"    Sets the environment variable partition_method to the value\n"
"    Method, and then builds the transition relation. Available\n"
"    methods are Monolithic, Threshold and Iwls95CP.\n"
"\n"
"    -f\n"
"\n"
"    Forces model construction. By default, only one partition\n"
"    method is allowed. This option allows to overcome this default,\n"
"    and to build the transition relation with different\n"
"    partitioning methods.\n"
"\n"
"Side Effects: ",
"build_flat_model", "COMMAND= build_flat_model   Compiles the flattened hierarchy into\n"
"SEXP. \n"
"usage: build_flat_model  [-h] \n"
" Compiles the flattened hierarchy into\n"
"SEXP (initial states, invariants, and transition relation).\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -h\n"
"\n"
"    Prints the command usage.\n"
"\n"
"Side Effects: ",
"build_boolean_model", "COMMAND= build_boolean_model   Compiles the flattened hierarchy\n"
"into boolean SEXP. \n"
"usage: build_boolean_model  [-h] [-f] \n"
" Compiles the\n"
"flattened hierarchy into boolean SEXP (initial states, invariants,\n"
"and transition relation).\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -h\n"
"\n"
"    Prints the command usage.\n"
"\n"
"    -f\n"
"\n"
"    Forces the boolean model construction.\n"
"\n"
"Side Effects: ",
"write_flat_model", "COMMAND= write_flat_model   Writes a flat model of a given SMV file.\n"
"\n"
"usage: write_flat_model  [-h] [-o filename] \n"
" A corresponding equivalent\n"
"model is printed out. If no file is specified, the file specified with\n"
"the environment variable output_flatten_model_file is used if any,\n"
"otherwise standard output is used as output.\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -h\n"
"\n"
"    Prints the command usage.\n"
"\n"
"    -o filename\n"
"\n"
"    Attempts to write the flat SMV model in filename.\n"
"\n"
"Side Effects: ",
"write_boolean_model", "COMMAND= write_boolean_model   Writes a flattened and booleanized\n"
"model of a given SMV file. \n"
"usage: write_boolean_model  [-h] [-o filename] \n"
"\n"
"Writes the currently loaded SMV model in the specified file, after\n"
"having flattened and booleanized it. If no file is specified, the file\n"
"specified via the environment variable output_boolean_model_file\n"
"is used if any, otherwise standard output is used.\n"
"\n"
"** New in 2.4.0 and later ** Scalar variables are dumped as DEFINEs\n"
"whose body is their boolean encoding. This allows the user to still\n"
"express and see parts of the generated boolean model in terms of the\n"
"original model's scalar variables names and values, and still\n"
"keeping the generated model purely boolean. Also, symbolic\n"
"constants are dumped within a CONSTANTS statement to declare the\n"
"values of the original scalar variables' for future reading of the\n"
"generated file. Side Effects: ",
"check_property", "COMMAND= check_property   Checks one or more properties in the\n"
"current list of properties. \n"
"usage: check_property  [-h] [[-n number] |\n"
"[[-r (1|2)] [(-a | -A | -d | -D | -b | -l | -g)]]] \n"
" Checks the\n"
"specified properties taken from the property list. Either a number or\n"
"a (combination of) player and property type can be given.\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -h\n"
"\n"
"    Prints the command usage.\n"
"\n"
"    -n number\n"
"\n"
"    Checks the property with id number in the property list if it\n"
"    exists.\n"
"\n"
"    -r (1|2)\n"
"\n"
"    Checks the properties of player 1|2 not already checked.\n"
"\n"
"    -a/tt>\n"
"\n"
"    Checks all the REACHTARGET properties not already checked.\n"
"\n"
"    -A/tt>\n"
"\n"
"    Checks all the AVOIDTARGET properties not already checked.\n"
"\n"
"    -d/tt>\n"
"\n"
"    Checks all the REACHDEADLOCK properties not already checked.\n"
"\n"
"    -D/tt>\n"
"\n"
"    Checks all the AVOIDDEADLOCK properties not already checked.\n"
"\n"
"    -b/tt>\n"
"\n"
"    Checks all the BUCHIGAME properties not already checked.\n"
"\n"
"    -l/tt>\n"
"\n"
"    Checks all the LTLGAME properties not already checked.\n"
"\n"
"    -g/tt>\n"
"\n"
"    Checks all the GENREACTIVITY properties not already checked.\n"
"\n"
"Side Effects: ",
"show_property", "COMMAND= show_property   Shows one or more properties in the current\n"
"list of properties. \n"
"usage: show_property  [-h] [-s] [-m | -o\n"
"output-file] [[-n number] | [[-u | -t | -f] [-r (1|2)] [(-a |\n"
"-A | -d | -D | -b | -l | -g)]]] \n"
" Show the specified properties taken\n"
"from the property list. Either a number or a (combination of) status\n"
"and player and property type can be given.\n"
"\n"
"    For every property, the following informations are displayed:\n"
"\n"
"    - the identifier of the property (a progressive number);\n"
"\n"
"    - the property formula;\n"
"\n"
"    - the type (ReachTarget, AvoidTarget, etc.); the status of the\n"
"    formula (Unchecked, True, False);\n"
"\n"
"    - if the formula has been found to be false, the number of the\n"
"    corresponding counterexample trace.\n"
"\n"
"\n"
"\n"
"By default, all the properties currently stored in the list of\n"
"properties are shown. Specifying the suitable options, it is\n"
"possible to let the system show a restricted set of properties. It is\n"
"allowed to insert only one option per status and one option per type.\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -h\n"
"\n"
"    Prints the command usage.\n"
"\n"
"    -m\n"
"\n"
"    Pipes the output through the program specified by the PAGER shell\n"
"    variable if defined, else through the UNIX \"more\" command.\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to output-file<\tt>.\n"
"\n"
"    -s Prints the number of stored properties.\n"
"\n"
"    <\tt>.\n"
"\n"
"    -u\n"
"\n"
"    Prints only unchecked properties.\n"
"\n"
"    -t\n"
"\n"
"    Prints only those properties found to be true.\n"
"\n"
"    -f\n"
"\n"
"    Prints only those properties found to be false.\n"
"\n"
"    -n number\n"
"\n"
"    Prints the property with number number.\n"
"\n"
"    -r (1|2)\n"
"\n"
"    Prints only the properties of player 1|2.\n"
"\n"
"    -t/tt>\n"
"\n"
"    Prints only the REACHTARGET properties.\n"
"\n"
"    -T/tt>\n"
"\n"
"    Prints only the AVOIDTARGET properties.\n"
"\n"
"    -d/tt>\n"
"\n"
"    Prints only the REACHDEADLOCK properties.\n"
"\n"
"    -D/tt>\n"
"\n"
"    Prints only the AVOIDDEADLOCK properties.\n"
"\n"
"    -b/tt>\n"
"\n"
"    Prints only the BUCHIGAME properties.\n"
"\n"
"    -l/tt>\n"
"\n"
"    Prints only the LTLGAME properties.\n"
"\n"
"    -g/tt>\n"
"\n"
"    Prints only the GENREACTIVITY properties.\n"
"\n"
"Side Effects: ",
"print_usage", "COMMAND= print_usage   Prints processor and BDD statistics.\n"
"\n"
"usage: print_usage  [-h] \n"
" Prints a formatted dump of\n"
"processor-specific usage statistics, and BDD usage statistics. For\n"
"Berkeley Unix, this includes all of the information in the\n"
"getrusage() structure.\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -h\n"
"\n"
"    Prints the command usage.\n"
"\n"
"Side Effects: ",
"check_reach_target", "COMMAND= check_reach_target   \n"
"usage: check_reach_target  [-h] [-m | -o\n"
"output-file] [-n number] [-s] [-d] [-e] [-f\n"
"strategy-file] \n"
" Determines whether a REACHTARGET specification\n"
"is realizable. If yes, then a strategy for the protagonist may be\n"
"computed, a counterstrategy for the opponent otherwise.\n"
"\n"
"Option -n can be used for checking a particular formula in the property\n"
"database. Otherwise all REACHTARGET specifications in the database\n"
"are checked.\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -h\n"
"\n"
"    Prints the command usage.\n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the command to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX command \"more\".\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -n number\n"
"\n"
"    Checks the REACHTARGET specification with index number in the\n"
"    property database.\n"
"\n"
"    -s\n"
"\n"
"    Requests strategy printing (default to stdout).\n"
"\n"
"    -d\n"
"\n"
"    Requests strategy printing, generate a DAG (implies -s).\n"
"\n"
"    -e\n"
"\n"
"    Requests strategy printing, generate an easily readable output\n"
"    (implies -s).\n"
"\n"
"    -f strategy-file\n"
"\n"
"    Writes strategy to \"strategy-file\" (implies -s).\n"
"\n"
"\n"
"\n"
"Strategy printing can also be requested by setting the environment\n"
"variable game_print_strategy. Side Effects: ",
"check_avoid_target", "COMMAND= check_avoid_target   \n"
"usage: check_avoid_target  [-h] [-m | -o\n"
"output-file] [-n number] [-a algorithm] [-s] [-d] [-e]\n"
"[-f strategy-file] \n"
" Determines whether a AVOIDTARGET\n"
"specification is realizable. If yes, then a strategy for the\n"
"protagonist may be computed, a counterstrategy for the opponent\n"
"otherwise.\n"
"\n"
"Option -n can be used for checking a particular formula in the property\n"
"database. Otherwise all AVOIDTARGET specifications in the database\n"
"are checked.\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -h\n"
"\n"
"    Prints the command usage.\n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the command to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX command \"more\".\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -n number\n"
"\n"
"    Checks the AVOIDTARGET specification with index number in the\n"
"    property database.\n"
"\n"
"    -a algorithm\n"
"\n"
"    Uses algorithm algorithm. Available algorithms are: default,\n"
"    apprxA, apprxB, apprxC, and apprxD.\n"
"\n"
"    -s\n"
"\n"
"    Requests strategy printing (default to stdout).\n"
"\n"
"    -d\n"
"\n"
"    Requests strategy printing, generate a DAG (implies -s).\n"
"\n"
"    -e\n"
"\n"
"    Requests strategy printing, generate an easily readable output\n"
"    (implies -s).\n"
"\n"
"    -f strategy-file\n"
"\n"
"    Writes strategy to \"strategy-file\" (implies -s).\n"
"\n"
"\n"
"\n"
"Strategy printing can also be requested by setting the environment\n"
"variable game_print_strategy. Side Effects: ",
"check_reach_deadlock", "COMMAND= check_reach_deadlock   \n"
"usage: check_reach_deadlock  [-h] [-m | -o\n"
"output-file] [-n number] [-s] [-d] [-e] [-f\n"
"strategy-file] \n"
" Determines whether a REACHDEADLOCK\n"
"specification is realizable. If yes, then a strategy for the\n"
"protagonist may be computed, a counterstrategy for the opponent\n"
"otherwise.\n"
"\n"
"Option -n can be used for checking a particular formula in the property\n"
"database. Otherwise all REACHDEADLOCK specifications in the\n"
"database are checked.\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -h\n"
"\n"
"    Prints the command usage.\n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the command to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX command \"more\".\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -n number\n"
"\n"
"    Checks the REACHDEADLOCK specification with index number in the\n"
"    property database.\n"
"\n"
"    -s\n"
"\n"
"    Requests strategy printing (default to stdout).\n"
"\n"
"    -d\n"
"\n"
"    Requests strategy printing, generate a DAG (implies -s).\n"
"\n"
"    -e\n"
"\n"
"    Requests strategy printing, generate an easily readable output\n"
"    (implies -s).\n"
"\n"
"    -f strategy-file\n"
"\n"
"    Writes strategy to \"strategy-file\" (implies -s).\n"
"\n"
"\n"
"\n"
"Strategy printing can also be requested by setting the environment\n"
"variable game_print_strategy. Side Effects: ",
"check_avoid_deadlock", "COMMAND= check_avoid_deadlock   \n"
"usage: check_avoid_deadlock  [-h] [-m | -o\n"
"output-file] [-n number] [-s] [-d] [-e] [-f\n"
"strategy-file] \n"
" Determines whether a AVOIDDEADLOCK\n"
"specification is realizable. If yes, then a strategy for the\n"
"protagonist may be computed, a counterstrategy for the opponent\n"
"otherwise.\n"
"\n"
"Option -n can be used for checking a particular formula in the property\n"
"database. Otherwise all AVOIDDEADLOCK specifications in the\n"
"database are checked.\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -h\n"
"\n"
"    Prints the command usage.\n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the command to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX command \"more\".\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -n number\n"
"\n"
"    Checks the AVOIDDEADLOCK specification with index number in the\n"
"    property database.\n"
"\n"
"    -s\n"
"\n"
"    Requests strategy printing (default to stdout).\n"
"\n"
"    -d\n"
"\n"
"    Requests strategy printing, generate a DAG (implies -s).\n"
"\n"
"    -e\n"
"\n"
"    Requests strategy printing, generate an easily readable output\n"
"    (implies -s).\n"
"\n"
"    -f strategy-file\n"
"\n"
"    Writes strategy to \"strategy-file\" (implies -s).\n"
"\n"
"\n"
"\n"
"Strategy printing can also be requested by setting the environment\n"
"variable game_print_strategy. Side Effects: ",
"check_buchi_game", "COMMAND= check_buchi_game   \n"
"usage: check_buchi_game  [-h] [-m | -o\n"
"output-file] [-n number] [-s] [-d] [-e] [-f\n"
"strategy-file] \n"
" Determines whether a BUCHIGAME specification is\n"
"realizable. If yes, then a strategy for the protagonist may be\n"
"computed, a counterstrategy for the opponent otherwise.\n"
"\n"
"Option -n can be used for checking a particular formula in the property\n"
"database. Otherwise all BUCHIGAME specifications in the database\n"
"are checked.\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -h\n"
"\n"
"    Prints the command usage.\n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the command to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX command \"more\".\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -n number\n"
"\n"
"    Checks the BUCHIGAME specification with index number in the\n"
"    property database.\n"
"\n"
"    -s\n"
"\n"
"    Requests strategy printing (default to stdout).\n"
"\n"
"    -d\n"
"\n"
"    Requests strategy printing, generate a DAG (implies -s).\n"
"\n"
"    -e\n"
"\n"
"    Requests strategy printing, generate an easily readable output\n"
"    (implies -s).\n"
"\n"
"    -f strategy-file\n"
"\n"
"    Writes strategy to \"strategy-file\" (implies -s).\n"
"\n"
"\n"
"\n"
"Strategy printing can also be requested by setting the environment\n"
"variable game_print_strategy. Side Effects: ",
"check_gen_reactivity", "COMMAND= check_gen_reactivity   \n"
"usage: check_gen_reactivity  [-h] [-m | -o\n"
"output-file] [-n number] [-s] [-d] [-e] [-f\n"
"strategy-file] \n"
" Determines whether a GENREACTIVITY\n"
"specification is realizable. If yes, then a strategy for the\n"
"protagonist may be computed, a counterstrategy for the opponent\n"
"otherwise.\n"
"\n"
"Option -n can be used for checking a particular formula in the property\n"
"database. Otherwise all GENREACTIVITY specifications in the\n"
"database are checked.\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -h\n"
"\n"
"    Prints the command usage.\n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the command to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX command \"more\".\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -n number\n"
"\n"
"    Checks the GENREACTIVITY specification with index number in the\n"
"    property database.\n"
"\n"
"    -s\n"
"\n"
"    Requests strategy printing (default to stdout).\n"
"\n"
"    -d\n"
"\n"
"    Requests strategy printing, generate a DAG (implies -s).\n"
"\n"
"    -e\n"
"\n"
"    Requests strategy printing, generate an easily readable output\n"
"    (implies -s).\n"
"\n"
"    -f strategy-file\n"
"\n"
"    Writes strategy to \"strategy-file\" (implies -s).\n"
"\n"
"\n"
"\n"
"Strategy printing can also be requested by setting the environment\n"
"variable game_print_strategy. Side Effects: ",
"check_ltlgame_sf07", "COMMAND= check_ltlgame_sf07   Checks an LTLGAME specification\n"
"using the sf07 algorithm. \n"
"usage: check_ltlgame_sf07  [-h] [-m | -o\n"
"output-file] [-n number] [-s] [-d] [-e] [-f\n"
"strategy-file] [-k number] [-K number] [-w p|a|b|1|2] \n"
"\n"
"Determines whether an LTLGAME specification is realizable using the\n"
"sf07 algorithm. If yes, then a strategy for the protagonist may be\n"
"computed, a counterstrategy for the opponent otherwise.\n"
"\n"
"Option -n can be used for checking a particular formula in the property\n"
"database. Otherwise all LTLGAME specifications in the database are\n"
"checked.\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -h\n"
"\n"
"    Prints the command usage.\n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the command to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX command \"more\".\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -n number\n"
"\n"
"    Checks the LTLGAME specification with index number in the\n"
"    property database.\n"
"\n"
"    -s\n"
"\n"
"    Requests strategy printing (default to stdout).\n"
"\n"
"    -d\n"
"\n"
"    Requests strategy printing, generate a DAG (implies -s).\n"
"\n"
"    -e\n"
"\n"
"    Requests strategy printing, generate an easily readable output\n"
"    (implies -s).\n"
"\n"
"    -f strategy-file\n"
"\n"
"    Writes strategy to \"strategy-file\" (implies -s).\n"
"\n"
"    -k number\n"
"\n"
"    Start value for k (default 0).\n"
"\n"
"    -K number\n"
"\n"
"    End value for k (default 20).\n"
"\n"
"    -w p|a|b|1|2\n"
"\n"
"    Whom games are played for: (p)rotagonist, (a)ntagonist,\n"
"    (b)oth, player (1), or player (2). Default: b.\n"
"\n"
"\n"
"\n"
"Strategy printing can also be requested by setting the environment\n"
"variable game_print_strategy.\n"
"\n"
"    Further relevant environment variables are:\n"
"\n"
"    game_sf07_gba_wring_binary\n"
"\n"
"    Specifies the path to the LTL to Büchi translator.\n"
"\n"
"    game_sf07_gba_wring_has_cc\n"
"\n"
"    Specifies whether the LTL to Büchi translator has the option -cc\n"
"    to degeneralize the resulting automaton (this implementation\n"
"    requires simple automata).\n"
"\n"
"    game_sf07_gba_wring_input_dir\n"
"\n"
"    Directory to store input files to the LTL to Büchi translator.\n"
"\n"
"    game_sf07_gba_wring_input_templ\n"
"\n"
"    Template for filenames of input files to the LTL to Büchi\n"
"    translator.\n"
"\n"
"    game_sf07_gba_wring_input_keep\n"
"\n"
"    Whether input files to the LTL to Büchi translator should be kept\n"
"    or removed.\n"
"\n"
"    game_sf07_gba_wring_output_dir\n"
"\n"
"    Directory to store output files of the LTL to Büchi translator.\n"
"\n"
"    game_sf07_gba_wring_output_templ\n"
"\n"
"    Template for filenames of output files of the LTL to Büchi\n"
"    translator.\n"
"\n"
"    game_sf07_gba_wring_output_keep\n"
"\n"
"    Whether output files of the LTL to Büchi translator should be kept\n"
"    or removed.\n"
"\n"
"    game_sf07_strategy_printing_mode\n"
"\n"
"    Whether the monitor part of a winning stratey should be printed\n"
"    from its original sexp, from its BDD FSM, or from its BDD FSM after\n"
"    conjunction with the winning strategy in the subgame.\n"
"\n"
"\n"
"\n"
"Side Effects: ",
"extract_unrealizable_core", "COMMAND= extract_unrealizable_core   Extracts an (un)realizable\n"
"core. \n"
"usage: extract_unrealizable_core  [-h] [-m | -o output-file] [-n number]\n"
"[-a algorithm] [-c type] [-i] [-v] [-t] [-p] [-w\n"
"p|b|1|2] [-N number] \n"
" Extracts an (un)realizable core from a game\n"
"+ property (after checking realizability of the property, if\n"
"needed). For details see A. Cimatti, M. Roveri, V. Schuppan, A.\n"
"Tchaltsev: Diagnostic Information for Realizability. In:\n"
"VMCAI'08.\n"
"\n"
"The following two algorithms are available:\n"
"\n"
"actvars: an activation variable is introduced for each part of the\n"
"game that is to be considered for minimization. Then the\n"
"parameterized realizability problem is solved by a single call to the\n"
"realizability algorithm. The resulting set of (parameterized)\n"
"winning states is then used to extract the core.\n"
"\n"
"explicit: the parts of the game that are to be considered for\n"
"minimization are removed tentatively to see whether their removal\n"
"changes the outcome. If no, the removal is made permanent; if yes, the\n"
"removal is undone. I.o.w., trial and error.\n"
"\n"
"Note that the two algorithms differ in what they do when minimizing\n"
"both players. The following core types are available:\n"
"\n"
"core: This indicates which guarantees need to be kept to preserve\n"
"unrealizability (resp. which assumptions need to be kept to preserve\n"
"realizability). fix: This indicates which guarantees need to be\n"
"removed to obtain a realizable property (resp. which assumptions\n"
"need to be removed to obtain unrealizability). Only available with\n"
"algorithm actvars. Command options:\n"
"\n"
"    \n"
"\n"
"    -h\n"
"\n"
"    Prints the command usage.\n"
"\n"
"    -m\n"
"\n"
"    Pipes the output generated by the command to the program\n"
"    specified by the PAGER shell variable if defined, else through\n"
"    the UNIX command \"more\".\n"
"\n"
"    -o output-file\n"
"\n"
"    Writes the output generated by the command to the file\n"
"    output-file.\n"
"\n"
"    -n number\n"
"\n"
"    Extracts the core of the property with the given index number.\n"
"\n"
"    -a algorithm\n"
"\n"
"    Uses algorithm algorithm. Available algorithms are actvars and\n"
"    explicit (default explicit).\n"
"\n"
"    -c type\n"
"\n"
"    Core type type. Available types are core and fix (only for\n"
"    algorithm actvars) (default core).\n"
"\n"
"    -i\n"
"\n"
"    Doesn't minimize INIT constraints.\n"
"\n"
"    -v\n"
"\n"
"    Doesn't minimize INVAR constraints.\n"
"\n"
"    -t\n"
"\n"
"    Doesn't minimize TRANS constraints.\n"
"\n"
"    -p\n"
"\n"
"    Doesn't minimize the property.\n"
"\n"
"    -w w|b|1|2\n"
"\n"
"    Who is minimized: loser, both, player 1, or player 2. With\n"
"    algorithm actvars 1, 2, and b are available (default b. With\n"
"    algorithm explicit l and b are available (default l).\n"
"\n"
"    -N number\n"
"\n"
"    With algorithm actvars this specifies how many constraints are\n"
"    guarded by a single activation variable. 0 means no activation\n"
"    variables are introduced. Default: 1.\n"
"\n"
"\n"
"\n"
"Side Effects: ",
"fame_parse_xml", "COMMAND= fame_parse_xml   Parse the input XML file and creates a\n"
"SynthSpecification \n"
"usage: fame_parse_xml -q -r\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -q\n"
"\n"
"    Quiet printing\n"
"\n"
"    -r\n"
"\n"
"    Enable FR synthesis\n"
"\n"
"",
"fame_dump_slim", "COMMAND= fame_dump_slim   Dumps the SLIM model of the FDIR - FDIR -\n"
"FDIRConf - FR (if enabled) - FD (if enabled) \n"
"usage: fame_dump_slim [-o ] [-c ]\n"
"[-e ] [-s ] [-v] [-d] [-f] -r root_name\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -o filename\n"
"\n"
"    Redirects the output to file\n"
"\n"
"    -c filename\n"
"\n"
"    Writes slim changes to file\n"
"\n"
"    -e filename\n"
"\n"
"    Writes results to file\n"
"\n"
"    -s filename\n"
"\n"
"    Writes statistics to file\n"
"\n"
"    -v\n"
"\n"
"    Verbose printing\n"
"\n"
"    -d\n"
"\n"
"    Dumps the FD component\n"
"\n"
"    -f\n"
"\n"
"    Dumps the FR component\n"
"\n"
"    -r root_name\n"
"\n"
"    Sets the name of the root component\n"
"\n"
"",
"fame_dump_change", "COMMAND= fame_dump_change   Dumps the change XML file used by the\n"
"SLIM Combinator to combine the FDIR with the system\n"
"\n"
"usage: fame_dump_change [-o filename] -r root_name\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -o filename\n"
"\n"
"    Redirects the output to file\n"
"\n"
"    -r root_name\n"
"\n"
"    Sets the name of the root component\n"
"\n"
"",
"fame_dump_stats", "COMMAND= fame_dump_stats   Outputs statistics concerning the\n"
"generated FDIR \n"
"usage: fame_dump_stats -o output_file (optional)\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -o output_file\n"
"\n"
"    Redirects the output to file\n"
"\n"
"",
"fame_dump_result_xml", "COMMAND= fame_dump_result_xml   Outputs the FDIRSpec XML file with\n"
"result of the synthesis process \n"
"usage: fame_dump_result_xml [-h] [-o\n"
"output_filename] [-u]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -o output_filename\n"
"\n"
"    Redirects the output to file\n"
"\n"
"    -u\n"
"\n"
"    Update existing file\n"
"\n"
"",
"fame_FR", "COMMAND= fame_FR   \n"
"usage: fame_FR [-h] [-o output_filename] -i\n"
"init_expr -g goal_expr\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -o output_filename\n"
"\n"
"    Redirects the output to file\n"
"\n"
"    -i init-expr\n"
"\n"
"    Specifies init expression\n"
"\n"
"    -g goal-expr\n"
"\n"
"    Specifies goal expression\n"
"\n"
"",
"fame_test_fdirconf", "COMMAND= fame_test_fdirconf   \n"
"usage: fame_test_fdirconf [-h] [-v]\n"
"\n"
"\n"
"Command options:\n"
"\n"
"    \n"
"\n"
"    -v\n"
"\n"
"    Verbose printing\n"
"\n"
"",
"fame_test_sys2tfpg", "COMMAND= fame_test_sys2tfpg   \n"
"usage: fame_test_sys2tfpg [-h]\n"
"\n"
"\n"
"Command options:\n"
"\n"
""

};

/* this is computed by scripts/create_help.py -n */
const int xsap_command_number=91
;


/*---------------------------------------------------------------------------*/
/* Variable declarations                                                     */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* Macro declarations                                                        */
/*---------------------------------------------------------------------------*/


/**AutomaticStart*************************************************************/

/*---------------------------------------------------------------------------*/
/* Static function prototypes                                                */
/*---------------------------------------------------------------------------*/

/**AutomaticEnd***************************************************************/


/*---------------------------------------------------------------------------*/
/* Definition of exported functions                                          */
/*---------------------------------------------------------------------------*/

void xsap_cmd_help_init(NuSMVEnv_ptr env){
  /*Here initialize the xsap command help hash table, in fact it add xsap command */
  /*  help to the NuSMV hash table */
  int i;
  for (i=0; i<xsap_command_number; i++){
    cmd_help_add(env, xsap_command_help[i].command_name,
                 xsap_command_help[i].command_description);
  }
}

void xsap_cmd_help_deinit(NuSMVEnv_ptr env){
  int i;
  for (i=0; i<xsap_command_number; i++){
    cmd_help_remove(env, (char *) xsap_command_help[i].command_name);
  }
}

/*---------------------------------------------------------------------------*/
/* Definition of internal functions                                          */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* Definition of static functions                                            */
/*---------------------------------------------------------------------------*/
