
# %HEADER

################ USER EXCEPTIONS ########################

class UserException(Exception):
    """All user related exceptions have to be derived from this class"""
    def __init__(self, error_message):
        super(UserException, self).__init__(error_message)
        pass
    pass # end of class


class MistypeUserException(UserException):
    """An exception to deal with type errors in user specified expressions

     Attributes:
        file_name -- (string) the name of the file a type error is found
        line_number -- (int) the line number a type error is found
        error_message -- (string) a message explaining a type error
     """
    def __init__(self, file_name, line_number, error_message):

        msg = "Type error:%s line %d : Type Error : %s" %\
        ((file_name+",") if file_name else "", line_number, error_message)
        super(MistypeUserException, self).__init__(msg)
        pass
    pass # end of class

################ INTERNAL EXCEPTIONS ########################

class InternalException(Exception):
    """All tools related exception have to be derived from this class"""
    def __init__(self, error_message):
        super(InternalException, self).__init__(error_message)
        pass
    pass # end of class


class ProcessInternalException (InternalException):
    """This exception is raised when external tool behaves
    unexpectedly, i.e. dies, for example."""
    def __init__(self, title, details):
        super(ProcessInternalException, self).__init__(
            "Error in %s: %s" % (title, details))
        self.title = title
        self.description = details
        pass

    def get_title(self): return self.title
    def get_description(self): return self.description
    pass # end of class


class TraceException (ProcessInternalException):
    def __init__(self, details):
        super(TraceException, self).__init__("Xsap Trace", details)
        pass
    pass # end of class
