import sys

#####
##### subroutines
#####
#####
##### main function
#####
# parse argument
param = len(sys.argv)
if param != 2:
    print "syntax: generator.py <number>"
    sys.exit()
try:
    N = int(sys.argv[1])
except ValueError:
    print "syntax: generator.py <number>"
    sys.exit()
if N < 3:
    print "parameter must be an integer >= 3"
    sys.exit()

#open output file
filename = 'SC_TMG_' + str(N) + ".smv"
f=open(filename, "w")

# generate model
f.write("MODULE main\n")
f.write("  VAR\n")
f.write("    CN : Controller_Synth(SC.event_Gs, SC.event_CBs, SC.init_Gs, SC.init_CBs);\n")
f.write("    SC: System_Configuration(CN.cmd_Gs, CN.cmd_CBs);\n")
f.write("  INVARSPEC NAME R1_true := !(")
for i in range(1,N+1):
    f.write("SC.B%d.is_broken" % i)
    if i < N:
        f.write(" | ")
    else:
        f.write(");\n")
f.write("  INVARSPEC NAME R2_false := (")
for i in range(1,N+1):
    f.write("SC.B%d.is_powered" % i)
    if i < N:
        f.write(" & ")
    else:
        f.write(");\n")
f.write("  INVARSPEC NAME R2_true := ")
f.write("     ((CN.MN.faults_counter < 3) -> (")
for i in range(1,N+1):
    f.write("SC.B%d.is_powered" % i)
    if i < N:
        f.write(" & ")
    else:
        f.write("));\n")
f.write("  INVARSPEC NAME R4_true := ((")
for i in range(1,N+1):
    f.write("SC.G%d.is_off" % i)
    if i < N:
        f.write(" & ")
    else:
        f.write(") -> (")
for i in range(1,N+1):
    f.write("!SC.B%d.is_powered" % i)
    if i < N:
        f.write(" & ")
    else:
        f.write("));\n")
f.write("  INVARSPEC NAME R6_true := (CN.MN.faults_counter = 0) -> (")
for i in range(1,N+1):
    f.write("SC.G%d.is_off" % i)
    if i < N:
        f.write(" | ")
    else:
        f.write(");\n")
f.write("  INVARSPEC NAME Monitor_State_Consistency_true := ")
for i in range(1,N+1):
    f.write("(SC.G%d.state  = CN.MN.state_est_G%d) & " % (i, i))
for i in range(1,N+1):
    f.write("(SC.GB%d.state  = CN.MN.state_est_GB%d) & " % (i, i))
for i in range(1,N+1):
    f.write("(SC.BB%d.state  = CN.MN.state_est_BB%d)" % (i, i))
    if i < N:
        f.write(" & ")
    else:
        f.write(";\n")
f.write("  DEFINE init_conf := SC.G1.is_on & SC.G2.is_on & ")
for i in range(3,N+1):
    f.write("SC.G%d.is_off & " % i)
f.write("SC.GB1.is_closed & SC.GB2.is_closed & ")
for i in range(3,N+1):
    f.write("SC.GB%d.is_open & " % i)
f.write("SC.BB1.is_open & ")
for i in range(2,N):
    f.write("SC.BB%d.is_closed & " % i)
f.write("SC.BB%d.is_open;\n" % N)
f.write("  LTLSPEC NAME system_in_init_configuration_until_fault_true := (init_conf U (CN.MN.faults_counter > 0)) | G init_conf;\n")
f.write("  CTLSPEC NAME system_can_always_work_true := EG (CN.MN.faults_counter = 0);\n")
f.write("  CTLSPEC NAME system_can_always_fail_true := AG(CN.MN.faults_counter = 0 -> EX (CN.MN.faults_counter >0));\n")
for i in range(1,N+1):
    f.write("  CTLSPEC NAME G%d_can_always_fail_true := AG(CN.MN.mode_est_G%d = ok -> EX (CN.MN.mode_est_G%d = ko))\n" % (i, i, i))
for i in range(1,N+1):
    f.write("  CTLSPEC NAME GB%d_can_always_fail_true := AG(CN.MN.mode_est_GB%d = ok -> EX (CN.MN.mode_est_GB%d = ko_open) & EX (CN.MN.mode_est_GB%d = ko_closed))\n" % (i, i, i, i))
for i in range(1,N+1):
    f.write("  CTLSPEC NAME BB%d_can_always_fail_true := AG(CN.MN.mode_est_BB%d = ok -> EX (CN.MN.mode_est_BB%d = ko_open) & EX (CN.MN.mode_est_BB%d = ko_closed))\n" % (i, i, i, i))
f.write("\n")
##
f.write("MODULE System_Configuration(cmd_Gs, cmd_CBs)\n")
f.write("  DEFINE\n")
f.write("    init_G1 := on;\n")
f.write("    init_G2 := on;\n")
for i in range(3,N+1):
    f.write("    init_G%d := off;\n" % i)
f.write("    init_GB1 := closed;\n")
f.write("    init_GB2 := closed;\n")
for i in range(3,N+1):
    f.write("    init_GB%d := open;\n" % i)
f.write("    init_BB1 := open;\n")
for i in range(2,N):
    f.write("    init_BB%d := closed;\n" % i)
f.write("    init_BB%d := open;\n" % N)
f.write("  VAR\n")
for i in range(1,N+1):
    f.write("    G%d : Generator(cmd_Gs[index_G%d], init_G%d);\n" % (i, i, i))
for i in range(1,N+1):
    f.write("    GB%d : Switch(cmd_CBs[index_GB%d], init_GB%d);\n" % (i, i, i))
for i in range(1,N+1):
    f.write("    BB%d : Switch(cmd_CBs[index_BB%d], init_BB%d);\n" % (i, i, i))
for i in range(1,N+1):
    f.write("    B%d : Bus(" % i)
    for j in range(1,N+1):
        f.write("B%d_poweredby_G%d" % (i, j))
        if j < N:
            f.write(", ")
        else:
            f.write(");\n")
f.write("  DEFINE\n")
f.write("    init_Gs := [\n")
for i in range(1,N+1):
    f.write("      init_G%d" % i)
    if i < N:
        f.write(",\n")
    else:
        f.write("];\n")
f.write("    init_CBs := [\n")
for i in range(1,N+1):
    f.write("      init_GB%d,\n" % i)
for i in range(1,N+1):
    f.write("      init_BB%d" % i)
    if i < N:
        f.write(",\n")
    else:
        f.write("];\n")
f.write("  DEFINE\n")
for i in range(1,N+1):
    f.write("    index_GB%d := %d;\n" % (i, i-1))
for i in range(1,N+1):
    f.write("    index_BB%d := %d;\n" % (i, i+N-1))
for i in range(1,N+1):
    f.write("    index_G%d := %d;\n" % (i, i-1))
f.write("  DEFINE\n")
f.write("    event_Gs := [\n")
f.write("      fev_off_Gs,\n")
f.write("      nev_Gs];\n")
f.write("    event_CBs := [\n")
f.write("      fev_open_CBs,\n")
f.write("      fev_closed_CBs,\n")
f.write("      nev_CBs];\n")
f.write("  DEFINE\n")
f.write("    fev_open_CBs := [\n")
for i in range(1,N+1):
    f.write("      GB%d.fev_stuck_at_open,\n" % i)
for i in range(1,N+1):
    f.write("      BB%d.fev_stuck_at_open" % i)
    if i < N:
        f.write(",\n")
    else:
        f.write("];\n")
f.write("    fev_closed_CBs := [\n")
for i in range(1,N+1):
    f.write("      GB%d.fev_stuck_at_closed,\n" % i)
for i in range(1,N+1):
    f.write("      BB%d.fev_stuck_at_closed" % i)
    if i < N:
        f.write(",\n")
    else:
        f.write("];\n")
f.write("    nev_CBs := [\n")
for i in range(1,N+1):
    f.write("      GB%d.nev,\n" % i)
for i in range(1,N+1):
    f.write("      BB%d.nev" % i)
    if i < N:
        f.write(",\n")
    else:
        f.write("];\n")
f.write("  DEFINE\n")
f.write("    fev_off_Gs := [\n")
for i in range(1,N+1):
    f.write("      G%d.fev_stuck_at_off" % i)
    if i < N:
        f.write(",\n")
    else:
        f.write("];\n")
f.write("    nev_Gs := [\n")
for i in range(1,N+1):
    f.write("      G%d.nev" % i)
    if i < N:
        f.write(",\n")
    else:
        f.write("];\n")
for i in range(1,N+1):
    left = (i-1 if i>1 else N)
    right = (i+1 if i< N else 1)
    f.write("  DEFINE\n")
    for j in range(1,N+1):
        if i == j:
            f.write("    B%d_poweredby_G%d_U := (G%d.is_on) & (GB%d.is_closed);\n" % (i, i, i, i))
        else:
            f.write("    B%d_poweredby_G%d_L := " % (i, j))
            if j == left:
                f.write("B%d_poweredby_G%d_U & (BB%d.is_closed);\n" % (j, j, j))
            else:
                f.write("B%d_poweredby_G%d_L & (BB%d.is_closed);\n" % (left, j, left))
            f.write("    B%d_poweredby_G%d_R := " % (i, j))
            if j == right:
                f.write("B%d_poweredby_G%d_U & (BB%d.is_closed);\n" % (j, j, i))
            else:
                f.write("B%d_poweredby_G%d_R & (BB%d.is_closed);\n" % (right, j, i))
for i in range(1,N+1):
    f.write("  DEFINE\n")
    for j in range(1,N+1):
        if i == j:
            f.write("    B%d_poweredby_G%d := B%d_poweredby_G%d_U;\n" % (i, i, i, i))
        else:
            f.write("    B%d_poweredby_G%d := B%d_poweredby_G%d_R | B%d_poweredby_G%d_L;\n" % (i, j, i, j, i, j))
f.write("\n")
##
f.write("MODULE Generator(cmd, init_state)\n")
f.write("  VAR\n")
f.write("    state : {on, off};\n")
f.write("  IVAR\n")
f.write("    fev_stuck_at_off : boolean;\n")
f.write("    nev : boolean;\n")
f.write("  TRANS\n")
f.write("   nev = FALSE;\n")
f.write("  DEFINE\n")
f.write("    is_on := (state = on);\n")
f.write("  DEFINE\n")
f.write("    is_off := (state = off);\n")
f.write("  ASSIGN\n")
f.write("    next(state) :=\n")
f.write("      case\n")
f.write("        (cmd = cmd_on) : on;\n")
f.write("        (cmd = cmd_off) : off;\n")
f.write("        TRUE : state;\n")
f.write("      esac;\n")
f.write("  ASSIGN\n")
f.write("   init(state) := init_state;\n")
f.write("\n")
##
f.write("MODULE Switch(cmd, init_state)\n")
f.write("  VAR\n")
f.write("    state : {open, closed};\n")
f.write("  IVAR\n")
f.write("    fev_stuck_at_closed : boolean;\n")
f.write("    fev_stuck_at_open : boolean;\n")
f.write("    nev : boolean;\n")
f.write("  DEFINE\n")
f.write("    is_closed := (state = closed);\n")
f.write("  DEFINE\n")
f.write("    is_open := (state = open);\n")
f.write("  ASSIGN\n")
f.write("    next(state) :=\n")
f.write("      case\n")
f.write("        (cmd = cmd_open) : open;\n")
f.write("        (cmd = cmd_closed) : closed;\n")
f.write("        TRUE : state;\n")
f.write("      esac;\n")
f.write("  ASSIGN\n")
f.write("   init(state) := init_state;\n")
f.write("\n")
##
f.write("MODULE Bus(")
for i in range(1,N+1):
    f.write("in%d" % i)
    if i < N:
        f.write(", ")
    else:
        f.write(")\n")
f.write("  VAR\n")
f.write("    state : {working, broken};\n")
f.write("  DEFINE\n")
f.write("   is_broken := (state = broken);\n")
f.write("  DEFINE\n")
f.write("    is_powered := (state = working) & (count(")
for i in range(1,N+1):
    f.write("in%d" % i)
    if i < N:
        f.write(", ")
    else:
        f.write(") = 1);\n")
f.write("  ASSIGN\n")
f.write("    init(state) :=\n")
f.write("      case\n")
f.write("        count(")
for i in range(1,N+1):
    f.write("in%d" % i)
    if i < N:
        f.write(", ")
    else:
        f.write(") > 1 : broken;\n")
f.write("        TRUE : working;\n")
f.write("      esac;\n")
f.write("    next(state) :=\n")
f.write("      case\n")
f.write("        next(count(")
for i in range(1,N+1):
    f.write("in%d" % i)
    if i < N:
        f.write(", ")
    else:
        f.write(")) > 1 : broken;\n")
f.write("        TRUE : state;\n")
f.write("      esac;\n")
f.write("\n")
##
f.write("MODULE Controller_Synth(event_Gs, event_CBs, init_Gs, init_CBs)\n")
f.write("  VAR\n")
f.write("    MN : Monitor(event_Gs, event_CBs, cmd_Gs, cmd_CBs, init_Gs, init_CBs);\n")
f.write("  IVAR\n")
for i in range(1,N+1):
    f.write("    cmd_GB%d : {cmd_closed, cmd_open, nop};\n" % i)
for i in range(1,N+1):
    f.write("    cmd_BB%d : {cmd_closed, cmd_open, nop};\n" % i)
for i in range(1,N+1):
    f.write("    cmd_G%d : {cmd_on, cmd_off, nop};\n" % i)
f.write("  DEFINE\n")
f.write("    cmd_Gs := [\n")
for i in range(1,N+1):
    f.write("      cmd_G%d" % i)
    if i < N:
        f.write(",\n")
    else:
        f.write("];\n")
f.write("    cmd_CBs := [\n")
for i in range(1,N+1):
    f.write("      cmd_GB%d,\n" % i)
for i in range(1,N+1):
    f.write("      cmd_BB%d" % i)
    if i < N:
        f.write(",\n")
    else:
        f.write("];\n")
f.write("  TRANS\n")
for i in range(1,N+1):
    f.write("    ((next(MN.state_est_GB%d) = open) -> (cmd_G%d = cmd_off))" % (i, i))
    if i < N:
        f.write(" &\n")
    else:
        f.write(";\n")
f.write("  TRANS\n")
for i in range(1,N+1):
    f.write("    ((next(MN.mode_est_GB%d) = ok & next(MN.state_est_GB%d) = closed) -> (cmd_G%d = cmd_on))" % (i, i, i))
    if i < N:
        f.write(" &\n")
    else:
        f.write(";\n")
f.write("  TRANS\n")
for i in range(1,N+1):
    f.write("    (next(MN.mode_est_G%d) = ko -> cmd_G%d = cmd_off) &\n" % (i, i))
for i in range(1,N+1):
    f.write("    (next(MN.mode_est_GB%d) = ko_open -> cmd_GB%d = cmd_open) &\n" % (i, i))
for i in range(1,N+1):
    f.write("    (next(MN.mode_est_BB%d) = ko_open -> cmd_BB%d = cmd_open) &\n" % (i, i))
for i in range(1,N+1):
    f.write("    (next(MN.mode_est_GB%d) = ko_closed -> cmd_GB%d = cmd_closed) &\n" % (i, i))
for i in range(1,N+1):
    f.write("    (next(MN.mode_est_BB%d) = ko_closed -> cmd_BB%d = cmd_closed)" % (i, i))
    if i < N:
        f.write("&\n")
    else:
        f.write(";\n")
for i in range(1,N+1):
    left = (i-1 if i>1 else N)
    right = (i+1 if i< N else 1)
    f.write("  DEFINE\n")
    for j in range(1,N+1):
        if i == j:
            f.write("    B%d_poweredby_G%d_U := (next(MN.state_est_G%d) = on) & (next(MN.state_est_GB%d) = closed);\n" % (i, i, i, i))
        else:
            f.write("    B%d_poweredby_G%d_L := " % (i, j))
            if j == left:
                f.write("B%d_poweredby_G%d_U & (next(MN.state_est_BB%d) = closed);\n" % (j, j, j))
            else:
                f.write("B%d_poweredby_G%d_L & (next(MN.state_est_BB%d) = closed);\n" % (left, j, left))
            f.write("    B%d_poweredby_G%d_R := " % (i, j))
            if j == right:
                f.write("B%d_poweredby_G%d_U & (next(MN.state_est_BB%d) = closed);\n" % (j, j, i))
            else:
                f.write("B%d_poweredby_G%d_R & (next(MN.state_est_BB%d) = closed);\n" % (right, j, i))
for i in range(1,N+1):
    f.write("  DEFINE\n")
    for j in range(1,N+1):
        if i == j:
            f.write("    B%d_poweredby_G%d := B%d_poweredby_G%d_U;\n" % (i, i, i, i))
        else:
            f.write("    B%d_poweredby_G%d := B%d_poweredby_G%d_R | B%d_poweredby_G%d_L;\n" % (i, j, i, j, i, j))
for i in range(1,N+1):
    f.write("  DEFINE\n")
    f.write("    B%d_is_working :=\n" % i)
    for j in range(1,N):
        f.write("      (B%d_poweredby_G%d -> (" % (i, j))
        for k in range(j+1,N+1):
            f.write("!B%d_poweredby_G%d" % (i, k))
            if k < N:
                f.write(" & ")
            else:
                f.write(")")
        if j < N-1:
            f.write(") &\n")
        else:
            f.write(");\n")
f.write("  TRANS\n    ")
for i in range(1,N+1):
    f.write("B%d_is_working" % i)
    if i < N:
        f.write(" & ")
    else:
        f.write(";\n")
## Powering options
f.write("  CONSTANTS\n")
for i in range(1,N+1):
    f.write("    G%d_U,\n" % i)
    f.write("    G%d_L,\n" % i)
    f.write("    G%d_R,\n" % i)
f.write("    unpowered;\n")
## stage i: Bi priorities
for stage in range(1,N+1):
    ## can_do definitions
    for bus in range(stage,N+1):
        left = (bus-1 if bus>1 else N)
        right = (bus+1 if bus< N else 1)
        f.write("  DEFINE\n")
        for gen in range(1,N+1):
            if bus == gen:
                if stage == 1:
                    f.write("    stage_%d_can_do_B%d_G%d_U := (next(MN.mode_est_G%d) = ok) & (next(MN.mode_est_GB%d) in {ok, ko_closed});\n" % (stage, bus, bus, bus, bus))
                else:
                    f.write("    stage_%d_can_do_B%d_G%d_U := case\n" % (stage, bus, bus))
                    f.write("      stage_%d_do_B%d = G%d_U : TRUE;\n"% (stage-1, bus, bus))
                    f.write("      stage_%d_do_B%d = unpowered : (next(MN.mode_est_G%d) = ok) & (next(MN.mode_est_GB%d) in {ok, ko_closed});\n" % (stage-1, bus, bus, bus))
                    f.write("      TRUE : FALSE;\n")
                    f.write("    esac;\n")
            else:
                f.write("    stage_%d_can_do_B%d_G%d_L := " % (stage, bus, gen))
                if gen == left:
                    if gen < stage:
                        f.write("stage_%d_do_B%d = G%d_U & (next(MN.mode_est_BB%d) in {ok, ko_closed});\n" % (gen, gen, gen, gen))
                    else:
                        f.write("stage_%d_can_do_B%d_G%d_U & (next(MN.mode_est_BB%d) in {ok, ko_closed});\n" % (stage, gen, gen, gen))
                else:
                    if left < stage:
                        f.write("stage_%d_do_B%d = G%d_L & (next(MN.mode_est_BB%d) in {ok, ko_closed});\n" % (left, left, gen, left))
                    else:
                        f.write("stage_%d_can_do_B%d_G%d_L & (next(MN.mode_est_BB%d) in {ok, ko_closed});\n" % (stage, left, gen, left))
                f.write("    stage_%d_can_do_B%d_G%d_R := " % (stage, bus, gen))
                if gen == right:
                    if gen < stage:
                        f.write("stage_%d_do_B%d = G%d_U & (next(MN.mode_est_BB%d) in {ok, ko_closed});\n" % (gen, gen, gen, bus))
                    else:
                        f.write("stage_%d_can_do_B%d_G%d_U & (next(MN.mode_est_BB%d) in {ok, ko_closed});\n" % (stage, gen, gen, bus))
                else:
                    if right < stage:
                        f.write("stage_%d_do_B%d = G%d_R & (next(MN.mode_est_BB%d) in {ok, ko_closed});\n" % (right, right, gen, bus))
                    else:
                        f.write("stage_%d_can_do_B%d_G%d_R & (next(MN.mode_est_BB%d) in {ok, ko_closed});\n" % (stage, right, gen, bus))
    # do definitions
    for bus in range(stage,N+1):
        f.write("  DEFINE\n")
        f.write("    stage_%d_do_B%d := case\n" %  (stage, bus))
        ## do definitions: first entry (bus = stage)
        if bus == stage:
            if stage > 1:
                f.write("      stage_%d_do_B%d != unpowered : stage_%d_do_B%d;\n" % (stage-1, bus, stage-1, bus))
            if stage <= 2:
                f.write("      stage_%d_can_do_B%d_G%d_U : G%d_U;\n" % (stage, stage, stage, stage))
                for gen in range(1,N+1):
                    if bus < gen:
                        dist_L = N-gen+bus;
                        dist_R = gen-bus
                        if dist_R <= dist_L:
                            f.write("      stage_%d_can_do_B%d_G%d_R : G%d_R;\n" % (stage, bus, gen, gen))
                            f.write("      stage_%d_can_do_B%d_G%d_L : G%d_L;\n" % (stage, bus, gen, gen))
                        else:
                            f.write("      stage_%d_can_do_B%d_G%d_L : G%d_L;\n" % (stage, bus, gen, gen))
                            f.write("      stage_%d_can_do_B%d_G%d_R : G%d_R;\n" % (stage, bus, gen, gen))
                    elif gen < bus:
                        dist_L = bus-gen
                        dist_R = N-bus+gen;
                        if dist_L <= dist_R:
                            f.write("      stage_%d_can_do_B%d_G%d_L : G%d_L;\n" % (stage, bus, gen, gen))
                            f.write("      stage_%d_can_do_B%d_G%d_R : G%d_R;\n" % (stage, bus, gen, gen))
                        else:
                            f.write("      stage_%d_can_do_B%d_G%d_R : G%d_R;\n" % (stage, bus, gen, gen))
                            f.write("      stage_%d_can_do_B%d_G%d_L : G%d_L;\n" % (stage, bus, gen, gen))
                f.write("      TRUE : unpowered;\n")
                f.write("    esac;\n")
            else:
                ## stage >= 3
                f.write("      stage_%d_can_do_B%d_G%d_L : G%d_L;\n" % (stage, bus, 2, 2))
                f.write("      stage_%d_can_do_B%d_G%d_R : G%d_R;\n" % (stage, bus, 2, 2))
                for gen in range(1,bus):
                    if gen != 2:
                        dist_L = bus-gen
                        dist_R = N-bus+gen;
                        if dist_L <= dist_R:
                            f.write("      stage_%d_can_do_B%d_G%d_L : G%d_L;\n" % (stage, bus, gen, gen))
                            f.write("      stage_%d_can_do_B%d_G%d_R : G%d_R;\n" % (stage, bus, gen, gen))
                        else:
                            f.write("      stage_%d_can_do_B%d_G%d_R : G%d_R;\n" % (stage, bus, gen, gen))
                            f.write("      stage_%d_can_do_B%d_G%d_L : G%d_L;\n" % (stage, bus, gen, gen))
                for gen in range(bus+1,N+1):
                    dist_L = N-gen+bus;
                    dist_R = gen-bus
                    if dist_R <= dist_L:
                        f.write("      stage_%d_can_do_B%d_G%d_R : G%d_R;\n" % (stage, bus, gen, gen))
                        f.write("      stage_%d_can_do_B%d_G%d_L : G%d_L;\n" % (stage, bus, gen, gen))
                    else:
                        f.write("      stage_%d_can_do_B%d_G%d_L : G%d_L;\n" % (stage, bus, gen, gen))
                        f.write("      stage_%d_can_do_B%d_G%d_R : G%d_R;\n" % (stage, bus, gen, gen))
                f.write("      stage_%d_can_do_B%d_G%d_U : G%d_U;\n" % (stage, bus, bus, bus))
                f.write("      TRUE : unpowered;\n")
                f.write("    esac;\n")
        else:
            ## do definitions: other entries (bus > stage)
            if stage > 1:
                f.write("      stage_%d_do_B%d != unpowered : stage_%d_do_B%d;\n" % (stage-1, bus, stage-1, bus))
            ## consequences of definitions for this stage and bus=stage: derived powering
            if stage == 1:
                f.write("      stage_%d_do_B%d in {G%d_L, G%d_R} : G%d_U;\n" % (stage, stage, bus, bus, bus))
            else:
                f.write("      stage_%d_do_B%d = G%d_R : G%d_U;\n" % (stage, stage, bus, bus))
            for gen in range(stage+1,bus):
                f.write("      stage_%d_do_B%d = G%d_L : G%d_L;\n" % (stage, stage, gen, gen))
            for gen in range(bus+1,N+1):
                f.write("      stage_%d_do_B%d = G%d_R : G%d_R;\n" % (stage, stage, gen, gen))
            if stage > 1:
                f.write("      stage_%d_do_B%d = G%d_R : G%d_R;\n" % (stage, stage, 1, 1))
            ## consequences of definitions for this stage and bus=stage: stuck_at_closed faults of BBs
            if stage == 1:
                f.write("      stage_%d_do_B%d = G%d_U & " % (stage, stage, stage))
                for bb in range(stage,bus):
                    f.write("(next(MN.mode_est_BB%d) = ko_closed)" % bb)
                    if bb < bus-1:
                        f.write(" & ")
                    else:
                        f.write(" : G%d_L;\n" % stage)
                f.write("      stage_%d_do_B%d = G%d_U & " % (stage, stage, stage))
                for bb in range(bus,N+1):
                    f.write("(next(MN.mode_est_BB%d) = ko_closed)" % bb)
                    if bb < N:
                        f.write(" & ")
                    else:
                        f.write(" : G%d_R;\n" % stage)
                for gen in range(2,bus):
                    f.write("      stage_%d_do_B%d = G%d_R & " % (stage, stage, gen))
                    for bb in range(gen,bus):
                        f.write("(next(MN.mode_est_BB%d) = ko_closed)" % bb)
                        if bb < bus-1:
                            f.write(" & ")
                        else:
                            f.write(" : G%d_L;\n" % gen)
                    f.write("      stage_%d_do_B%d = G%d_R & " % (stage, stage, gen))
                    for bb in range(bus,N+1):
                        f.write("(next(MN.mode_est_BB%d) = ko_closed)" % bb)
                        if bb < N:
                            f.write(" & ")
                        else:
                            f.write(" : G%d_R;\n" % gen)
                for gen in range(bus+1,N+1):
                    f.write("      stage_%d_do_B%d = G%d_L & " % (stage, stage, gen))
                    for bb in range(1,bus):
                        f.write("(next(MN.mode_est_BB%d) = ko_closed)" % bb)
                        if bb < bus-1:
                            f.write(" & ")
                        else:
                            f.write(" : G%d_L;\n" % gen)
                    f.write("      stage_%d_do_B%d = G%d_L & " % (stage, stage, gen))
                    for bb in range(bus,gen):
                        f.write("(next(MN.mode_est_BB%d) = ko_closed)" % bb)
                        if bb < gen-1:
                            f.write(" & ")
                        else:
                            f.write(" : G%d_R;\n" % gen)
                f.write("      TRUE : unpowered;\n")
                f.write("    esac;\n")
            else:
                ## stage > 1
                f.write("      stage_%d_do_B%d = G%d_U & " % (stage, stage, stage))
                for bb in range(stage,bus):
                    f.write("(next(MN.mode_est_BB%d) = ko_closed)" % bb)
                    if bb < bus-1:
                        f.write(" & ")
                    else:
                        f.write(" : G%d_L;\n" % stage)
                for gen in range(stage+1,bus):
                    f.write("      stage_%d_do_B%d = G%d_R & " % (stage, stage, gen))
                    for bb in range(gen,bus):
                        f.write("(next(MN.mode_est_BB%d) = ko_closed)" % bb)
                        if bb < bus-1:
                            f.write(" & ")
                        else:
                            f.write(" : G%d_L;\n" % gen)
                for gen in range(bus+1,N+1):
                    f.write("      stage_%d_do_B%d = G%d_L & " % (stage, stage, gen))
                    for bb in range(bus,gen):
                        f.write("(next(MN.mode_est_BB%d) = ko_closed)" % bb)
                        if bb < gen-1:
                            f.write(" & ")
                        else:
                            f.write(" : G%d_R;\n" % gen)
                for gen in range(1,stage):
                    f.write("      stage_%d_do_B%d = G%d_L & " % (stage, stage, gen))
                    for bb in range(stage,bus):
                        f.write("(next(MN.mode_est_BB%d) = ko_closed)" % bb)
                        if bb < bus-1:
                            f.write(" & ")
                        else:
                            f.write(" : G%d_L;\n" % gen)
                f.write("      TRUE : unpowered;\n")
                f.write("    esac;\n")
## Bi commands
for i in range(1,N+1):
    left = (i-1 if i>1 else N)
    right = (i+1 if i< N else 1)
    f.write("  DEFINE\n")
    for j in range(1,N+1):
        if i == j:
            f.write("    cmd_B%d_G%d_U := (cmd_G%d = cmd_on) & (cmd_GB%d = cmd_closed);\n" % (i, i, i, i))
        else:
            f.write("    cmd_B%d_G%d_L := " % (i, j))
            if j == left:
                f.write("cmd_B%d_G%d_U & (cmd_BB%d = cmd_closed);\n" % (j, j, j))
            else:
                f.write("cmd_B%d_G%d_L & (cmd_BB%d = cmd_closed);\n" % (left, j, left))
            f.write("    cmd_B%d_G%d_R := " % (i, j))
            if j == right:
                f.write("cmd_B%d_G%d_U & (cmd_BB%d = cmd_closed);\n" % (j, j, i))
            else:
                f.write("cmd_B%d_G%d_R & (cmd_BB%d = cmd_closed);\n" % (right, j, i))
## Bi preferences
for i in range(1,3):
    f.write("  TRANS\n")
    f.write("    case\n")
    f.write("      stage_%d_do_B%d = G%d_U : cmd_B%d_G%d_U;\n" % (i, i, i, i, i))
    for j in range(1,N+1):
        if i < j:
            dist_L = N-j+i;
            dist_R = j-i
            if dist_R <= dist_L:
                f.write("      stage_%d_do_B%d = G%d_R : cmd_B%d_G%d_R;\n" % (i, i, j, i, j))
                f.write("      stage_%d_do_B%d = G%d_L : cmd_B%d_G%d_L;\n" % (i, i, j, i, j))
            else:
                f.write("      stage_%d_do_B%d = G%d_L : cmd_B%d_G%d_L;\n" % (i, i, j, i, j))
                f.write("      stage_%d_do_B%d = G%d_R : cmd_B%d_G%d_R;\n" % (i, i, j, i, j))
        elif j < i:
            dist_L = i-j
            dist_R = N-i+j;
            if dist_L <= dist_R:
                f.write("      stage_%d_do_B%d = G%d_L : cmd_B%d_G%d_L;\n" % (i, i, j, i, j))
                f.write("      stage_%d_do_B%d = G%d_R : cmd_B%d_G%d_R;\n" % (i, i, j, i, j))
            else:
                f.write("      stage_%d_do_B%d = G%d_R : cmd_B%d_G%d_R;\n" % (i, i, j, i, j))
                f.write("      stage_%d_do_B%d = G%d_L : cmd_B%d_G%d_L;\n" % (i, i, j, i, j))
    f.write("      TRUE : TRUE;\n")
    f.write("    esac;\n")
for i in range(3,N+1):
    priority = 1;
    f.write("  TRANS\n")
    f.write("    case\n")
    f.write("      stage_%d_do_B%d = G%d_L : cmd_B%d_G%d_L;\n" % (i, i, 2, i, 2))
    f.write("      stage_%d_do_B%d = G%d_R : cmd_B%d_G%d_R;\n" % (i, i, 2, i, 2))
    for j in range(1,i):
        if j != 2:
            dist_L = i-j
            dist_R = N-i+j;
            if dist_L <= dist_R:
                f.write("      stage_%d_do_B%d = G%d_L : cmd_B%d_G%d_L;\n" % (i, i, j, i, j))
                f.write("      stage_%d_do_B%d = G%d_R : cmd_B%d_G%d_R;\n" % (i, i, j, i, j))
            else:
                f.write("      stage_%d_do_B%d = G%d_R : cmd_B%d_G%d_R;\n" % (i, i, j, i, j))
                f.write("      stage_%d_do_B%d = G%d_L : cmd_B%d_G%d_L;\n" % (i, i, j, i, j))
    for j in range(i+1,N+1):
        dist_L = N-j+i;
        dist_R = j-i
        if dist_R <= dist_L:
            f.write("      stage_%d_do_B%d = G%d_R : cmd_B%d_G%d_R;\n" % (i, i, j, i, j))
            f.write("      stage_%d_do_B%d = G%d_L : cmd_B%d_G%d_L;\n" % (i, i, j, i, j))
        else:
            f.write("      stage_%d_do_B%d = G%d_L : cmd_B%d_G%d_L;\n" % (i, i, j, i, j))
            f.write("      stage_%d_do_B%d = G%d_R : cmd_B%d_G%d_R;\n" % (i, i, j, i, j))
    f.write("      stage_%d_do_B%d = G%d_U : cmd_B%d_G%d_U;\n" % (i, i, i, i, i))
    f.write("      TRUE : TRUE;\n")
    f.write("    esac;\n")
f.write("\n")
##
f.write("MODULE Monitor(event_Gs, event_CBs, cmd_Gs, cmd_CBs, init_Gs, init_CBs)\n")
f.write("  VAR\n")
for i in range(1,N+1):
    f.write("    mode_est_GB%d : {ok, ko_open, ko_closed};\n" % i)
for i in range(1,N+1):
    f.write("    mode_est_BB%d : {ok, ko_open, ko_closed};\n" % i)
for i in range(1,N+1):
    f.write("    mode_est_G%d : {ok, ko};\n" % i)
f.write("  VAR\n")
for i in range(1,N+1):
    f.write("    state_est_GB%d : {closed, open};\n" % i)
for i in range(1,N+1):
    f.write("    state_est_BB%d : {closed, open};\n" % i)
for i in range(1,N+1):
    f.write("    state_est_G%d : {on, off};\n" % i)
f.write("  DEFINE\n")
f.write("    faults_counter := count(\n")
for i in range(1,N+1):
    f.write("      (mode_est_GB%d != ok),\n" % i)
for i in range(1,N+1):
    f.write("      (mode_est_BB%d != ok),\n" % i)
for i in range(1,N+1):
    f.write("      (mode_est_G%d != ok)" % i)
    if i < N:
        f.write(",\n")
    else:
        f.write(");\n")
f.write("  DEFINE\n")
f.write("    fev_open_CBs := event_CBs[0];\n")
f.write("    fev_closed_CBs := event_CBs[1];\n")
f.write("    nev_CBs := event_CBs[2];\n")
f.write("    fev_off_Gs := event_Gs[0];\n")
f.write("    nev_Gs := event_Gs[1];\n")
for i in range(1,N+1):
    f.write("    index_GB%d := %d;\n" % (i, i-1))
for i in range(1,N+1):
    f.write("    index_BB%d := %d;\n" % (i, i+N-1))
for i in range(1,N+1):
    f.write("    index_G%d := %d;\n" % (i, i-1))
for i in range(1,N+1):
    f.write("  ASSIGN\n")
    f.write("    init(mode_est_GB%d) := ok;\n" % i)
    f.write("    next(mode_est_GB%d) :=\n" % i)
    f.write("      case\n")
    f.write("        fev_open_CBs[index_GB%d] : ko_open;\n" % i)
    f.write("        fev_closed_CBs[index_GB%d] : ko_closed;\n" % i)
    f.write("        nev_CBs[index_GB%d] : ok;\n" % i)
    f.write("        TRUE : mode_est_GB%d;\n" % i)
    f.write("      esac;\n")
for i in range(1,N+1):
    f.write("  ASSIGN\n")
    f.write("    init(mode_est_BB%d) := ok;\n" % i)
    f.write("    next(mode_est_BB%d) :=\n" % i)
    f.write("      case\n")
    f.write("        fev_open_CBs[index_BB%d] : ko_open;\n" % i)
    f.write("        fev_closed_CBs[index_BB%d] : ko_closed;\n" % i)
    f.write("        nev_CBs[index_BB%d] : ok;\n" % i)
    f.write("        TRUE : mode_est_BB%d;\n" % i)
    f.write("      esac;\n")
for i in range(1,N+1):
    f.write("  ASSIGN\n")
    f.write("    init(mode_est_G%d) := ok;\n" % i)
    f.write("    next(mode_est_G%d) :=\n" % i)
    f.write("      case\n")
    f.write("        fev_off_Gs[index_G%d] : ko;\n" % i)
    f.write("        nev_Gs[index_G%d] : ok;\n" % i)
    f.write("        TRUE : mode_est_G%d;\n" % i)
    f.write("      esac;\n")
for i in range(1,N+1):
    f.write("  ASSIGN\n")
    f.write("    init(state_est_G%d) := init_Gs[index_G%d];\n" % (i, i))
    f.write("    next(state_est_G%d) :=\n" % i)
    f.write("      case\n")
    f.write("        fev_off_Gs[index_G%d] : off;\n" % i)
    f.write(" (next(mode_est_G%d) = ok) & (cmd_Gs[index_G%d] = cmd_on) : on;\n" % (i, i))
    f.write(" (next(mode_est_G%d) = ok) & (cmd_Gs[index_G%d] = cmd_off) : off;\n" % (i, i))
    f.write("        TRUE : state_est_G%d;\n" % i)
    f.write("      esac;\n")
for i in range(1,N+1):
    f.write("  ASSIGN\n")
    f.write("    init(state_est_GB%d) := init_CBs[index_GB%d];\n" % (i, i))
    f.write("    next(state_est_GB%d) :=\n" % i)
    f.write("      case\n")
    f.write("        fev_open_CBs[index_GB%d] : open;\n" % i)
    f.write("        fev_closed_CBs[index_GB%d] : closed;\n" % i)
    f.write(" (next(mode_est_GB%d) = ok) & (cmd_CBs[index_GB%d] = cmd_open) : open;\n" % (i, i))
    f.write(" (next(mode_est_GB%d) = ok) & (cmd_CBs[index_GB%d] = cmd_closed) : closed;\n" % (i, i))
    f.write("        TRUE : state_est_GB%d;\n" % i)
    f.write("      esac;\n")
for i in range(1,N+1):
    f.write("  ASSIGN\n")
    f.write("    init(state_est_BB%d) := init_CBs[index_BB%d];\n" % (i, i))
    f.write("    next(state_est_BB%d) :=\n" % i)
    f.write("      case\n")
    f.write("        fev_open_CBs[index_BB%d] : open;\n" % i)
    f.write("        fev_closed_CBs[index_BB%d] : closed;\n" % i)
    f.write(" (next(mode_est_BB%d) = ok) & (cmd_CBs[index_BB%d] = cmd_open) : open;\n" % (i, i))
    f.write(" (next(mode_est_BB%d) = ok) & (cmd_CBs[index_BB%d] = cmd_closed) : closed;\n" % (i, i))
    f.write("        TRUE : state_est_BB%d;\n" % i)
    f.write("      esac;\n")

# close output file
f.close()
