#!/usr/bin/env python3
from __future__ import print_function
import argparse
import sys

import _paths
_paths.setup_path()

from xsap.features.tfpg_smt.reasoner import TFPGReasoner
from xsap.data_structure.tfpg.tfpg import OPEN_INFINITY
from xsap.data_structure.tfpg.tfpg import Tfpg

def main():
    parser = argparse.ArgumentParser(description="Statistics information on TFPG")
    parser.add_argument('inputTFPG', metavar='inputTFPG',
                        type=str,
                        help='The TFPG model to use')
    parser.add_argument('--syntax-only', '-s',
                        action='store_true',
                        help='Perform only syntactical analysis')
    options = parser.parse_args()

    # Command line handling
    tfpg = Tfpg.parse_tfpg(options.inputTFPG)
    if tfpg is None:
        sys.exit(1)

    if not options.syntax_only:

        # Coherce the semantics to open-infinity.
        # The type of analysis here is not influenced by this.
        tfpg.set_infinity_semantics(OPEN_INFINITY)
        reasoner = TFPGReasoner(tfpg, include_health=False)
        inactive = reasoner.check_activability()

    print("All Nodes: %d" % len(tfpg.get_nodes_list()))
    print("FM Nodes: %d" % len(tfpg.get_fm_nodes()))
    print("AND Nodes: %d" % len(tfpg.get_and_nodes()))
    print("OR Nodes: %d" % len(tfpg.get_or_nodes()))
    print("")
    print("Monitored Nodes: %d" % len(tfpg.get_monitored_discrepancies()))
    print("Unmonitored Nodes: %d" % len(tfpg.get_unmonitored_discrepancies()))
    print("")
    print("Edges: %d" % len(tfpg.get_edges_list()))
    print("")
    print("Modes: %d" % len(tfpg.get_modes_list()))

    if not options.syntax_only:
        print("Activable Nodes: %d" % (len(tfpg.get_nodes_list()) - len(inactive)))
        print("Inactivable Nodes: %d" % len(inactive))
        print("")

if __name__ == "__main__":
    main()
