#!/usr/bin/env python3
import sys
import os
import re
import subprocess

import _paths


def handle_options():
    """Returns options values as map."""

    import argparse
    parser = argparse.ArgumentParser(description="")

    parser.add_argument("--smv-file", "-m",
                        dest="smv",
                        action="store",
                        default=None,
                        help="SMV file")

    parser.add_argument("--observables-file", "-o",
                        dest="obs_file",
                        action="store",
                        default=None,
                        help="File specifying observable variables")

    parser.add_argument("--asl-file", "-f",
                        dest="asl_file",
                        action="store",
                        default=None,
                        help="File specifying alarm specification")

    parser.add_argument("--stand-alone",
                        dest="dump_fd_only",
                        action="store_true",
                        default=False,
                        help="Output only the FD " +\
                        "(By default outputs the combination of FD and plant)")

    parser.add_argument("--out-file",
                        dest="target_smv",
                        action="store",
                        default=None,
                        help="Where to write the synthesized model")

    parser.add_argument("--verbosity-level", "-v",
                        dest="verbose_level",
                        action="store",
                        type=int,
                        default=0,
                        help="Sets the output verbosity level")

    parser.add_argument("--composition-semantics",
                        dest="composition_semantics",
                        action="store",
                        default="s",
                        help="Specify composition semantics:" +\
                        "(s)ynchronous or (a)synchronous")

    parser.add_argument("--no-dag",
                        dest="no_dag",
                        action="store_true",
                        default=False,
                        help="Do not use the DAG printing of the transition relation" +\
                        "(This should be used only on small FD)")

    args = parser.parse_args()

    # check arguments

    if args.smv is None or not os.path.exists(args.smv):
        parser.error("The SMV model was not specified.")

    if args.obs_file is None or not os.path.exists(args.obs_file):
        parser.error("The observables file was not specified.")

    if args.asl_file is None or not os.path.exists(args.obs_file):
        parser.error("The alarm specification file was not specified of does not exist.")

    if args.target_smv is None:
        parser.error("The output file was not specified.")

    return args

if __name__ == "__main__":
    _paths.setup_path()

    # setup logging
    import xsap.utils.misc.log
    xsap.utils.misc.log.init()
    args = handle_options()

    from xsap.utils.processes.xsap_process import XsapProcess
    xsap_process = XsapProcess(args.smv,
                               fail_obs_alm_fname=None,
                               ord_fname=None,
                               need_msat=False,
                               stdout=subprocess.PIPE,
                               stderr=subprocess.STDOUT,
                               dumpers=[],
                               verbose_level=args.verbose_level)
    res = xsap_process.synthesize_FD(args.asl_file, args.obs_file,
                                     args.target_smv, not args.dump_fd_only,
                                     args.composition_semantics,
                                     args.no_dag)
    xsap_process.kill_and_close_streams()

    if res:
        print("FD correctly synthesized")
    else:
        print("Error while synthesizing the FD")

    exit(0)
